import 'dart:convert';

import 'package:android_play_install_referrer/android_play_install_referrer.dart';
import 'package:flutter/material.dart';
import 'package:flyyfluttersdk/helpers/APIClient.dart';
import 'package:flyyfluttersdk/helpers/FlyyUtility.dart';
import 'package:flyyfluttersdk/helpers/SharedPrefTags.dart';
import 'package:flyyfluttersdk/model/FlyyEventResponse.dart';
import 'package:flyyfluttersdk/model/FlyyUserData.dart';
import 'package:shared_preferences/shared_preferences.dart';

import 'model/FlyyAppInfo.dart';

class Flyy {
  static String APP_PACKAGE, PARTNER_TOKEN, EXT_USER_ID, USER_NAME;
  static Color STATUSBAR_COLOR, TOOLBAR_COLOR;
  SharedPreferences sharedPreferences;
  FlyyUtility flyyUtility;
  APIClient apiClient;

  void init(String partnerKey) async {
    if (partnerKey == null || partnerKey.length == 0) {
      print('Flyy Init", "init failed, please provide all the data.');
      return;
    }

    flyyUtility = new FlyyUtility();

    Flyy.STATUSBAR_COLOR = Colors.black;
    Flyy.TOOLBAR_COLOR = Colors.black;
    Flyy.PARTNER_TOKEN = partnerKey;

    sharedPreferences = await SharedPreferences.getInstance();
    if (sharedPreferences.getBool(SharedPrefTags.IS_FIRST_OPENED) == null ||
        !sharedPreferences.getBool(SharedPrefTags.IS_FIRST_OPENED)) {
      initReferrerDetails();
//      sendEvent("registration", "done"); ///removed as we are using setUser/setNewUser methods
      sharedPreferences.setBool(SharedPrefTags.IS_FIRST_OPENED, true);
    }

    fetchAppInfo();

  }

  ///To catch installation from playstore
  // Platform messages are asynchronous, so we initialize in an async method.
  Future<void> initReferrerDetails() async {
    String referrerDetailsString;
    // Platform messages may fail, so we use a try/catch PlatformException.
    try {
      ReferrerDetails referrerDetails =
          await AndroidPlayInstallReferrer.installReferrer;

      referrerDetailsString = referrerDetails.installReferrer;
      if (referrerDetailsString != null && referrerDetailsString.isNotEmpty) {
        sendEvent("install", referrerDetailsString);
//        sendAppInstalledEvent(referrerDetailsString);
      }
    } catch (e) {
      referrerDetailsString = 'Failed to get referrer details: $e';
    }
  }

  ///fetch app information
  Future<FlyyAppInfo> fetchAppInfo() async {
    apiClient = new APIClient();
    final response = await apiClient.getClient("v1/flyy_app_info");
    if (response.statusCode == 200 && response.body != null) {
      final responseJson = json.decode(response.body);

      flyyUtility = new FlyyUtility();
      flyyUtility.fetchWalletData();

      sharedPreferences = await SharedPreferences.getInstance();

      sharedPreferences.setString(SharedPrefTags.FLYY_APP_INFO, response.body);

      return FlyyAppInfo.fromJson(responseJson);
    } else {
      throw Exception('Failed to load data ' + response.toString());
    }
  }

  ///send event method
  Future<FlyyEventResponse> sendEvent(String key, String value) async {
    Map<String, String> bodyParams = new Map();
    bodyParams['key'] = key.toLowerCase();
    bodyParams['value'] = value.toLowerCase();

    List<String> sdkEvents = new List();
    sdkEvents.add("set_new_user");
    sdkEvents.add("set_user");
    sdkEvents.add("install");
    sdkEvents.add("app_foreground");
    sdkEvents.add("app_background");
    sdkEvents.add("checkin_button_clicked");
    sdkEvents.add("invite_button_clicked");
    sdkEvents.add("invite_details_screen_view");
    sdkEvents.add("offers_screen_open");
    sdkEvents.add("offers_screen_closed");

    bool isInternal = sdkEvents.contains(key);

    apiClient = new APIClient();
    final response = await apiClient.postClient(
        "track_flyy_event", bodyParams, key, value, isInternal);
    if (response.statusCode == 200) {
      final responseJson = json.decode(response.body);
      FlyyEventResponse flyyEventResponse =
          FlyyEventResponse.fromJson(responseJson);

      if (key != null && (key == "registration" || key == "install")) {
        fetchAppInfo();
      }

      if (!flyyEventResponse.success) sendEvent(key, value);

      return flyyEventResponse;
    } else {
      sendEvent(key, value);
    }

    return null;
  }

  ///send installed event
//  static Future<FlyyEventResponse> sendAppInstalledEvent(
//      String referrerUrl) async {
//    Map<String, String> bodyParams = new Map();
//    bodyParams['referrer'] = referrerUrl;
//
//    final response =
//        await APIClient.postClient("app_installed", bodyParams, null, null);
//    if (response.statusCode == 200) {
//      final responseJson = json.decode(response.body);
//      FlyyEventResponse flyyEventResponse =
//          FlyyEventResponse.fromJson(responseJson);
//
//      return flyyEventResponse;
//    } else {
//      sendAppInstalledEvent(referrerUrl);
//    }
//    return null;
//  }

  void setUsername(String userName) {
    Flyy.USER_NAME = userName;
    setUserData("user_name", userName);
  }

//    public static void setContactNumber(String contact_number){
//        setUserData("contact_number", contact_number);
//    }

//  void setExtUid(String exUid) {
//    Flyy.EXT_USER_ID = exUid;
//    setUserData("ext_uid", exUid);
//  }

  void setNewUser(String extUid) {
    if (extUid != null && extUid.isNotEmpty) {
      Flyy.EXT_USER_ID = extUid;
      sendEvent("set_new_user", extUid);
    }
  }

  void setUser(String extUid) {
    if (extUid != null && extUid.isNotEmpty) {
      sendEvent("set_user", extUid);
    }
  }

  Future<FlyyUserData> setUserData(String key, String value) async {
    Map<String, String> bodyParams = new Map();
    bodyParams[key] = value.toLowerCase();

    var response;
    apiClient = new APIClient();

    switch (key) {
      case "user_name":
        response = await apiClient.putClient("v1/set_user_name", bodyParams);
        break;
      case "contact_number":
        response =
            await apiClient.putClient("v1/set_contact_number", bodyParams);
        break;
      case "ext_uid":
        response = await apiClient.putClient("v1/set_ext_uid", bodyParams);
        break;
    }

    if (response.statusCode == 200) {
      final responseJson = json.decode(response.body);
      FlyyUserData flyyUserData = FlyyUserData.fromJson(responseJson);

      if (!flyyUserData.success) setUserData(key, value);

      return flyyUserData;
    } else {
      setUserData(key, value);
    }
    return null;
  }
}
