import 'dart:convert';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:fluttertoast/fluttertoast.dart';
import 'package:flyyfluttersdk/Flyy.dart';
import 'package:flyyfluttersdk/helpers/APIClient.dart';
import 'package:flyyfluttersdk/helpers/FlyyUtility.dart';
import 'package:flyyfluttersdk/model/FlyyGiftCardData.dart';
import 'package:flyyfluttersdk/model/FlyyGiftCardOrderResponse.dart';
import 'package:flyyfluttersdk/model/FlyyWalletCompleteData.dart';
import 'package:flyyfluttersdk/model/FlyyWalletModel.dart';
import 'package:flyyfluttersdk/views/FlyyTransactionsPage.dart';

class FlyyBuyGiftCardDialog extends StatelessWidget {
  FlyyUtility flyyUtility;
  FlyyGiftCardData flyyGiftCardData;
  FlyyWalletModel flyyWalletModel;
  FlyyWalletCompleteData flyyWalletCompleteData;
  FocusNode nameFocusNode = new FocusNode();
  FocusNode emailFocusNode = new FocusNode();
  final receiversName = TextEditingController(),
      receiversEmail = TextEditingController();

  FlyyBuyGiftCardDialog(
      {Key key,
      @required this.flyyGiftCardData,
      this.flyyWalletModel,
      this.flyyWalletCompleteData})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    flyyUtility = new FlyyUtility();
    return Dialog(
      shape: RoundedRectangleBorder(
        borderRadius: BorderRadius.circular(5),
      ),
      backgroundColor: Colors.white,
      insetPadding: EdgeInsets.all(40),
      child: dialogContent(context),
    );
  }

  dialogContent(BuildContext context) {
    return Container(
      padding: const EdgeInsets.all(30),
      child: SingleChildScrollView(
        child: Column(
          crossAxisAlignment: CrossAxisAlignment.start,
          children: <Widget>[
            ///gift card name title
            Center(
              child: Padding(
                padding: const EdgeInsets.only(bottom: 25),
                child: Text(
                  flyyGiftCardData.name,
                  style: TextStyle(
                      fontSize: 15, color: FlyyUtility.parseColor('#373535')),
                ),
              ),
            ),

            ///receivers name title
            Text(
              'Receiver\'s Name*',
              style: TextStyle(
                  fontSize: 13, color: FlyyUtility.parseColor('#575555')),
            ),

            ///receivers name input
            Padding(
              padding: const EdgeInsets.only(bottom: 25),
              child: TextField(
                controller: receiversName,
                autofocus: true,
                keyboardType: TextInputType.text,
                textInputAction: TextInputAction.next,
                textCapitalization: TextCapitalization.characters,
                decoration: InputDecoration(
                    hintText: 'Input Name here',
                    labelStyle: TextStyle(
                        fontSize: 13, color: FlyyUtility.parseColor('#544e4e')),
                    hintStyle: TextStyle(
                        fontSize: 12,
                        color: FlyyUtility.parseColor('#b8b3b3'))),
                style: TextStyle(fontSize: 13),
                onSubmitted: (_) => FocusScope.of(context).nextFocus(),
                focusNode: nameFocusNode,
                /* validator: (String arg) {
                  if (arg != null && arg.isNotEmpty) {
                    return null;
                  } else {
                    return 'Required';
                  }
                },*/
              ),
            ),

            ///receivers email title
            Text(
              'Receiver\'s Email*',
              style: TextStyle(
                  fontSize: 13, color: FlyyUtility.parseColor('#575555')),
            ),

            ///receivers email input
            Padding(
              padding: const EdgeInsets.only(bottom: 25),
              child: TextField(
                controller: receiversEmail,
                keyboardType: TextInputType.emailAddress,
                decoration: InputDecoration(
                    hintText: 'Input email here',
                    labelStyle: TextStyle(
                        fontSize: 13, color: FlyyUtility.parseColor('#544e4e')),
                    hintStyle: TextStyle(
                        fontSize: 12,
                        color: FlyyUtility.parseColor('#b8b3b3'))),
                style: TextStyle(fontSize: 13),
                focusNode: emailFocusNode,
                /*validator: (String arg) {
                  if (arg != null && arg.isNotEmpty) {
                    if (FlyyUtility.isValidEmail(arg)) {
                      return 'Not a valid email';
                    } else {
                      return null;
                    }
                  } else {
                    return 'Required';
                  }
                },*/
              ),
            ),

            ///Value and required balance layout
            Row(
              mainAxisAlignment: MainAxisAlignment.spaceBetween,
              children: <Widget>[
                Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children: <Widget>[
                    Text(
                      'Value: ',
                      style: TextStyle(
                          fontSize: 13,
                          color: FlyyUtility.parseColor('#575555')),
                    ),
                    Text(
                      '₹' + flyyGiftCardData.value,
                      style: TextStyle(
                          fontSize: 13,
                          color: FlyyUtility.parseColor('#323232')),
                    )
                  ],
                ),
                Column(
                  children: <Widget>[
                    Text(
                      'Required Balance: ',
                      style: TextStyle(
                          fontSize: 13,
                          color: FlyyUtility.parseColor('#575555')),
                    ),
                    Row(
                      children: <Widget>[
                        flyyGiftCardData.currencyIconUrl != null &&
                                flyyGiftCardData.currencyIconUrl.isNotEmpty
                            ? Image.network(
                                flyyGiftCardData.currencyIconUrl,
                                height: 15,
                                width: 15,
                              )
                            : SizedBox.shrink(),
                        Text(
                          flyyUtility.setRewardText(
                              flyyGiftCardData.currencyReq,
                              flyyGiftCardData.currencyLabel),
                          style: TextStyle(
                              fontSize: 13,
                              color: FlyyUtility.parseColor('#323232')),
                        ),
                      ],
                    )
                  ],
                ),
              ],
            ),

            ///Withdraw button
            GestureDetector(
              onTap: () {
                if (receiversName.text != null &&
                    receiversName.text.isNotEmpty &&
                    receiversEmail.text != null &&
                    receiversEmail.text.isNotEmpty &&
                    FlyyUtility.isValidEmail(receiversEmail.text)) {
                  FlyyUtility.checkIfInterentPresent().then((isConnected) {
                    if (isConnected != null && isConnected) {
                      // Internet Present Case
                      try {
                        sendDataToServer(context);
                      } catch (e) {
                        print(e);
                      }
                    } else {
                      // No-Internet Case
                      FlyyUtility.showToast(
                          context,
                          ("Please check your internet connection and try again."),
                          ToastGravity.BOTTOM);
                    }
                  });
                } else {
                  if (receiversName.text != null &&
                      receiversName.text.isEmpty) {
                    FlyyUtility.showToast(
                        context, 'Name cannot be empty', ToastGravity.CENTER);
                    nameFocusNode.requestFocus();
                  } else if (receiversEmail.text != null &&
                      receiversEmail.text.isEmpty) {
                    FlyyUtility.showToast(
                        context, 'Email cannot be empty', ToastGravity.CENTER);
                    emailFocusNode.requestFocus();
                  } else if (receiversEmail.text != null &&
                      receiversEmail.text.isNotEmpty &&
                      !FlyyUtility.isValidEmail(receiversEmail.text)) {
                    FlyyUtility.showToast(context, 'Please enter valid email',
                        ToastGravity.CENTER);
                    emailFocusNode.requestFocus();
                  }
                }
              },
              child: Center(
                child: Container(
                  height: 45,
                  width: 100,
                  margin: const EdgeInsets.only(top: 25),
                  child: Card(
                    color: Flyy.TOOLBAR_COLOR,
                    child: Center(
                      child: Text(
                        'Buy',
                        style: new TextStyle(fontSize: 14, color: Colors.white),
                      ),
                    ),
                  ),
                ),
              ),
            ),
          ],
        ),
      ),
    );
  }

  Future<FlyyGiftCardOrderResponse> sendDataToServer(
      BuildContext context) async {
    showAlertDialog(context);

    Map<String, String> bodyParams = new Map();
    bodyParams['name'] = receiversName.text;
    bodyParams['email'] = receiversEmail.text;
    bodyParams['sku'] = flyyGiftCardData.sku;
    bodyParams['price'] = flyyGiftCardData.value;
    APIClient apiClient = new APIClient();
    final response =
        await apiClient.postClient("buy_gift_cards", bodyParams, null, null, false);
    if (response.statusCode == 200) {
      final responseJson = json.decode(response.body);
      FlyyGiftCardOrderResponse flyyGiftCardOrderResponse =
          FlyyGiftCardOrderResponse.fromJson(responseJson);

      FlyyUtility.showToast(
          context, flyyGiftCardOrderResponse.message, ToastGravity.CENTER);

      if (flyyGiftCardOrderResponse.success) {
        flyyUtility.fetchWalletData();

        Navigator.push(
          context,
          MaterialPageRoute(
              builder: (context) => FlyyTransactionsPage(
                    flyyWalletCompleteData: flyyWalletCompleteData,
                    flyyWalletModel: flyyWalletModel,
                    showLastBoughtGiftCardPopup: true,
                  )),
        ).then((value) => Navigator.of(context).pop());
      }

      return FlyyGiftCardOrderResponse.fromJson(responseJson);
    } else {
      FlyyUtility.showToast(
          context, 'Something went wrong', ToastGravity.CENTER);
      throw Exception('Failed to load data ' + response.toString());
    }
  }

  Widget showAlertDialog(BuildContext context) {
    showDialog(
      barrierDismissible: false,
      useRootNavigator: true,
      context: context,
      builder: (BuildContext context) {
        return AlertDialog(
          content: new Row(
            children: [
              CircularProgressIndicator(),
              Container(
                  margin: EdgeInsets.only(left: 5),
                  child: Text("Processing, please wait...")),
            ],
          ),
        );
      },
    );
  }
}
