import 'dart:convert';
import 'dart:ui';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_dash/flutter_dash.dart';
import 'package:flutter_html/flutter_html.dart';
import 'package:flutter_svg/flutter_svg.dart';
import 'package:flyyfluttersdk/Flyy.dart';
import 'package:flyyfluttersdk/helpers/APIClient.dart';
import 'package:flyyfluttersdk/helpers/FlyyCommonColors.dart';
import 'package:flyyfluttersdk/helpers/FlyyUtility.dart';
import 'package:flyyfluttersdk/helpers/ReusableAppbarWidget.dart';
import 'package:flyyfluttersdk/model/FlyyCheckInDataGetResponse.dart';
import 'package:flyyfluttersdk/model/FlyyOfferPrizes.dart';
import 'package:flyyfluttersdk/model/FlyyOffers.dart';
import 'package:flyyfluttersdk/model/FlyyStepBean.dart';

class FlyyCheckinDetailPage extends StatelessWidget {
  FlyyOffers flyyOffers;

  FlyyCheckinDetailPage({Key key, @required this.flyyOffers}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: ReusableAppbarWidget.getAppBar(
          context, FlyyUtility.cleanHtmlStrings(flyyOffers.offerTypeTitle)),
      body: FlyyCheckinDetailStateFulWidget(
        flyyOffers: flyyOffers,
      ),
    );
  }
}

class FlyyCheckinDetailStateFulWidget extends StatefulWidget {
  FlyyOffers flyyOffers;

  FlyyCheckinDetailStateFulWidget({Key key, @required this.flyyOffers})
      : super(key: key);

  @override
  State<StatefulWidget> createState() {
    return new FlyyCheckinDetailState();
  }
}

class FlyyCheckinDetailState extends State<FlyyCheckinDetailStateFulWidget> {
  FlyyUtility flyyUtility;
  
  @override
  Widget build(BuildContext context) {
    flyyUtility = new FlyyUtility();
    flyyUtility.getBeanList(widget.flyyOffers);
//    FlyyUtility.checkinBeanList = FlyyUtility.getBeanList(widget.flyyOffers);
    return Scaffold(
      body: Container(
        color: FlyyCommonColors.BG_GREY,
        height: double.maxFinite,
        child: SingleChildScrollView(
          child: Center(
            child: Container(
              margin:
                  const EdgeInsets.only(top: 5, right: 5, left: 5, bottom: 20),
              child: Card(
                child: Column(
                  crossAxisAlignment: CrossAxisAlignment.stretch,
                  children: <Widget>[
                    ///Invite and earn icon
                    Container(
                      margin: const EdgeInsets.only(top: 10, bottom: 15),
                      child: SvgPicture.network(
                        "https://ghc.theflyy.com/assets/fs_assets/daily_checkin.svg",
                        height: 35,
                        width: 35,
                        alignment: Alignment.center,
                        color: Flyy.TOOLBAR_COLOR,
                      ),
                    ),

                    ///Invite and earn title
                    Container(
                      margin: const EdgeInsets.only(bottom: 15),
                      child: Text(
                        FlyyUtility.cleanHtmlStrings(widget.flyyOffers.title),
                        textAlign: TextAlign.center,
                        style: new TextStyle(
                            fontSize: 16,
                            color: FlyyUtility.parseColor("#373535")),
                      ),
                    ),

                    ///Description
                    Container(
                      margin: const EdgeInsets.only(
                          bottom: 25, left: 80, right: 80),
                      child: Text(
                        FlyyUtility.cleanHtmlStrings(
                            widget.flyyOffers.description),
                        textAlign: TextAlign.center,
                        style: new TextStyle(
                            fontSize: 14,
                            color: FlyyUtility.parseColor("#585957")),
                      ),
                    ),

                    ///Badge level or imageview
                    widget.flyyOffers.flyyOfferPrizes != null &&
                            widget.flyyOffers.flyyOfferPrizes.length > 0
                        ? Padding(
                            padding: const EdgeInsets.all(10),
                            child: Column(
                              children: <Widget>[
                                Padding(
                                  padding: const EdgeInsets.only(bottom: 15),
                                  child: Text(
                                    'Badge Level',
                                    style: new TextStyle(
                                        fontSize: 16,
                                        color:
                                            FlyyUtility.parseColor("#373535")),
                                  ),
                                ),
                                Padding(
                                  padding: const EdgeInsets.only(
                                      left: 10, right: 10),
                                  child: ListView.separated(
                                    separatorBuilder: (context, index) =>
                                        Divider(
                                      color: FlyyUtility.parseColor("#D3D3D3"),
                                    ),
                                    itemBuilder: setOfferPrizesList,
                                    itemCount: widget
                                        .flyyOffers.flyyOfferPrizes.length,
                                    scrollDirection: Axis.vertical,
                                    shrinkWrap: true,
                                    physics:
                                        const NeverScrollableScrollPhysics(),
                                  ),
                                ),
                              ],
                            ))
                        : Container(
                            margin: const EdgeInsets.all(20),
                            child: widget.flyyOffers.imageUrl != null
                                ? Image.network(
                                    widget.flyyOffers.imageUrl,
                                    height: 200,
                                    width: 400,
                                  )
                                : Container(
                                    height: 200.0,
                                    width: 400.0,
                                    color: Colors.blueGrey,
                                  ),
                          ),

                    ///Divider
                    widget.flyyOffers.flyyOfferPrizes != null &&
                            widget.flyyOffers.flyyOfferPrizes.length > 0
                        ? Container(
                            width: double.infinity,
                            height: 1,
                            margin: const EdgeInsets.only(
                                left: 20, right: 20, bottom: 10),
                            child: Divider(
                              color: FlyyUtility.parseColor("#D3D3D3"),
                            ),
                          )
                        : SizedBox.shrink(),

                    ///checkin listview
                    Center(
                      child: Container(
                        margin:
                            const EdgeInsets.only(top: 10, left: 20, right: 20),
                        constraints: new BoxConstraints(
                          minHeight: 30.0,
                          maxHeight: 50.0,
                        ),
                        child: ListView.builder(
                          itemBuilder: setCheckinListView,
                          itemCount: FlyyUtility.checkinBeanList.length,
                          scrollDirection: Axis.horizontal,
                          shrinkWrap: true,
                        ),
                      ),
                    ),

                    ///Checkin button
                    GestureDetector(
                      onTap: () {
                        if (widget.flyyOffers.checkedIn == null ||
                            !widget.flyyOffers.checkedIn) {
                          setState(() {
                            FlyyUtility.checkInTitle = "Checking in...";
                          });
                          sendCheckInData(widget.flyyOffers);
                        }
                      },
                      child: Container(
                        height: 45,
                        width: 100,
                        margin: const EdgeInsets.only(
                            top: 20, bottom: 15, left: 100, right: 100),
                        child: Card(
                          color: Flyy.TOOLBAR_COLOR,
                          child: Center(
                            child: Text(
                              FlyyUtility.checkInTitle != null &&
                                      FlyyUtility.checkInTitle.isNotEmpty
                                  ? FlyyUtility.checkInTitle
                                  : widget.flyyOffers.checkedIn == null ||
                                          !widget.flyyOffers.checkedIn
                                      ? "Check In"
                                      : "Checked In",
                              style: new TextStyle(
                                  fontSize: 14, color: Colors.white),
                            ),
                          ),
                        ),
                      ),
                    ),

                    ///Offers terms and conditions
                    widget.flyyOffers.offerTerms != null &&
                            widget.flyyOffers.offerTerms.length > 0
                        ? Container(
                            margin: const EdgeInsets.only(top: 20, bottom: 20),
                            child: Column(
                              children: <Widget>[
                                Padding(
                                  padding: const EdgeInsets.only(bottom: 10),
                                  child: Text(
                                    'Offer Details',
                                    style: new TextStyle(
                                        fontSize: 16,
                                        color:
                                            FlyyUtility.parseColor("#373535")),
                                  ),
                                ),
                                Container(
                                    margin: const EdgeInsets.only(
                                        left: 10, right: 50),
                                    child: Html(
                                        data: widget.flyyOffers.offerTerms,
                                        defaultTextStyle:
                                            TextStyle(fontSize: 12))),
                              ],
                            ),
                          )
                        : SizedBox.shrink(),
                  ],
                ),
              ),
            ),
          ),
        ),
      ),
    );
  }

  Widget setOfferPrizesList(BuildContext context, int index) {
    FlyyOfferPrizes flyyOfferPrizes = widget.flyyOffers.flyyOfferPrizes[index];
    return Padding(
      padding: const EdgeInsets.only(left: 15, right: 15),
      child: Container(
        child: Row(
          children: <Widget>[
            ///Icon
            Padding(
              padding: const EdgeInsets.only(right: 10),
              child: flyyOfferPrizes.badgeUrl != null &&
                      flyyOfferPrizes.badgeUrl.isNotEmpty
                  ? SvgPicture.network(
                      flyyOfferPrizes.badgeUrl,
                      height: 80,
                      width: 50,
                    )
                  : Icon(
                      Icons.photo,
                      size: 40,
                    ),
            ),

            ///Two texts title and description
            Expanded(
              child: Padding(
                padding: const EdgeInsets.only(left: 5),
                child: Container(
                  alignment: Alignment.centerLeft,
                  child: Column(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: <Widget>[
                      Text(
                        flyyOfferPrizes.label,
                        style: new TextStyle(
                            fontSize: 12,
                            color: FlyyUtility.parseColor("#373535")),
                      ),
                      Text(
                        'For Referral ' +
                            flyyOfferPrizes.minRef.toString() +
                            ' to ' +
                            flyyOfferPrizes.maxRef.toString(),
                        style: new TextStyle(
                            fontSize: 10,
                            color: FlyyUtility.parseColor("#585957")),
                      )
                    ],
                  ),
                ),
              ),
            ),

            ///small box to display tokens and cash
            Column(
              children: <Widget>[
                Container(
                  child: Padding(
                    padding: const EdgeInsets.only(bottom: 5),
                    child: Row(
                      mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                      children: <Widget>[
                        widget.flyyOffers.currencyIconUrl != null
                            ? Image.network(
                                widget.flyyOffers.currencyIconUrl,
                                height: 10,
                                width: 10,
                              )
                            : SizedBox.shrink(),

                        ///Text
                        Padding(
                          padding: const EdgeInsets.only(left: 5.0),
                          child: Text(
                            widget.flyyOffers.currencyLabel != null &&
                                    widget.flyyOffers.currencyLabel.isNotEmpty
                                ? widget.flyyOffers.currencyLabel
                                : 'Scratch Card',
                            style: new TextStyle(
                                fontSize: 8,
                                color: FlyyUtility.parseColor("#373535")),
                          ),
                        )
                      ],
                    ),
                  ),
                ),
                Container(
                  height: 25,
                  width: 70,
                  decoration: BoxDecoration(
                    border: Border.all(
                      color: Flyy.TOOLBAR_COLOR,
                    ),
                    borderRadius: BorderRadius.circular(5.0),
                  ),
                  child: Padding(
                    padding: const EdgeInsets.all(5),
                    child: Center(
                      child: Text(
                        setOffersPrizeListText(context, flyyOfferPrizes,
                            widget.flyyOffers.currencyLabel),
                        textAlign: TextAlign.center,
                        style: new TextStyle(
                            fontSize: 10,
                            color: FlyyUtility.parseColor("#373535")),
                      ),
                    ),
                  ),
                ),
              ],
            )
          ],
        ),
      ),
    );
  }

  String setOffersPrizeListText(BuildContext context,
      FlyyOfferPrizes flyyOfferPrizes, String currencyLabel) {
    if (flyyOfferPrizes.minWin > 0) {
      if (currencyLabel == ('Cash') || currencyLabel == ('Cash')) {
        return '₹' +
            flyyUtility.checkAmount(flyyOfferPrizes.minWin) +
            ' - ' +
            '₹' +
            flyyUtility.checkAmount(flyyOfferPrizes.maxWin);
      } else {
        if (currencyLabel == ('Cash') || currencyLabel == ('Cash')) {
          return '₹' + flyyUtility.checkAmount(flyyOfferPrizes.maxWin);
        } else {
          return flyyUtility.checkAmount(flyyOfferPrizes.maxWin) +
              ' ' +
              currencyLabel;
        }
      }
    } else {
      if (currencyLabel == ('Cash') || currencyLabel == ('Cash')) {
        return '₹' + flyyUtility.checkAmount(flyyOfferPrizes.maxWin);
      } else {
        return flyyUtility.checkAmount(flyyOfferPrizes.maxWin) +
            ' ' +
            currencyLabel;
      }
    }
  }

  String setRewardText(BuildContext context, FlyyOffers flyyOffers) {
    if (flyyOffers.flyyOfferPrizes != null &&
        flyyOffers.flyyOfferPrizes.length > 0) {
      int minWin = flyyOffers.flyyOfferPrizes.first.minWin;
      int maxWin = flyyOffers.flyyOfferPrizes.last.maxWin;
      if (flyyOffers.flyyOfferPrizes.first.minWin > 0) {
        if (flyyOffers.currencyLabel == ('cash') ||
            flyyOffers.currencyLabel == ('Cash')) {
          return flyyUtility.setRewardText(minWin, flyyOffers.currencyLabel) +
              ' - ' +
              flyyUtility.setRewardText(maxWin, flyyOffers.currencyLabel);
        } else {
          return minWin.toString() +
              ' - ' +
              flyyUtility.setRewardText(maxWin, flyyOffers.currencyLabel);
        }
      } else {
        return 'Upto ' +
            flyyUtility.setRewardText(maxWin, flyyOffers.currencyLabel);
      }
    } else {
      int minWin = flyyOffers.minReward;
      int maxWin = flyyOffers.maxReward;
      if (flyyOffers.minReward > 0) {
        if (flyyOffers.currencyLabel == ('cash') ||
            flyyOffers.currencyLabel == ('Cash')) {
          return flyyUtility.setRewardText(minWin, flyyOffers.currencyLabel) +
              ' - ' +
              flyyUtility.setRewardText(maxWin, flyyOffers.currencyLabel);
        } else {
          return minWin.toString() +
              ' - ' +
              flyyUtility.setRewardText(maxWin, flyyOffers.currencyLabel);
        }
      } else {
        return 'Upto ' +
            flyyUtility.setRewardText(maxWin, flyyOffers.currencyLabel);
      }
    }
  }

  Widget setCheckinListView(BuildContext context, int index) {
    FlyyStepBean flyyStepBean = FlyyUtility.checkinBeanList[index];
    Color dashColor = widget.flyyOffers.checkedIn
        ? FlyyUtility.parseColor("585957")
        : FlyyUtility.parseColor("585957");
    return Container(
      child: Row(
        children: <Widget>[
          Column(
            children: <Widget>[
              setImageAsPerCheckinState(flyyStepBean),
              Padding(
                padding: const EdgeInsets.only(top: 5),
                child: Text(
                  flyyStepBean.name,
                  style: TextStyle(
                      fontSize: 12, fontWeight: flyyStepBean.fontWeight),
                ),
              )
            ],
          ),
          index < FlyyUtility.checkinBeanList.length - 1
              ? Padding(
                  padding: const EdgeInsets.only(bottom: 35),
                  child: Dash(
                      direction: Axis.horizontal,
                      length: 30,
                      dashLength: 5,
                      dashColor: dashColor),
                )
              : SizedBox.shrink(),
        ],
      ),
    );
  }

  Widget setImageAsPerCheckinState(FlyyStepBean flyyStepBean) {
    Color iconColor = null;
    if (flyyStepBean.iconDrawable.contains("ic_checkin_filled") ||
        flyyStepBean.iconDrawable.contains("ic_checkin_filler")) {
      iconColor = Flyy.TOOLBAR_COLOR;
    }
    if (flyyStepBean.iconDrawable.contains("http") ||
        flyyStepBean.iconDrawable.contains("https")) {
      if (flyyStepBean.iconDrawable.contains(".png") ||
          flyyStepBean.iconDrawable.contains(".jpg")) {
        return Image.network(
          flyyStepBean.iconDrawable,
          height: 20,
          width: 20,
          color: iconColor,
        );
      } else if (flyyStepBean.iconDrawable.contains(".svg")) {
        return SvgPicture.network(
          flyyStepBean.iconDrawable,
          height: 20,
          width: 20,
          color: iconColor,
        );
      }
    }
    /* else if (flyyStepBean.iconDrawable.contains("svg")) {
      return SvgPicture.asset(
        flyyStepBean.iconDrawable,
        height: 20,
        width: 20,
      );
    } else {
      return Image.asset(
        flyyStepBean.iconDrawable,
        height: 20,
        width: 20,
        color: Flyy.TOOLBAR_COLOR,
      );
    }*/
  }

  /*Future<FlyyCheckInDataGetResponse> sendCheckInData() async {
    final response = await APIClient.getClient("checkin");
    if (response.statusCode == 200) {
      final responseJson = json.decode(response.body);

      setState(() {
        if (response.statusCode == 200) {
          final responseJson = json.decode(response.body);
          FlyyCheckInDataGetResponse flyyCheckInResponse =
              FlyyCheckInDataGetResponse.fromJson(responseJson);
          widget.flyyOffers.streak = flyyCheckInResponse.streak;
          widget.flyyOffers.checkedIn = true;
          widget.flyyOffers.buttonText = "Checked In";
          checkinBeanList.addAll(getBeanList(widget.flyyOffers));
          checkInTitle = "Checked In";
        }
      });

      return FlyyCheckInDataGetResponse.fromJson(responseJson);
    } else {
      setState(() {
        checkInTitle = "Check In";
      });
      throw Exception('Failed to load data ' + response.toString());
    }
  }*/

  Future<FlyyCheckInDataGetResponse> sendCheckInData(
      FlyyOffers flyyOffers) async {
    APIClient apiClient = new APIClient();
    final response = await apiClient.getClient("checkin");
    if (response.statusCode == 200) {
      final responseJson = json.decode(response.body);
      setState(() {
        if (response.statusCode == 200) {
          final responseJson = json.decode(response.body);
          FlyyCheckInDataGetResponse flyyCheckInResponse =
              FlyyCheckInDataGetResponse.fromJson(responseJson);
          flyyOffers.streak = flyyCheckInResponse.streak;
          flyyOffers.checkedIn = true;
          flyyOffers.buttonText = "Checked In";
          widget.flyyOffers = flyyOffers;
          flyyUtility.getBeanList(widget.flyyOffers);
          FlyyUtility.checkInTitle = "Checked In";
        }
      });

      return FlyyCheckInDataGetResponse.fromJson(responseJson);
    } else {
      setState(() {
        FlyyUtility.checkInTitle = "Check In";
      });
      throw Exception('Failed to load data ' + response.toString());
    }
  }
}
