import 'dart:async';
import 'dart:convert';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_html/flutter_html.dart';
import 'package:flutter_svg/flutter_svg.dart';
import 'package:fluttertoast/fluttertoast.dart';
import 'package:flyyfluttersdk/Flyy.dart';
import 'package:flyyfluttersdk/helpers/APIClient.dart';
import 'package:flyyfluttersdk/helpers/CustomCountdownTimer.dart';
import 'package:flyyfluttersdk/helpers/FlyyCommonColors.dart';
import 'package:flyyfluttersdk/helpers/FlyyUtility.dart';
import 'package:flyyfluttersdk/helpers/ScrollingText.dart';
import 'package:flyyfluttersdk/model/FlyyLeaderboard.dart';
import 'package:flyyfluttersdk/model/FlyyPlayGame.dart';
import 'package:flyyfluttersdk/model/FlyyTournament.dart';
import 'package:flyyfluttersdk/model/FlyyTournamentData.dart';
import 'package:flyyfluttersdk/model/FlyyTournamentPrize.dart';
import 'package:flyyfluttersdk/model/FlyyWinText.dart';
import 'package:flyyfluttersdk/views/FlyyGamesLeaderBoardDialog.dart';
import 'package:flyyfluttersdk/views/FlyyPlayGamesPage.dart';
import 'package:flyyfluttersdk/views/FlyyTournamentHistoryPage.dart';

///Stateless widget
class FlyyGameDetailsPage extends StatelessWidget {
  FlyyTournament flyyTournament;
  FlyyTournamentData flyyTournamentData;

  FlyyGameDetailsPage(
      {Key key, @required this.flyyTournament, this.flyyTournamentData})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        bottomOpacity: 0.0,
        elevation: 0.0,
        titleSpacing: 0.0,
        leading: IconButton(
          icon: Icon(Icons.arrow_back, color: Colors.white),
          onPressed: () {
            Navigator.of(context, rootNavigator: false).pop();
          },
        ),
        title: Text(
          flyyTournamentData.title,
          style: TextStyle(fontSize: 18),
        ),
        actions: <Widget>[
          Container(
            margin: const EdgeInsets.only(right: 10),
            child: IconButton(
              icon: Image.network(
                "https://ghc.theflyy.com/assets/fs_assets/history.png",
                height: 35,
                width: 35,
              ),
              onPressed: () {
                Navigator.push(
                  context,
                  MaterialPageRoute(
                      builder: (context) => FlyyTournamentHistoryPage(
                            prizeType: flyyTournamentData.prizeType,
                          )),
                );
              },
            ),
          )
        ],
        backgroundColor: Flyy.TOOLBAR_COLOR,
      ),
      body: makeNetworkCall(context),
    );
  }

  ///fetch data
  Widget makeNetworkCall(BuildContext context) {
    return FutureBuilder<FlyyTournament>(
      future: getTournamentData(flyyTournamentData.gameId),
      builder: (BuildContext context, AsyncSnapshot response) {
        if (response.connectionState == ConnectionState.waiting) {
          return Center(child: CircularProgressIndicator());
        } else {
          if (response.hasData && response.data != null) {
            return FlyyGameDetailsStateFulWidget(
              flyyTournament: flyyTournament,
              flyyTournamentData: flyyTournamentData,
            );
          } else if (response.hasError) {
            print(response.error);
          }
        }
        return Center(child: CircularProgressIndicator());
      },
    );
  }

  Future<FlyyTournament> getTournamentData(int gameId) async {
    APIClient apiClient = new APIClient();
    final response = await apiClient.getClient("v1/web_games?partner=" +
        Flyy.PARTNER_TOKEN +
        "&game_id=" +
        gameId.toString() +
        "&user_name=" +
        "" +
        "&contact_number=" +
        Flyy.EXT_USER_ID);
    if (response.statusCode == 200) {
      final responseJson = json.decode(response.body);
      flyyTournament = FlyyTournament.fromJson(responseJson);
      flyyTournamentData = flyyTournament.game.tournament;

      return flyyTournament;
    } else {
      throw Exception('Failed to load data ' + response.toString());
    }
  }
}

///Stateful widget
class FlyyGameDetailsStateFulWidget extends StatefulWidget {
  FlyyTournament flyyTournament;
  FlyyTournamentData flyyTournamentData;

  FlyyGameDetailsStateFulWidget(
      {Key key, @required this.flyyTournament, this.flyyTournamentData})
      : super(key: key);

  @override
  State<StatefulWidget> createState() {
    return new FlyyGameDetailsState();
  }
}

///State class
class FlyyGameDetailsState extends State<FlyyGameDetailsStateFulWidget> {
  FlyyTournament flyyTournament;
  FlyyTournamentData flyyTournamentData;
  int page = 1;
  Timer timer;
  String chancesLeft, rules, playersJoinedText, winnerPercent, rank;
  List<FlyyLeaderboard> leaderboardList;
  List<FlyyTournamentPrize> prizesList;

  @override
  void initState() {
    super.initState();
    timer = Timer.periodic(Duration(seconds: 10), (Timer t) {
      fetchLiveTournamentData(flyyTournamentData.id);
    });
  }

  @override
  void dispose() {
    ///    question mark is intentional and not typo, but not absolutely necessary. It's just to avoid an exception in case the timer was not yet set or set to null by some other code. It's the null-save navigation operator.
    timer?.cancel();
    super.dispose();
  }

  void preloadData(
      FlyyTournament flyyTournament, FlyyTournamentData flyyTournamentData) {
    if (flyyTournamentData.playsLeft >= 0) {
      chancesLeft = flyyTournamentData.playsLeft.toString() + " Chance(s) Left";
    }
    if (flyyTournamentData.livePlayers != null) {
      playersJoinedText = flyyTournamentData.livePlayers.toString() +
          "/" +
          flyyTournamentData.maxPlayers.toString();
    } else {
      playersJoinedText = "0" + "/" + flyyTournamentData.maxPlayers.toString();
    }
    if (flyyTournamentData.yourRank != null && flyyTournamentData.yourRank > 0)
      rank = "#" + flyyTournamentData.yourRank.toString();
    if (flyyTournamentData.rules != null) rules = flyyTournamentData.rules;
    if (flyyTournamentData.prizes != null)
      prizesList = flyyTournamentData.prizes;
    if (flyyTournamentData.winnerPercent != null)
      winnerPercent = flyyTournamentData.winnerPercent.toString() + "% Winners";
    leaderboardList = flyyTournamentData.leaderboardData;
  }

  @override
  Widget build(BuildContext context) {
    flyyTournament = widget.flyyTournament;
    flyyTournamentData = widget.flyyTournamentData;
    preloadData(flyyTournament, flyyTournamentData);
    return Scaffold(
      bottomSheet: Container(
        height: 130,
        margin: const EdgeInsets.only(top: 10),
        alignment: Alignment.bottomCenter,
        color: Colors.white,
        child: Column(
          children: <Widget>[
            ///chances left
            chancesLeft != null && chancesLeft.isNotEmpty
                ? Text(chancesLeft,
                    style: TextStyle(
                        fontSize: 13, color: FlyyUtility.parseColor("#323232")))
                : SizedBox.shrink(),

            ///game ends in
            Padding(
              padding: const EdgeInsets.only(bottom: 10),
              child: setStartEndtext(flyyTournamentData.startsAt,
                  flyyTournamentData.endsAt, flyyTournamentData.isLive),
            ),
          ],
        ),
      ),
      body: Container(
        height: double.infinity,
        color: FlyyCommonColors.BG_GREY,
        padding: const EdgeInsets.only(bottom: 150),
        child: SingleChildScrollView(
          child: Column(
            children: <Widget>[
              Container(
                child: Column(
                  children: <Widget>[
                    ///first card with game details
                    Container(
                      margin: const EdgeInsets.all(5),
                      child: Card(
                        shape: RoundedRectangleBorder(
                          borderRadius: BorderRadius.circular(10),
                        ),
                        child: Column(
                          children: <Widget>[
                            ///game banner
                            Container(
                              decoration: BoxDecoration(
                                borderRadius: BorderRadius.only(
                                  topLeft: Radius.circular(5),
                                  topRight: Radius.circular(5),
                                  bottomLeft: Radius.zero,
                                  bottomRight: Radius.zero,
                                ),
                              ),
                              child: Image.network(
                                flyyTournament.game.wallUrl,
                                height: 150,
                                width: double.maxFinite,
                                fit: BoxFit.fill,
                              ),
                            ),

                            ///game time, players joined, rank layout
                            Padding(
                              padding: const EdgeInsets.only(top: 10),
                              child: Row(
                                mainAxisAlignment:
                                    MainAxisAlignment.spaceEvenly,
                                children: <Widget>[
                                  ///game time
                                  Column(
                                    children: <Widget>[
                                      ///timer image
                                      Row(
                                        children: <Widget>[
                                          Image.network(
                                            "https://ghc.theflyy.com/assets/fs_assets/timer.png",
                                            height: 25,
                                            width: 25,
                                          ),
                                          Padding(
                                            padding:
                                                const EdgeInsets.only(left: 5),
                                            child: Text(
                                              "Game Time",
                                              style: TextStyle(
                                                  fontSize: 12,
                                                  color: FlyyUtility.parseColor(
                                                      "#323232")),
                                            ),
                                          )
                                        ],
                                      ),

                                      Padding(
                                        padding: const EdgeInsets.only(top: 5),
                                        child: Row(
                                          mainAxisAlignment:
                                              MainAxisAlignment.spaceEvenly,
                                          children: <Widget>[
                                            Column(
                                              children: <Widget>[
                                                Text(
                                                    FlyyUtility.getTimeFormat(
                                                        flyyTournamentData
                                                            .startsAt),
                                                    style: TextStyle(
                                                        fontSize: 12,
                                                        color: FlyyUtility
                                                            .parseColor(
                                                                "#323232"))),
                                                Text(
                                                    FlyyUtility.getDateFormat(
                                                        flyyTournamentData
                                                            .startsAt),
                                                    style: TextStyle(
                                                        fontSize: 10,
                                                        color: FlyyUtility
                                                            .parseColor(
                                                                "#9A9A9A"))),
                                              ],
                                            ),
                                            Padding(
                                              padding: const EdgeInsets.only(
                                                  right: 5, left: 5),
                                              child: Text(
                                                "-",
                                                style: TextStyle(
                                                    color:
                                                        FlyyUtility.parseColor(
                                                            "#323232")),
                                              ),
                                            ),
                                            Column(
                                              children: <Widget>[
                                                Text(
                                                    FlyyUtility.getTimeFormat(
                                                        flyyTournamentData
                                                            .endsAt),
                                                    style: TextStyle(
                                                        fontSize: 12,
                                                        color: FlyyUtility
                                                            .parseColor(
                                                                "#323232"))),
                                                Text(
                                                    FlyyUtility.getDateFormat(
                                                        flyyTournamentData
                                                            .endsAt),
                                                    style: TextStyle(
                                                        fontSize: 10,
                                                        color: FlyyUtility
                                                            .parseColor(
                                                                "#9A9A9A"))),
                                              ],
                                            ),
                                          ],
                                        ),
                                      )
                                    ],
                                  ),

                                  ///rank and amount
                                  Column(
                                    children: <Widget>[
                                      ///to display image and text in center to show prize and type
                                      Stack(
                                          alignment: Alignment.center,
                                          children: <Widget>[
                                            Image.network(
                                              "https://ghc.theflyy.com/assets/fs_assets/ticket_colored.png",
                                              height: 80,
                                              width: 130,
                                            ),
                                            Column(
                                              children: <Widget>[
                                                Text(
                                                  flyyTournamentData.totalPrize
                                                      .toString(),
                                                  textAlign: TextAlign.center,
                                                  style: TextStyle(
                                                      color: Colors.white,
                                                      fontSize: 20),
                                                ),
                                                Text(
                                                  flyyTournamentData.prizeType,
                                                  textAlign: TextAlign.center,
                                                  style: TextStyle(
                                                      color: Colors.white,
                                                      fontSize: 12),
                                                ),
                                              ],
                                            )
                                          ]),
                                    ],
                                  ),

                                  ///players joined
                                  Column(
                                    children: <Widget>[
                                      ///user joined
                                      Row(
                                        children: <Widget>[
                                          SvgPicture.network(
                                            "https://ghc.theflyy.com/assets/fs_assets/user.svg",
                                            height: 25,
                                            width: 25,
                                          ),
                                          Padding(
                                            padding:
                                                const EdgeInsets.only(left: 5),
                                            child: Text(
                                              "Players Joined",
                                              style: TextStyle(
                                                  fontSize: 12,
                                                  color: FlyyUtility.parseColor(
                                                      "#323232")),
                                            ),
                                          )
                                        ],
                                      ),

                                      ///number of players joined
                                      Padding(
                                          padding:
                                              const EdgeInsets.only(top: 5),
                                          child: Text(
                                            playersJoinedText,
                                            style: TextStyle(
                                                fontSize: 14,
                                                color: FlyyUtility.parseColor(
                                                    "#323232")),
                                          ))
                                    ],
                                  ),
                                ],
                              ),
                            ),

                            rank != null && rank.isNotEmpty
                                ? Padding(
                                    padding: const EdgeInsets.only(bottom: 10),
                                    child: Row(
                                      mainAxisAlignment:
                                          MainAxisAlignment.center,
                                      children: <Widget>[
                                        Text(
                                          "Your rank ",
                                          style: TextStyle(
                                              fontSize: 14,
                                              color: FlyyUtility.parseColor(
                                                  "#323232")),
                                        ),
                                        GestureDetector(
                                          onTap: () {
                                            if (leaderboardList == null)
                                              leaderboardList = new List();
//                                            if (leaderboardList != null &&
//                                                leaderboardList.length > 0)
//                                              leaderboardList.addAll(
//                                                  flyyTournamentData
//                                                      .leaderboardData);
                                            FlyyLeaderboard flyyLeaderBoard =
                                                new FlyyLeaderboard(
                                                    name: flyyTournamentData
                                                        .yourName,
                                                    mobile: flyyTournamentData
                                                        .yourContactNumber,
                                                    score: flyyTournamentData
                                                                .yourScore !=
                                                            null
                                                        ? flyyTournamentData
                                                            .yourScore
                                                            .toDouble()
                                                        : 0,
                                                    rank: flyyTournamentData
                                                        .yourRank,
                                                    prize: flyyTournamentData
                                                        .yourPrize,
                                                    prizeType:
                                                        flyyTournamentData
                                                            .yourPrizeType);
                                            if (leaderboardList != null)
                                              leaderboardList.insert(
                                                  0, flyyLeaderBoard);

                                            showDialog(
                                                context: context,
                                                builder: (BuildContext
                                                        context) =>
                                                    FlyyGamesLeaderBoardDialog(
                                                      flyyLeaderboardList:
                                                          leaderboardList,
                                                      flyyTournamentData:
                                                          flyyTournamentData,
                                                      shareText: flyyTournament
                                                          .shareText,
                                                    ));
                                          },
                                          child: Text(
                                            rank,
                                            style: TextStyle(
                                                fontSize: 16,
                                                decoration:
                                                    TextDecoration.underline,
                                                color: FlyyUtility.parseColor(
                                                    "#ff669900")),
                                          ),
                                        ),
                                      ],
                                    ),
                                  )
                                : SizedBox.shrink()
                          ],
                        ),
                      ),
                    ),

                    ///winners text layout
                    Container(
                      margin: const EdgeInsets.only(top: 5, bottom: 5),
                      child: FutureBuilder<FlyyWinText>(
                        future: getWinTexts(page, flyyTournamentData.gameId),
                        builder:
                            (BuildContext context, AsyncSnapshot response) {
                          if (response.hasData && response.data != null) {
                            return setWinTexts(response.data);
                          } else if (response.hasError) {
                            return SizedBox.shrink();
                          }
                          return SizedBox.shrink();
                        },
                      ),
                    ),

                    ///prizes
                    Container(
                      margin: const EdgeInsets.all(5),
                      child: Card(
                        shape: RoundedRectangleBorder(
                          borderRadius: BorderRadius.circular(10),
                        ),
                        child: Column(
                          children: <Widget>[
                            Container(
                              width: double.maxFinite,
                              alignment: Alignment.center,
                              margin: const EdgeInsets.only(bottom: 10),
                              decoration: BoxDecoration(
                                  color: FlyyUtility.parseColor("#EEEEEE"),
                                  borderRadius: BorderRadius.only(
                                      topLeft: Radius.circular(10.0),
                                      topRight: Radius.circular(10.0))),
                              padding:
                                  const EdgeInsets.only(top: 10, bottom: 10),
                              child: Text("PRIZES",
                                  style: TextStyle(
                                      fontSize: 14,
                                      color:
                                          FlyyUtility.parseColor("#323232"))),
                            ),
                            Padding(
                              padding: const EdgeInsets.only(bottom: 10),
                              child: Column(
                                mainAxisAlignment: MainAxisAlignment.center,
                                children: <Widget>[
                                  Row(
                                    mainAxisAlignment: MainAxisAlignment.center,
                                    children: <Widget>[
                                      Image.network(
                                        "https://ghc.theflyy.com/assets/fs_assets/trophy.png",
                                        height: 20,
                                        width: 20,
                                      ),
                                      Padding(
                                        padding: const EdgeInsets.only(left: 5),
                                        child: Text(
                                          winnerPercent != null
                                              ? winnerPercent.toString()
                                              : "0" + "% Winners",
                                          style: TextStyle(
                                              fontSize: 14,
                                              color: FlyyUtility.parseColor(
                                                  "#323232")),
                                        ),
                                      )
                                    ],
                                  ),
                                  prizesList != null && prizesList.length > 0
                                      ? ListView.builder(
                                          physics:
                                              NeverScrollableScrollPhysics(),
                                          itemBuilder: setPrizesList,
                                          itemCount: prizesList.length,
                                          scrollDirection: Axis.vertical,
                                          shrinkWrap: true,
                                        )
                                      : SizedBox.shrink(),
                                ],
                              ),
                            ),
                          ],
                        ),
                      ),
                    ),

                    ///rules
                    Container(
                      margin: const EdgeInsets.all(5),
                      child: Card(
                        shape: RoundedRectangleBorder(
                          borderRadius: BorderRadius.circular(10),
                        ),
                        child: Column(
                          children: <Widget>[
                            Container(
                              decoration: BoxDecoration(
                                  color: FlyyUtility.parseColor("#EEEEEE"),
                                  borderRadius: BorderRadius.only(
                                      topLeft: Radius.circular(10.0),
                                      topRight: Radius.circular(10.0))),
                              width: double.maxFinite,
                              alignment: Alignment.center,
                              margin: const EdgeInsets.only(bottom: 10),
                              padding:
                                  const EdgeInsets.only(top: 10, bottom: 10),
                              child: Text("RULES",
                                  style: TextStyle(
                                      fontSize: 14,
                                      color:
                                          FlyyUtility.parseColor("#323232"))),
                            ),
                            Column(
                              mainAxisAlignment: MainAxisAlignment.center,
                              children: <Widget>[
                                Container(
                                    margin: const EdgeInsets.only(
                                        left: 10, right: 50),
                                    child: Html(
                                        data: rules != null && rules.isNotEmpty
                                            ? rules
                                            : "",
                                        defaultTextStyle: TextStyle(
                                            fontSize: 12,
                                            color: FlyyUtility.parseColor(
                                                "#323232")))),
                              ],
                            ),
                          ],
                        ),
                      ),
                    ),
                  ],
                ),
              )
            ],
          ),
        ),
      ),
    );
  }

  Future<FlyyWinText> getWinTexts(int page, int gameId) async {
    APIClient apiClient = new APIClient();
    final response = await apiClient.getClient("wins_text?partner=" +
        Flyy.PARTNER_TOKEN +
        "&page=" +
        page.toString() +
        "&game_id=" +
        gameId.toString());
    if (response.statusCode == 200) {
      final responseJson = json.decode(response.body);

      return FlyyWinText.fromJson(responseJson);
    } else {
      throw Exception('Failed to load data ' + response.toString());
    }
  }

  Widget setWinTexts(FlyyWinText flyyWinText) {
    List<String> winTextsList = new List();
    var concatenate = StringBuffer();
    if (flyyWinText.success) {
      if (flyyWinText.texts.length > 0) {
        if (page == 1) {
          winTextsList.addAll(flyyWinText.texts);
          winTextsList.forEach((item) {
            concatenate.write(item + "\t\t\t");
          });

          return Container(
            height: 50,
            width: double.maxFinite,
            color: FlyyUtility.parseColor("#5E1010"),
            child: ScrollingText(
              text: concatenate.toString(),
              textStyle: TextStyle(
                  fontSize: 12, color: Colors.white, letterSpacing: 1),
            ),
          );
        } else {
          winTextsList.addAll(flyyWinText.texts);
          return Container(
            color: FlyyUtility.parseColor("#5E1010"),
            child: ScrollingText(
              text: concatenate.toString() + " ",
              textStyle: TextStyle(fontSize: 12, color: Colors.white),
            ),
          );
        }
      } else {
        return SizedBox.shrink();
      }
    } else {
      return SizedBox.shrink();
    }
  }

  Widget setPrizesList(BuildContext context, int index) {
    FlyyTournamentPrize flyyTournamentPrize = flyyTournamentData.prizes[index];
    return Container(
        padding: const EdgeInsets.only(top: 5, bottom: 5, left: 20, right: 20),
        child: Row(
          mainAxisAlignment: MainAxisAlignment.spaceBetween,
          children: <Widget>[
            Text(
              "Rank: " + setRankText(flyyTournamentPrize),
              style: TextStyle(
                  fontSize: 12, color: FlyyUtility.parseColor("#323232")),
            ),
            Text(
              flyyTournamentPrize.prize.toString() +
                  " " +
                  flyyTournamentPrize.prizeType,
              style: TextStyle(
                  fontSize: 12, color: FlyyUtility.parseColor("#323232")),
            )
          ],
        ));
  }

  String setRankText(FlyyTournamentPrize flyyTournamentPrize) {
    if (flyyTournamentPrize.fromRank == flyyTournamentPrize.toRank)
      return flyyTournamentPrize.toRank.toString();
    else
      return flyyTournamentPrize.fromRank.toString() +
          " to " +
          flyyTournamentPrize.toRank.toString();
  }

  Widget setStartEndtext(String startsAt, String endsAt, bool live) {
    DateTime startDate = new DateTime.now();
    DateTime endDate = DateTime.parse(endsAt);
    int timeDiff =
        endDate.millisecondsSinceEpoch - startDate.millisecondsSinceEpoch;
    if (live) {
      if (timeDiff > 0) {
        return Column(
          children: <Widget>[
            ///play game button
            GestureDetector(
              onTap: () {
                if (flyyTournamentData.isLive != null &&
                    flyyTournamentData.isLive) {
                  redirectToGame(flyyTournamentData.id.toString());
                } else {
                  FlyyUtility.showToast(
                      context, "Tournament has ended", ToastGravity.BOTTOM);
                }
              },
              child: Container(
                height: 70,
                width: double.maxFinite,
                margin: const EdgeInsets.only(right: 10, left: 10),
                padding: const EdgeInsets.only(top: 10, bottom: 10),
                child: Card(
                  shape: RoundedRectangleBorder(
                    borderRadius: BorderRadius.circular(5),
                  ),
                  color: Flyy.TOOLBAR_COLOR,
                  child: Center(
                    child: Text(
                      'Play',
                      style: new TextStyle(fontSize: 16, color: Colors.white),
                    ),
                  ),
                ),
              ),
            ),
            Row(
              mainAxisAlignment: MainAxisAlignment.center,
              children: <Widget>[
                Text(
                    live != null && live
                        ? "Game Ends in :"
                        : "Game Starts in :",
                    style: TextStyle(
                        fontSize: 13,
                        color: FlyyUtility.parseColor("#323232"))),
                setCountDownTimerText(endDate.millisecondsSinceEpoch)
              ],
            ),
          ],
        );
      }
    } else {
      if (timeDiff > 0) {
        return Column(
          children: <Widget>[
            ///play game button
            Container(
              height: 70,
              width: double.maxFinite,
              margin: const EdgeInsets.only(right: 10, left: 10),
              padding: const EdgeInsets.only(top: 10, bottom: 10),
              child: Card(
                shape: RoundedRectangleBorder(
                  borderRadius: BorderRadius.circular(5),
                ),
                color: FlyyUtility.parseColor("#dadada"),
                child: Center(
                  child: Text(
                    'Play',
                    style: new TextStyle(fontSize: 16, color: Colors.white),
                  ),
                ),
              ),
            ),
            Row(
              mainAxisAlignment: MainAxisAlignment.center,
              children: <Widget>[
                Text(
                    live != null && live
                        ? "Game Ends in :"
                        : "Game Starts in :",
                    style: TextStyle(
                        fontSize: 13,
                        color: FlyyUtility.parseColor("#323232"))),
                setCountDownTimerText(endDate.millisecondsSinceEpoch)
              ],
            ),
          ],
        );
      }
    }
  }

  Widget setCountDownTimerText(int millisInFuture) {
    return CustomCountdownTimer(
      onEnd: () {
        getTournamentData(flyyTournamentData.gameId);
      },
      endTime: millisInFuture,
      showDays: true,
      daysSymbol: "d:",
      hoursSymbol: "h:",
      minSymbol: "m:",
      secSymbol: "s",
      textStyle:
          TextStyle(fontSize: 13, color: FlyyUtility.parseColor("#ffcc0000")),
    );
  }

  Future<FlyyPlayGame> redirectToGame(String tournamentId) async {
    showAlertDialog(context);
    APIClient apiClient = new APIClient();
    final response = await apiClient.getClient("play_tournament?partner=" +
        Flyy.PARTNER_TOKEN +
        "&tournament_id=" +
        tournamentId +
        "&contact_number=" +
        Flyy.EXT_USER_ID);
    if (response.statusCode == 200) {
      final responseJson = json.decode(response.body);
      FlyyPlayGame flyyPlayGame = FlyyPlayGame.fromJson(responseJson);

      Navigator.of(context, rootNavigator: true).pop();

      if (flyyPlayGame.success != null && flyyPlayGame.success) {
        Navigator.push(
          context,
          MaterialPageRoute(
              builder: (context) => FlyyPlayGamesPage(
                    flyyPlayGame: flyyPlayGame,
                  )),
        );
      } else {
        FlyyUtility.showToast(
            context, flyyPlayGame.message, ToastGravity.CENTER);
      }

      return flyyPlayGame;
    } else {
      throw Exception('Failed to load data ' + response.toString());
    }
  }

  Widget showAlertDialog(BuildContext context) {
    showDialog(
      barrierDismissible: false,
      useRootNavigator: true,
      context: context,
      builder: (BuildContext context) {
        return AlertDialog(
          content: new Row(
            children: [
              CircularProgressIndicator(),
              Container(
                  margin: EdgeInsets.only(left: 5),
                  child: Text("Loading Game...")),
            ],
          ),
        );
      },
    );
  }

  Future<FlyyTournament> fetchLiveTournamentData(int tournamentId) async {
    APIClient apiClient = new APIClient();
    final response = await apiClient.getClient("v1/live_tournament?partner=" +
        Flyy.PARTNER_TOKEN +
        "&tr_id=" +
        tournamentId.toString() +
        "&contact_number=" +
        Flyy.EXT_USER_ID);
    if (response.statusCode == 200) {
      final responseJson = json.decode(response.body);
      flyyTournament = FlyyTournament.fromJson(responseJson);

      if (flyyTournament.success) {
        setState(() {
          if (flyyTournament != null && flyyTournament.tournament != null) {
            flyyTournamentData = flyyTournament.tournament;
            if (flyyTournamentData.joined) {
              preloadData(flyyTournament, flyyTournamentData);
              if (flyyTournamentData.livePlayers != null) {
                playersJoinedText = flyyTournamentData.livePlayers.toString() +
                    "/" +
                    flyyTournamentData.maxPlayers.toString();
              } else {
                playersJoinedText =
                    "0" + "/" + flyyTournamentData.maxPlayers.toString();
              }
              rank = "#" + flyyTournamentData.yourRank.toString();
              rules = flyyTournamentData.rules;
              prizesList = flyyTournamentData.prizes;
              if (flyyTournamentData.winnerPercent != null)
                winnerPercent =
                    flyyTournamentData.winnerPercent.toString() + "% Winners";
            }

            leaderboardList = flyyTournamentData.leaderboardData;
            FlyyLeaderboard flyyLeaderBoard = new FlyyLeaderboard(
                name: flyyTournamentData.yourName,
                mobile: flyyTournamentData.yourContactNumber,
                score: flyyTournamentData.yourScore.toDouble(),
                rank: flyyTournamentData.yourRank,
                prize: flyyTournamentData.yourPrize,
                prizeType: flyyTournamentData.yourPrizeType);
            leaderboardList.insert(0, flyyLeaderBoard);

            if (flyyTournamentData.playsLeft != null &&
                flyyTournamentData.playsLeft >= 0) {
              chancesLeft =
                  flyyTournamentData.playsLeft.toString() + " Chance(s) Left";
            }
          }
        });
      }

      return flyyTournament;
    } else {
      throw Exception('Failed to load data ' + response.toString());
    }
  }

  Future<FlyyTournament> getTournamentData(int gameId) async {
    APIClient apiClient = new APIClient();
    final response = await apiClient.getClient("v1/web_games?partner=" +
        Flyy.PARTNER_TOKEN +
        "&game_id=" +
        gameId.toString() +
        "&user_name=" +
        "" +
        "&contact_number=" +
        Flyy.EXT_USER_ID);
    if (response.statusCode == 200) {
      final responseJson = json.decode(response.body);
      flyyTournament = FlyyTournament.fromJson(responseJson);

      if (flyyTournament.success) {
        if (flyyTournament != null && flyyTournament.tournament != null) {
          setState(() {
            flyyTournamentData = flyyTournament.tournament;
          });
        }
      }

      return flyyTournament;
    } else {
      throw Exception('Failed to load data ' + response.toString());
    }
  }
}
