import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:flutter_share_me/flutter_share_me.dart';
import 'package:flyyfluttersdk/Flyy.dart';
import 'package:flyyfluttersdk/helpers/FlyyUtility.dart';
import 'package:flyyfluttersdk/model/FlyyLeaderboard.dart';
import 'package:flyyfluttersdk/model/FlyyTournamentData.dart';

class FlyyGamesLeaderBoardDialog extends StatelessWidget {
  List<FlyyLeaderboard> flyyLeaderboardList;
  FlyyTournamentData flyyTournamentData;
  String shareText;
  List<TableRow> tableRows = [];

  FlyyGamesLeaderBoardDialog(
      {Key key,
      @required this.flyyLeaderboardList,
      this.flyyTournamentData,
      this.shareText})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Dialog(
      shape: RoundedRectangleBorder(
        borderRadius: BorderRadius.circular(5),
      ),
      backgroundColor: Colors.white,
      insetPadding: EdgeInsets.all(40),
      child: dialogContent(context),
    );
  }

  dialogContent(BuildContext context) {
    return Container(
      child: SingleChildScrollView(
        child: Column(
          crossAxisAlignment: CrossAxisAlignment.center,
          children: <Widget>[
            ///game leaderboard title
            Padding(
              padding: const EdgeInsets.only(top: 10),
              child: Text(
                "Game Leaderboard",
                style: TextStyle(
                    fontSize: 14,
                    fontWeight: FontWeight.bold,
                    color: FlyyUtility.parseColor("#323232")),
              ),
            ),

            ///divider
            Container(
                margin: const EdgeInsets.only(top: 5, bottom: 5),
                width: double.maxFinite,
                child: Divider(
                  thickness: 2,
                  color: FlyyUtility.parseColor("#D3D3D3"),
                )),

            ///Table container with all the tablerows
            Container(
              margin: const EdgeInsets.only(left: 5, right: 5),
              padding: const EdgeInsets.only(right: 8, left: 8),
              child: Column(
                mainAxisAlignment: MainAxisAlignment.center,
                crossAxisAlignment: CrossAxisAlignment.center,
                children: <Widget>[
                  Table(
                    columnWidths: {0: FractionColumnWidth(.2)},
                    border: TableBorder(
                        horizontalInside: BorderSide(
                            width: 1,
                            color: FlyyUtility.parseColor("#D3D3D3"))),
                    children: leaderBoardTableRows(flyyLeaderboardList),
                    // other stuff
                  ),
                ],
              ),
            ),

            GestureDetector(
              onTap: () async {
                if (shareText != null && shareText.isNotEmpty) {
                  if (shareText.contains("{score}")) {
                    shareText.replaceAll(
                        "{score}",
                        flyyTournamentData.yourScore.toString() +
                            " " +
                            flyyTournamentData.yourPrizeType);
                  } else {
                    shareText = flyyTournamentData.yourScore.toString() +
                        " " +
                        flyyTournamentData.yourPrizeType;
                  }
                } else {
                  shareText = flyyTournamentData.yourScore.toString() +
                      " " +
                      flyyTournamentData.yourPrizeType;
                }
                await FlutterShareMe().shareToWhatsApp(msg: shareText);
              },
              child: Container(
                alignment: Alignment.center,
                height: 50,
                width: double.maxFinite,
                margin: const EdgeInsets.only(
                    top: 10, bottom: 15, left: 20, right: 20),
                child: Card(
                  color: Flyy.TOOLBAR_COLOR,
                  child: Row(
                    mainAxisAlignment: MainAxisAlignment.center,
                    children: <Widget>[
                      Text(
                        "Share your rank",
                        style: new TextStyle(fontSize: 14, color: Colors.white),
                      ),
                      Padding(
                        padding: const EdgeInsets.all(10),
                        child: Image.network(
                          "https://ghc.theflyy.com/assets/fs_assets/game_share.png",
                          height: 20,
                          width: 20,
                        ),
                      )
                    ],
                  ),
                ),
              ),
            ),
          ],
        ),
      ),
    );
  }

  List<TableRow> leaderBoardTableRows(
      List<FlyyLeaderboard> flyyLeaderboardList) {
    Color containerBackGroundColor, textColor;

    List<TableRow> rows = [];
    TableRow row;
    for (int i = 0; i < flyyLeaderboardList.length; ++i) {
      if (i == 0) {
        containerBackGroundColor = Flyy.TOOLBAR_COLOR;
        textColor = Colors.white;
      } else {
        containerBackGroundColor = Colors.white;
        textColor = FlyyUtility.parseColor("#323232");
      }

      FlyyLeaderboard flyyLeaderboard = flyyLeaderboardList[i];
      rows.add(TableRow(
          decoration: BoxDecoration(color: containerBackGroundColor),
          children: [
            Center(
              child: Padding(
                padding: const EdgeInsets.only(top: 10, bottom: 10),
                child: Text(
                  flyyLeaderboard.rank.toString(),
                  style: TextStyle(fontSize: 13, color: textColor),
                ),
              ),
            ),
            Center(
              child: Container(
                width: 50,
                padding: const EdgeInsets.only(top: 10, bottom: 10),
                child: Text(
                  flyyLeaderboard.name != null ? flyyLeaderboard.name : "",
                  overflow: TextOverflow.ellipsis,
                  maxLines: 1,
                  style: TextStyle(fontSize: 13, color: textColor),
                ),
              ),
            ),
            Center(
              child: Padding(
                padding: const EdgeInsets.only(top: 10, bottom: 10),
                child: Text(
                  flyyLeaderboard.score.toInt().toString(),
                  style: TextStyle(fontSize: 13, color: textColor),
                ),
              ),
            ),
            Center(
              child: Padding(
                padding: const EdgeInsets.only(top: 10, bottom: 10),
                child: Text(
                  flyyLeaderboard.prize != null
                      ? flyyLeaderboard.prize.toString()
                      : "0" + " " + flyyLeaderboard.prizeType != null
                          ? flyyLeaderboard.prizeType
                          : "",
                  style: TextStyle(fontSize: 13, color: textColor),
                ),
              ),
            ),
          ]));
    }

    row = TableRow(children: [
      Center(
        child: Padding(
          padding: const EdgeInsets.only(top: 10, bottom: 10),
          child: Text(
            "Rank",
            style: TextStyle(
                fontSize: 13, color: FlyyUtility.parseColor("#323232")),
          ),
        ),
      ),
      Center(
        child: Padding(
          padding: const EdgeInsets.only(top: 10, bottom: 10),
          child: Text(
            "Player",
            style: TextStyle(
                fontSize: 13, color: FlyyUtility.parseColor("#323232")),
          ),
        ),
      ),
      Center(
        child: Padding(
          padding: const EdgeInsets.only(top: 10, bottom: 10),
          child: Text(
            "Score",
            style: TextStyle(
                fontSize: 13, color: FlyyUtility.parseColor("#323232")),
          ),
        ),
      ),
      Center(
        child: Padding(
          padding: const EdgeInsets.only(top: 10, bottom: 10),
          child: Text(
            "Prize",
            style: TextStyle(
                fontSize: 13, color: FlyyUtility.parseColor("#323232")),
          ),
        ),
      ),
    ]);

    rows.insert(0, row);
    return rows;
  }
}
