import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_linkify/flutter_linkify.dart';
import 'package:fluttertoast/fluttertoast.dart';
import 'package:flyyfluttersdk/Flyy.dart';
import 'package:flyyfluttersdk/helpers/FlyyCommonColors.dart';
import 'package:flyyfluttersdk/helpers/FlyyUtility.dart';
import 'package:flyyfluttersdk/model/FlyyGiftCardData.dart';
import 'package:flyyfluttersdk/model/FlyyWalletCompleteData.dart';
import 'package:flyyfluttersdk/model/FlyyWalletModel.dart';
import 'package:flyyfluttersdk/views/FlyyBuyGiftCardDialog.dart';
import 'package:url_launcher/url_launcher.dart';

class FlyyGiftCardDetailsPage extends StatelessWidget {
  FlyyGiftCardData flyyGiftCardData;
  FlyyWalletCompleteData flyyWalletCompleteData;
  FlyyWalletModel flyyWalletModel;

  FlyyGiftCardDetailsPage(
      {Key key,
      @required this.flyyGiftCardData,
      this.flyyWalletCompleteData,
      this.flyyWalletModel})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return FlyyGiftCardsDetailsStateFulWidget(
      flyyGiftCardData: flyyGiftCardData,
      flyyWalletCompleteData: flyyWalletCompleteData,
      flyyWalletModel: flyyWalletModel,
    );
  }
}

///Stateful widget
class FlyyGiftCardsDetailsStateFulWidget extends StatefulWidget {
  FlyyGiftCardData flyyGiftCardData;

  FlyyWalletCompleteData flyyWalletCompleteData;
  FlyyWalletModel flyyWalletModel;

  FlyyGiftCardsDetailsStateFulWidget(
      {Key key,
      @required this.flyyGiftCardData,
      this.flyyWalletCompleteData,
      this.flyyWalletModel})
      : super(key: key);

  @override
  State<StatefulWidget> createState() {
    return new FlyyGiftCardsDetailsState();
  }
}

class FlyyGiftCardsDetailsState
    extends State<FlyyGiftCardsDetailsStateFulWidget> {
  String activeCurrencyLabel, activeCurrencyIconUrl;
  int activeCurrencyBalance;
  FlyyUtility flyyUtility;

  @override
  void initState() {
    super.initState();
    flyyUtility = new FlyyUtility();
    getBalanceValues();
  }

  getBalanceValues() {
    flyyUtility.getActiveCurrencyBalance().then((balance) {
      setState(() {
        activeCurrencyBalance = balance;
      });
    });
    flyyUtility.getActiveCurrencyLabel().then((label) {
      setState(() {
        activeCurrencyLabel = label;
      });
    });
    flyyUtility.getActiveCurrencyIconUrl().then((iconUrl) {
      setState(() {
        activeCurrencyIconUrl = iconUrl;
      });
    });
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        bottomOpacity: 0.0,
        elevation: 0.0,
        leading: IconButton(
          icon: Icon(Icons.arrow_back, color: Colors.white),
          onPressed: () {
            if (Navigator.canPop(context)) {
              Navigator.of(context).pop();
            } else {
              SystemNavigator.pop();
            }
          },
        ),
        title: Text('Card Details'),
        actions: <Widget>[
          Container(
            height: 20,
            margin: const EdgeInsets.only(right: 10),
            padding: const EdgeInsets.all(5),
            child: Card(
              shape: RoundedRectangleBorder(
                borderRadius: BorderRadius.circular(15),
              ),
              color: Colors.white,
              child: Padding(
                padding: const EdgeInsets.all(5),
                child: Row(
                  mainAxisAlignment: MainAxisAlignment.center,
                  children: <Widget>[
                    activeCurrencyIconUrl != null &&
                            activeCurrencyIconUrl.isNotEmpty
                        ? Image.network(
                            activeCurrencyIconUrl,
                            height: 20,
                            width: 20,
                          )
                        : SizedBox.shrink(),
                    Padding(
                      padding: const EdgeInsets.only(left: 5),
                      child: Text(
                        flyyUtility.setRewardText(
                            activeCurrencyBalance, activeCurrencyLabel),
                        style: TextStyle(fontSize: 12),
                      ),
                    ),
                  ],
                ),
              ),
            ),
          ),
        ],
        backgroundColor: Flyy.TOOLBAR_COLOR,
      ),
      body: setGiftCardDetailsPage(),
    );
  }

  Widget setGiftCardDetailsPage() {
    return Container(
      color: FlyyCommonColors.BG_GREY,
      height: double.maxFinite,
      child: SingleChildScrollView(
          child: Container(
        margin: const EdgeInsets.all(10),
        child: Card(
            child: Padding(
          padding: const EdgeInsets.all(10),
          child: Column(
            children: <Widget>[
              ///Card title name
              Text(
                widget.flyyGiftCardData.name,
                style: TextStyle(
                    fontSize: 16, color: FlyyUtility.parseColor('#373535')),
              ),

              ///gift card image
              Padding(
                padding: const EdgeInsets.only(top: 10, bottom: 10),
                child: Image.network(
                  widget.flyyGiftCardData.imageUrl,
                  height: 50,
                  width: 100,
                ),
              ),

              ///card value
              Padding(
                padding: const EdgeInsets.only(bottom: 20),
                child: Text(
                  'Value : ₹' + widget.flyyGiftCardData.value,
                  style: TextStyle(
                      fontSize: 14, color: FlyyUtility.parseColor('#373535')),
                ),
              ),

              ///validty and use layout
              Row(
                mainAxisAlignment: MainAxisAlignment.spaceBetween,
                children: <Widget>[
                  ///validity layout
                  Row(
                    children: <Widget>[
                      Text(
                        'Validity : ',
                        style: TextStyle(
                            fontSize: 14,
                            color: FlyyUtility.parseColor('#9A9A9A')),
                      ),
                      Text(
                        widget.flyyGiftCardData.expiry != null
                            ? widget.flyyGiftCardData.expiry
                            : '1 Year',
                        style: TextStyle(
                            fontSize: 14,
                            color: FlyyUtility.parseColor('#373535')),
                      ),
                    ],
                  ),

                  Row(
                    children: <Widget>[
                      ///use layout
                      Text(
                        'Use: ',
                        style: TextStyle(
                            fontSize: 14,
                            color: FlyyUtility.parseColor('#9A9A9A')),
                      ),

                      ///rewards, coins, cash layout
                      GestureDetector(
                        onTap: () {
                          if (widget.flyyGiftCardData.currencyReq <=
                              activeCurrencyBalance) {
                            showDialog(
                                context: context,
                                builder: (BuildContext context) =>
                                    FlyyBuyGiftCardDialog(
                                      flyyGiftCardData: widget.flyyGiftCardData,
                                      flyyWalletModel: widget.flyyWalletModel,
                                      flyyWalletCompleteData:
                                          widget.flyyWalletCompleteData,
                                    )).then((value) {
                              setState(() {
                                getBalanceValues();
                              });
                            });
                          } else {
                            FlyyUtility.showToast(context, "Not enough balance",
                                ToastGravity.CENTER);
                          }
                        },
                        child: Container(
                          decoration: BoxDecoration(
                            border: Border.all(
                              color: Flyy.TOOLBAR_COLOR,
                            ),
                            borderRadius: BorderRadius.circular(5.0),
                          ),
                          child: Padding(
                            padding: const EdgeInsets.all(5),
                            child: Center(
                              child: Row(
                                children: <Widget>[
                                  activeCurrencyIconUrl != null &&
                                          activeCurrencyIconUrl.isNotEmpty
                                      ? Image.network(
                                          activeCurrencyIconUrl,
                                          height: 20,
                                          width: 20,
                                        )
                                      : SizedBox.shrink(),
                                  Padding(
                                    padding: const EdgeInsets.only(left: 5),
                                    child: Text(
                                      flyyUtility.setRewardText(
                                          widget.flyyGiftCardData.currencyReq,
                                          ""
                                          /* widget
                                              .flyyGiftCardData.currencyLabel*/
                                          ),
                                      textAlign: TextAlign.center,
                                      style: new TextStyle(
                                          fontSize: 10,
                                          color: FlyyUtility.parseColor(
                                              "#373535")),
                                    ),
                                  ),
                                ],
                              ),
                            ),
                          ),
                        ),
                      ),
                    ],
                  )
                ],
              ),

              ///Divider
              Container(
                width: double.infinity,
                height: 1,
                margin: const EdgeInsets.only(
                    top: 20, bottom: 15, left: 10, right: 10),
                child: Divider(
                  color: FlyyUtility.parseColor('#9A9A9A'),
                ),
              ),

              Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                children: <Widget>[
                  ///About title
                  Padding(
                    padding: const EdgeInsets.only(bottom: 10),
                    child: Text(
                      'About',
                      style: TextStyle(
                          fontSize: 14,
                          color: FlyyUtility.parseColor('#373535')),
                    ),
                  ),

                  ///about description
                  Padding(
                    padding: const EdgeInsets.only(bottom: 10),
                    child: Text(
                      FlyyUtility.cleanHtmlStrings(
                          widget.flyyGiftCardData.description),
                      style: TextStyle(
                          fontSize: 12,
                          color: FlyyUtility.parseColor('#373535')),
                    ),
                  ),

                  ///terms and condition title
                  Padding(
                    padding: const EdgeInsets.only(top: 10, bottom: 8),
                    child: Text(
                      'Terms & Conditions',
                      style: TextStyle(
                          fontSize: 14,
                          color: FlyyUtility.parseColor('#373535')),
                    ),
                  ),

                  ///terms and condition
                  Padding(
                    padding: const EdgeInsets.only(bottom: 5),
                    child: Text(
                      FlyyUtility.cleanHtmlStrings(
                          widget.flyyGiftCardData.tncContent),
                      style: TextStyle(
                          fontSize: 12,
                          color: FlyyUtility.parseColor('#373535')),
                    ),
                  ),

                  Linkify(
                    onOpen: (link) {
                      launchURL(link.url);
                    },
                    text: widget.flyyGiftCardData.tncLink,
                    style: TextStyle(color: Colors.blue),
                    linkStyle: TextStyle(color: Colors.blue, fontSize: 12),
                  ),
                ],
              )
            ],
          ),
        )),
      )),
    );
  }

  launchURL(String url) async {
    if (await canLaunch(url)) {
      await launch(url);
    } else {
      throw 'Could not launch $url';
    }
  }
}
