import 'dart:convert';
import 'dart:ui';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_html/flutter_html.dart';
import 'package:flutter_share_me/flutter_share_me.dart';
import 'package:flutter_svg/flutter_svg.dart';
import 'package:fluttertoast/fluttertoast.dart';
import 'package:flyyfluttersdk/Flyy.dart';
import 'package:flyyfluttersdk/helpers/APIClient.dart';
import 'package:flyyfluttersdk/helpers/FlyyCommonColors.dart';
import 'package:flyyfluttersdk/helpers/FlyyUtility.dart';
import 'package:flyyfluttersdk/helpers/ReusableAppbarWidget.dart';
import 'package:flyyfluttersdk/model/FlyyOfferPrizes.dart';
import 'package:flyyfluttersdk/model/FlyyOffers.dart';
import 'package:flyyfluttersdk/model/FlyyShareResponse.dart';

class FlyyInviteAndEarnPage extends StatelessWidget {
  FlyyOffers flyyOffers;

  FlyyInviteAndEarnPage({Key key, @required this.flyyOffers}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Scaffold(
//      appBar: ReusableAppbarWidget.getAppBar(context, FlyyUtility.cleanHtmlStrings(flyyOffers.offerTypeTitle)),
      appBar: ReusableAppbarWidget.getAppBar(context, "Invite & Earn"),
      body: FlyyInviteAndEarnStateFulWidget(
        flyyOffers: flyyOffers,
      ),
    );
  }
}

class FlyyInviteAndEarnStateFulWidget extends StatefulWidget {
  FlyyOffers flyyOffers;

  FlyyInviteAndEarnStateFulWidget({Key key, @required this.flyyOffers})
      : super(key: key);

  @override
  State<StatefulWidget> createState() {
    return new FlyyInviteAndEarnState();
  }
}

class FlyyInviteAndEarnState extends State<FlyyInviteAndEarnStateFulWidget> {
  FlyyUtility flyyUtility;

  @override
  Widget build(BuildContext context) {
    flyyUtility = new FlyyUtility();
    return Scaffold(
      bottomSheet: Container(
        height: 80,
        margin: const EdgeInsets.only(top: 10),
        alignment: Alignment.bottomCenter,
        child: Column(
          children: <Widget>[
            Padding(
              padding: const EdgeInsets.only(bottom: 5, top: 5),
              child: Text(
                'Invite now',
                style: new TextStyle(
                    fontSize: 16, color: FlyyUtility.parseColor("#373535")),
              ),
            ),
            Row(
              mainAxisSize: MainAxisSize.max,
              mainAxisAlignment: MainAxisAlignment.center,
              children: <Widget>[
                IconButton(
                  icon: Image.network(
                      "https://ghc.theflyy.com/assets/fs_assets/ic_whatsapp.png"),
                  onPressed: () async {
                    fetchShareData(context, widget.flyyOffers.id, 1);
                  },
                ),
                IconButton(
                  icon: Image.network(
                      "https://ghc.theflyy.com/assets/fs_assets/ic_facebook.png"),
                  onPressed: () async {
                    fetchShareData(context, widget.flyyOffers.id, 2);
                  },
                ),
                IconButton(
                  icon: Image.network(
                      "https://ghc.theflyy.com/assets/fs_assets/ic_instagram.png"),
                  onPressed: () async {
                    fetchShareData(context, widget.flyyOffers.id, 3);
                  },
                ),
                IconButton(
                  icon: Image.network(
                    "https://ghc.theflyy.com/assets/fs_assets/ic_more.png",
                    color: Flyy.TOOLBAR_COLOR,
                    colorBlendMode: BlendMode.overlay,
                  ),
                  onPressed: () async {
                    fetchShareData(context, widget.flyyOffers.id, 3);
                  },
                ),
              ],
            ),
          ],
        ),
      ),
      body: Container(
        color: FlyyCommonColors.BG_GREY,
        height: double.maxFinite,
        padding: const EdgeInsets.only(bottom: 80),
        child: SingleChildScrollView(
          child: Center(
            child: Container(
              margin:
                  const EdgeInsets.only(top: 5, right: 5, left: 5, bottom: 20),
              child: Card(
                child: Column(
                  crossAxisAlignment: CrossAxisAlignment.stretch,
                  children: <Widget>[
                    ///Invite and earn icon
                    Container(
                      margin: const EdgeInsets.only(top: 10, bottom: 15),
                      child: SvgPicture.network(
                        "https://ghc.theflyy.com/assets/fs_assets/ic_invite.svg",
                        height: 35,
                        width: 35,
                        alignment: Alignment.center,
                        color: Flyy.TOOLBAR_COLOR,
                      ),
                    ),

                    ///Invite and earn title
                    Container(
                      margin: const EdgeInsets.only(bottom: 25),
                      child: Text(
                        FlyyUtility.cleanHtmlStrings(widget.flyyOffers.title),
                        textAlign: TextAlign.center,
                        style: new TextStyle(
                            fontSize: 16,
                            color: FlyyUtility.parseColor("#373535")),
                      ),
                    ),

                    ///invite and earn rewards if its regular offer
                    widget.flyyOffers.offerTypeCode.contains('gamification')
                        ? SizedBox.shrink()
                        : Container(
                            margin: const EdgeInsets.only(bottom: 10),
                            child: Column(
                              children: <Widget>[
                                Row(
                                  mainAxisAlignment: MainAxisAlignment.center,
                                  children: <Widget>[
                                    widget.flyyOffers.currencyIconUrl != null &&
                                            widget.flyyOffers.currencyIconUrl
                                                .isNotEmpty
                                        ? Image.network(
                                            widget.flyyOffers.currencyIconUrl,
                                            height: 25,
                                            width: 25,
                                          )
                                        : SizedBox.shrink(),
                                    Padding(
                                      padding: const EdgeInsets.only(left: 5),
                                      child: Text(
                                        setRewardText(
                                            context, widget.flyyOffers),
                                        style: new TextStyle(
                                            fontSize: 14,
                                            color: FlyyUtility.parseColor(
                                                "#585957")),
                                      ),
                                    ),
                                  ],
                                ),
                              ],
                            ),
                          ),

                    ///Description
                    Container(
                      margin: const EdgeInsets.only(
                          bottom: 25, left: 80, right: 80),
                      child: Text(
                        FlyyUtility.cleanHtmlStrings(
                            widget.flyyOffers.description),
                        textAlign: TextAlign.center,
                        style: new TextStyle(
                            fontSize: 14,
                            color: FlyyUtility.parseColor("#585957")),
                      ),
                    ),

                    ///Badge level or imageview
                    widget.flyyOffers.flyyOfferPrizes != null &&
                            widget.flyyOffers.flyyOfferPrizes.length > 0
                        ? Padding(
                            padding: const EdgeInsets.all(10),
                            child: Column(
                              children: <Widget>[
                                Padding(
                                  padding: const EdgeInsets.only(bottom: 15),
                                  child: Text(
                                    'Badge Level',
                                    style: new TextStyle(
                                        fontSize: 16,
                                        color:
                                            FlyyUtility.parseColor("#373535")),
                                  ),
                                ),
                                Padding(
                                  padding: const EdgeInsets.only(
                                      left: 10, right: 10),
                                  child: ListView.separated(
                                    separatorBuilder: (context, index) =>
                                        Divider(
                                      color: FlyyUtility.parseColor("#D3D3D3"),
                                    ),
                                    itemBuilder: setOfferPrizesList,
                                    itemCount: widget
                                        .flyyOffers.flyyOfferPrizes.length,
                                    scrollDirection: Axis.vertical,
                                    shrinkWrap: true,
                                    physics:
                                        const NeverScrollableScrollPhysics(),
                                  ),
                                ),
                              ],
                            ))
                        : Container(
                            margin: const EdgeInsets.all(20),
                            child: widget.flyyOffers.imageUrl != null
                                ? Image.network(
                                    widget.flyyOffers.imageUrl,
                                    height: 200,
                                    width: 400,
                                  )
                                : Container(
                                    height: 200.0,
                                    width: 400.0,
                                    color: Colors.blueGrey,
                                  ),
                          ),

                    ///Divider
                    widget.flyyOffers.flyyOfferPrizes != null &&
                            widget.flyyOffers.flyyOfferPrizes.length > 0
                        ? Container(
                            width: double.infinity,
                            height: 1,
                            margin: const EdgeInsets.only(
                                left: 20, right: 20, bottom: 10),
                            child: Divider(
                              color: FlyyUtility.parseColor("#D3D3D3"),
                            ),
                          )
                        : SizedBox.shrink(),

                    ///campaign ends at
                    Container(
                      alignment: Alignment.centerRight,
                      margin:
                          const EdgeInsets.only(right: 10, bottom: 10, top: 10),
                      child: Text(
                        'Campaign ends on ' +
                            FlyyUtility.getFormattedDateDDMMMYYY(
                                widget.flyyOffers.endsAt),
                        style: new TextStyle(
                            fontSize: 10,
                            color: FlyyUtility.parseColor("#373535")),
                      ),
                    ),

                    ///Offers terms and conditions
                    widget.flyyOffers.offerTerms != null &&
                            widget.flyyOffers.offerTerms.length > 0
                        ? Container(
                            margin: const EdgeInsets.only(top: 20, bottom: 20),
                            child: Column(
                              children: <Widget>[
                                Padding(
                                  padding: const EdgeInsets.only(bottom: 10),
                                  child: Text(
                                    'Offer Details',
                                    style: new TextStyle(
                                        fontSize: 16,
                                        color:
                                            FlyyUtility.parseColor("#373535")),
                                  ),
                                ),
                                Container(
                                    margin: const EdgeInsets.only(
                                        left: 10, right: 50),
//                                    padding: const EdgeInsets.only(
//                                        left: 20, right: 20),
                                    child: Html(
                                        data: widget.flyyOffers.offerTerms,
                                        defaultTextStyle:
                                            TextStyle(fontSize: 12))),
                              ],
                            ),
                          )
                        : SizedBox.shrink(),
                  ],
                ),
              ),
            ),
          ),
        ),
      ),
    );
  }

  Widget setOfferPrizesList(BuildContext context, int index) {
    FlyyOfferPrizes flyyOfferPrizes = widget.flyyOffers.flyyOfferPrizes[index];
    return Padding(
      padding: const EdgeInsets.only(left: 15, right: 15),
      child: Container(
        child: Row(
          children: <Widget>[
            ///Icon
            Padding(
                padding: const EdgeInsets.only(right: 10),
                child: setBadgeIcon(flyyOfferPrizes.badgeUrl)),

            ///Two texts title and description
            Expanded(
              child: Padding(
                padding: const EdgeInsets.only(left: 5),
                child: Container(
                  alignment: Alignment.centerLeft,
                  child: Column(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: <Widget>[
                      Text(
                        flyyOfferPrizes.label,
                        style: new TextStyle(
                            fontSize: 12,
                            color: FlyyUtility.parseColor("#373535")),
                      ),
                      Text(
                        'For Referral ' +
                            flyyUtility.checkAmount(flyyOfferPrizes.minRef) +
                            ' to ' +
                            flyyUtility.checkAmount(flyyOfferPrizes.maxRef),
                        style: new TextStyle(
                            fontSize: 10,
                            color: FlyyUtility.parseColor("#585957")),
                      )
                    ],
                  ),
                ),
              ),
            ),

            ///small box to display tokens and cash
            Column(
              children: <Widget>[
                Container(
                  child: Padding(
                    padding: const EdgeInsets.only(bottom: 5),
                    child: Row(
                      mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                      children: <Widget>[
                        widget.flyyOffers.currencyIconUrl != null
                            ? Image.network(
                                widget.flyyOffers.currencyIconUrl,
                                height: 10,
                                width: 10,
                              )
                            : SizedBox.shrink(),

                        ///Text
                        Padding(
                          padding: const EdgeInsets.only(left: 5.0),
                          child: Text(
                            widget.flyyOffers.currencyLabel != null &&
                                    widget.flyyOffers.currencyLabel.isNotEmpty
                                ? widget.flyyOffers.currencyLabel
                                : 'Scratch Card',
                            style: new TextStyle(
                                fontSize: 8,
                                color: FlyyUtility.parseColor("#373535")),
                          ),
                        )
                      ],
                    ),
                  ),
                ),
                Container(
                  height: 25,
                  width: 100,
                  decoration: BoxDecoration(
                    border: Border.all(
                      color: Flyy.TOOLBAR_COLOR,
                    ),
                    borderRadius: BorderRadius.circular(5.0),
                  ),
                  child: Padding(
                    padding: const EdgeInsets.all(5),
                    child: Center(
                      child: Text(
                        setOffersPrizeListText(context, flyyOfferPrizes,
                            widget.flyyOffers.currencyLabel),
                        textAlign: TextAlign.center,
                        style: new TextStyle(
                            fontSize: 10,
                            color: FlyyUtility.parseColor("#373535")),
                      ),
                    ),
                  ),
                ),
              ],
            )
          ],
        ),
      ),
    );
  }

  String setOffersPrizeListText(BuildContext context,
      FlyyOfferPrizes flyyOfferPrizes, String currencyLabel) {
    if (flyyOfferPrizes.minWin > 0) {
      if (currencyLabel == ('Cash') || currencyLabel == ('cash'))
        return flyyUtility.checkAmount(flyyOfferPrizes.minWin) +
            " to " +
            flyyUtility.checkAmount(flyyOfferPrizes.maxWin);
      else
        return flyyUtility.checkAmount(flyyOfferPrizes.minWin) +
            " to " +
            flyyUtility.checkAmount(flyyOfferPrizes.maxWin);
    } else {
      return "Upto " + flyyUtility.checkAmount(flyyOfferPrizes.maxWin);
    }

    /* if (flyyOfferPrizes.minWin > 0) {
      if (currencyLabel == ('Cash') || currencyLabel == ('Cash')) {
        return '₹' +
            flyyUtility.checkAmount(flyyOfferPrizes.minWin) +
            " to " +
            '₹' +
            flyyUtility.checkAmount(flyyOfferPrizes.maxWin);
      } else {
        if (currencyLabel == ('Cash') || currencyLabel == ('Cash')) {
          return '₹' + flyyUtility.checkAmount(flyyOfferPrizes.maxWin);
        } else {
          return flyyUtility.checkAmount(flyyOfferPrizes.maxWin) +
              ' ' +
              currencyLabel;
        }
      }
    } else {
      if (currencyLabel == ('Cash') || currencyLabel == ('Cash')) {
        return '₹' + flyyUtility.checkAmount(flyyOfferPrizes.maxWin);
      } else {
        return flyyUtility.checkAmount(flyyOfferPrizes.maxWin) +
            ' ' +
            currencyLabel;
      }
    }*/
  }

  String setRewardText(BuildContext context, FlyyOffers flyyOffers) {
    if (flyyOffers.flyyOfferPrizes != null &&
        flyyOffers.flyyOfferPrizes.length > 0) {
      int minWin = flyyOffers.flyyOfferPrizes.first.minWin;
      int maxWin = flyyOffers.flyyOfferPrizes.last.maxWin;
      if (flyyOffers.flyyOfferPrizes.first.minWin > 0) {
        if (flyyOffers.currencyLabel == ('cash') ||
            flyyOffers.currencyLabel == ('Cash')) {
          return flyyUtility.setRewardText(minWin, flyyOffers.currencyLabel) +
              " to " +
              flyyUtility.setRewardText(maxWin, flyyOffers.currencyLabel);
        } else {
          return minWin.toString() +
              " to " +
              flyyUtility.setRewardText(maxWin, flyyOffers.currencyLabel);
        }
      } else {
        return 'Upto ' +
            flyyUtility.setRewardText(maxWin, flyyOffers.currencyLabel);
      }
    } else {
      int minWin = flyyOffers.minReward;
      int maxWin = flyyOffers.maxReward;
      if (flyyOffers.minReward > 0) {
        if (flyyOffers.currencyLabel == ('cash') ||
            flyyOffers.currencyLabel == ('Cash')) {
          return flyyUtility.setRewardText(minWin, flyyOffers.currencyLabel) +
              " to " +
              flyyUtility.setRewardText(maxWin, flyyOffers.currencyLabel);
        } else {
          return flyyUtility.checkAmount(minWin) +
              " to " +
              flyyUtility.setRewardText(maxWin, flyyOffers.currencyLabel);
        }
      } else {
        return 'Upto ' +
            flyyUtility.setRewardText(maxWin, flyyOffers.currencyLabel);
      }
    }
  }

  ///fetch app information
  Future<FlyyShareResponse> fetchShareData(
      BuildContext context, int offerId, int shareType) async {
    showAlertDialog(context);
    APIClient apiClient = new APIClient();
    final response = await apiClient
        .getClient("get_share_link?offer_id=" + offerId.toString());
    if (response.statusCode == 200) {
      final responseJson = json.decode(response.body);
      FlyyShareResponse flyyShareResponse =
          FlyyShareResponse.fromJson(responseJson);

      Navigator.of(context).pop();

      if (flyyShareResponse.success) {
        switch (shareType) {
          case 1:
            await FlutterShareMe()
                .shareToWhatsApp(msg: flyyShareResponse.message);
            break;
          case 2:
            try {
              /* await FlutterShareMe().shareToFacebook(
                  url: flyyShareResponse.message,
                  msg: flyyShareResponse.message);*/
              await FlutterShareMe()
                  .shareToSystem(msg: flyyShareResponse.message);
            } on Exception catch (exception) {
              await FlutterShareMe()
                  .shareToSystem(msg: flyyShareResponse.message);
            } catch (error) {
              await FlutterShareMe()
                  .shareToSystem(msg: flyyShareResponse.message);
            }

            break;
          case 3:
            await FlutterShareMe()
                .shareToSystem(msg: flyyShareResponse.message);
            break;
          default:
            await FlutterShareMe()
                .shareToSystem(msg: flyyShareResponse.message);
            break;
        }
      } else {
        FlyyUtility.showToast(
            context, "Something went wrong", ToastGravity.BOTTOM);
      }

      return flyyShareResponse;
    } else {
      throw Exception('Failed to load data ' + response.toString());
    }
  }

  Widget showAlertDialog(BuildContext context) {
    showDialog(
      barrierDismissible: true,
      useRootNavigator: false,
      context: context,
      builder: (BuildContext context) {
        return AlertDialog(
          content: new Row(
            children: [
              CircularProgressIndicator(),
              Container(
                  margin: EdgeInsets.only(left: 5),
                  child: Text("Preparing to share...")),
            ],
          ),
        );
      },
    );
  }

  Widget setBadgeIcon(String badgeUrl) {
    if (badgeUrl != null && badgeUrl.isNotEmpty) {
      if (badgeUrl.contains("svg")) {
        return SvgPicture.network(
          badgeUrl,
          height: 40,
          width: 40,
        );
      } else {
        return Image.network(
          badgeUrl,
          height: 40,
          width: 40,
        );
      }
    } else {
      return SvgPicture.network(
        "https://ghc.theflyy.com/assets/fs_assets/ic_level.svg",
        height: 40,
        width: 40,
      );
    }
  }
}
