import 'dart:convert';
import 'dart:io';

import 'package:android_intent/android_intent.dart';
import 'package:app_settings/app_settings.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'package:flutter_dash/flutter_dash.dart';
import 'package:flutter_svg/flutter_svg.dart';
import 'package:flyyfluttersdk/Flyy.dart';
import 'package:flyyfluttersdk/helpers/APIClient.dart';
import 'package:flyyfluttersdk/helpers/FlyyCommonColors.dart';
import 'package:flyyfluttersdk/helpers/FlyyOffersAndRewardsBalanceList.dart';
import 'package:flyyfluttersdk/helpers/FlyyUtility.dart';
import 'package:flyyfluttersdk/helpers/NoDataFoundWidget.dart';
import 'package:flyyfluttersdk/helpers/ReusableAppbarWidget.dart';
import 'package:flyyfluttersdk/model/DailyOffers.dart';
import 'package:flyyfluttersdk/model/FlyyAppInfo.dart';
import 'package:flyyfluttersdk/model/FlyyCheckInDataGetResponse.dart';
import 'package:flyyfluttersdk/model/FlyyOffers.dart';
import 'package:flyyfluttersdk/model/FlyyStepBean.dart';
import 'package:flyyfluttersdk/model/FlyyTournament.dart';
import 'package:flyyfluttersdk/model/FlyyWalletModel.dart';
import 'package:flyyfluttersdk/views/FlyyCheckinDetailPage.dart';
import 'package:flyyfluttersdk/views/FlyyGameZonePage.dart';
import 'package:flyyfluttersdk/views/FlyyInviteAndEarnPage.dart';
import 'package:flyyfluttersdk/views/FlyyRewardsPage.dart';

///Stateless widget
class FlyyOffersPage extends StatelessWidget {
  FlyyAppInfo flyyAppInfo;
  FlyyUtility flyyUtility;

  @override
  Widget build(BuildContext context) {
    flyyUtility = new FlyyUtility();
    return Scaffold(body: makeNetworkCall(context));
//    return MaterialApp(home: getAppInfo());
//    return MaterialApp(home: makeNetworkCall(context));
  }

  Widget getAppInfo(BuildContext context) {
    return FutureBuilder<FlyyAppInfo>(
      future: flyyUtility.getFlyyAppInfoFromSp(),
      builder: (context, response) {
        if (response.hasData && response.data != null) {
          flyyAppInfo = response.data;
          if (flyyAppInfo != null &&
              flyyAppInfo.info != null &&
              flyyAppInfo.info.bgTheme != null) {
            Flyy.TOOLBAR_COLOR =
                FlyyUtility.parseColor(flyyAppInfo.info.bgTheme);
          }

          return Scaffold(
            appBar: ReusableAppbarWidget.getAppBar(context, 'Offers'),
            body: FlyyOffersStateFulWidget(flyyAppInfo: flyyAppInfo),
          );
        } else if (response.hasError) {
          return makeNetworkCall(context);
        }
        return Center(
            child: CircularProgressIndicator()); // or some other widget
      },
    );
  }

  ///fetch data
  Widget makeNetworkCall(BuildContext context) {
    Flyy flyy = new Flyy();
    return FutureBuilder<bool>(
      future: FlyyUtility.checkIfInterentPresent(),
      builder: (BuildContext context, AsyncSnapshot response) {
        if (response != null && response.data != null && response.data) {
          // Internet Present Case
          return FutureBuilder<FlyyAppInfo>(
            future: flyy.fetchAppInfo(),
            builder: (BuildContext context, AsyncSnapshot response) {
              if (response.connectionState == ConnectionState.waiting) {
                return Center(child: CircularProgressIndicator());
              } else {
                if (response.hasData && response.data != null) {
                  flyyAppInfo = response.data;
                  if (flyyAppInfo != null &&
                      flyyAppInfo.info != null &&
                      flyyAppInfo.info.bgTheme != null) {
                    Flyy.TOOLBAR_COLOR =
                        FlyyUtility.parseColor(flyyAppInfo.info.bgTheme);
                  }

                  return Scaffold(
                    appBar: ReusableAppbarWidget.getAppBar(context, 'Offers'),
                    body: FlyyOffersStateFulWidget(flyyAppInfo: flyyAppInfo),
                  );
                } else if (response.hasError) {
                  print(response.error);
                }
              }
              return Center(child: CircularProgressIndicator());
            },
          );
        } else {
          // No-Internet Case
          return noInternetLayout(context);
        }
      },
    );
  }

  ///no internet layout
  Widget noInternetLayout(BuildContext context) {
    return Container(
      margin: const EdgeInsets.all(20),
      child: Column(
        mainAxisAlignment: MainAxisAlignment.center,
        children: <Widget>[
          Image.network(
            "https://ghc.theflyy.com/assets/fs_assets/ic_no_internet.png",
            height: 80,
            width: 80,
          ),
          Padding(
            padding: const EdgeInsets.only(top: 10, bottom: 10),
            child: Text(
              "Couldn't connect to the internet. Please check your network.",
              textAlign: TextAlign.center,
              style: TextStyle(
                  fontSize: 15, color: FlyyUtility.parseColor('#575555')),
            ),
          ),
          Row(
            mainAxisAlignment: MainAxisAlignment.spaceEvenly,
            children: <Widget>[
              GestureDetector(
                onTap: () {
//                  makeNetworkCall(context);
                },
                child: Row(
                  children: <Widget>[
                    Icon(
                      Icons.refresh,
                      size: 30,
                      color: FlyyUtility.parseColor('#717171'),
                    ),
                    Padding(
                      padding: const EdgeInsets.only(left: 5),
                      child: Text(
                        'Tap To Retry',
                        style: TextStyle(
                            fontSize: 14,
                            color: FlyyUtility.parseColor('#717171')),
                      ),
                    )
                  ],
                ),
              ),
              GestureDetector(
                onTap: () {
                  AppSettings.openWIFISettings();
                },
                child: Row(
                  children: <Widget>[
                    Image.network(
                      "https://ghc.theflyy.com/assets/fs_assets/ic_settings.png",
                      height: 30,
                      width: 30,
                    ),
                    Padding(
                      padding: const EdgeInsets.only(left: 5),
                      child: Text(
                        'Go To Settings',
                        style: TextStyle(
                            fontSize: 14,
                            color: FlyyUtility.parseColor('#717171')),
                      ),
                    )
                  ],
                ),
              ),
            ],
          )
        ],
      ),
    );
  }
}

///Stateful widget
class FlyyOffersStateFulWidget extends StatefulWidget {
  FlyyAppInfo flyyAppInfo;

  FlyyOffersStateFulWidget({Key key, @required this.flyyAppInfo})
      : super(key: key);

  @override
  State<StatefulWidget> createState() {
    return new FlyyOffersState();
  }
}

///State class
class FlyyOffersState extends State<FlyyOffersStateFulWidget> {
  FlyyAppInfo flyyAppInfo;
  FlyyTournament flyyTournament;
  GlobalKey key = new GlobalKey();
  FlyyUtility flyyUtility;
  Flyy flyy;

  @override
  void initState() {
    super.initState();
    flyyUtility = new FlyyUtility();
    flyy = new Flyy();
    flyy.fetchAppInfo().then((data) {
      setState(() {
        if (data != null) flyyAppInfo = data;
      });
    });

    /* FlyyUtility.getFlyyAppInfoFromSp().then((data) {
      setState(() {
        if (data != null) flyyAppInfo = data;
      });
    });*/
  }

  ///Call fetch info and load UI when response is success.
  @override
  Widget build(BuildContext context) {
    flyyAppInfo = widget.flyyAppInfo;
    return setOffersPageUI(context, flyyAppInfo);
  }

  ///Set up offers page when response is returned
  Widget setOffersPageUI(BuildContext context, FlyyAppInfo flyyAppInfo) {
    NoDataFoundWidget noDataFoundWidget = new NoDataFoundWidget();
    return Scaffold(
      body: Container(
        color: FlyyCommonColors.BG_GREY,
        height: double.maxFinite,
        child: SingleChildScrollView(
          child: Column(
            children: <Widget>[
              setUserNameLayout(context),

              ///Container holding the bottom content
              flyyAppInfo.info.flyyOfferTypes != null &&
                      flyyAppInfo.info.flyyOfferTypes.length > 0
                  ? Container(
                      margin: const EdgeInsets.all(5),
                      child: flyyAppInfo.info.flyyOfferTypes != null &&
                              flyyAppInfo.info.flyyOfferTypes.length > 0
                          ? Container(
                              child: ListView.builder(
                                physics: const NeverScrollableScrollPhysics(),
                                itemBuilder: setOfferTypesCardView,
                                itemCount:
                                    flyyAppInfo.info.flyyOfferTypes.length,
                                scrollDirection: Axis.vertical,
                                shrinkWrap: true,
                              ),
                            )
                          : noDataFoundWidget
                              .showNoDataLayout("No offers created yet."),
                    )
                  : Container(
                      height: 400,
                      child: noDataFoundWidget
                          .showNoDataLayout("No offers created yet."),
                    ),
            ],
          ),
        ),
      ),
    );
  }

  ///Ui to display various cards as per offer type
  Widget setOfferTypesCardView(BuildContext context, int index) {
    FlyyOffers flyyOffers = flyyAppInfo.info.flyyOfferTypes[index];
    String currentLevel, currentLevelIconUrl;
    for (int i = 0; i < flyyAppInfo.info.flyyOfferTypes.length; i++) {
      if (flyyAppInfo.info.flyyOfferTypes[i].currentLevel != null &&
          flyyAppInfo.info.flyyOfferTypes[i].currentLevel.isNotEmpty) {
        currentLevel = flyyAppInfo.info.flyyOfferTypes[index].currentLevel;
        currentLevelIconUrl =
            flyyAppInfo.info.flyyOfferTypes[index].currentIconUrl;
        break;
      }
    }
    flyyUtility.getBeanList(flyyOffers);
//    FlyyUtility.checkinBeanList = FlyyUtility.getBeanList(flyyOffers);
    if (index == 0 &&
        flyyOffers.bgTheme != null &&
        flyyOffers.bgTheme.isNotEmpty) {
      Flyy.TOOLBAR_COLOR = FlyyUtility.parseColor(flyyOffers.bgTheme);
    }
    if (flyyOffers.offerTypeCode.contains('checkin')) {
      return GestureDetector(
        onTap: () {
          Navigator.push(
            context,
            MaterialPageRoute(
                builder: (context) => FlyyCheckinDetailPage(
                      flyyOffers: flyyAppInfo.info.flyyOfferTypes[index],
                    )),
          ).then((value) {
            setState(() {
              loadCheckInList();
            });
          });
        },
        child: Container(
          ///checkin cardview
          child: Card(
            child: Column(
              children: <Widget>[
                ///Tokens layout icon and text
                Padding(
                  padding: const EdgeInsets.only(right: 10, top: 10),
                  child: Align(
                    alignment: Alignment.centerRight,
                    child: Container(
                      child: Column(
                        children: <Widget>[
                          flyyOffers.currencyIconUrl != null &&
                                  flyyOffers.currencyIconUrl.isNotEmpty
                              ? Image.network(
                                  flyyOffers.currencyIconUrl,
                                  height: 20,
                                  width: 20,
                                )
                              : SizedBox.shrink(),
                          Text(
                            flyyOffers.currencyLabel != null &&
                                    flyyOffers.currencyLabel.isNotEmpty
                                ? 'Earn ' + flyyOffers.currencyLabel
                                : '',
                            style: new TextStyle(
                                fontSize: 10,
                                color: FlyyUtility.parseColor("#373535")),
                          )
                        ],
                      ),
                    ),
                  ),
                ),

                ///Daily check in title
                Padding(
                    padding: const EdgeInsets.only(bottom: 10),
                    child: setCardIconAndTitle(
                        context,
                        flyyOffers.title != null && flyyOffers.title.isNotEmpty
                            ? FlyyUtility.cleanHtmlStrings(flyyOffers.title)
                            : 'Daily Check In',
                        "https://ghc.theflyy.com/assets/fs_assets/daily_checkin.svg")),

                ///Stepper checking view
                Padding(
                  padding: const EdgeInsets.all(10),
                  child: Text(
                    FlyyUtility.cleanHtmlStrings(flyyOffers.description),
                    style: new TextStyle(
                        fontSize: 12, color: FlyyUtility.parseColor("#585957")),
                  ),
                ),

                ///checkin listview
                Container(
                  margin: const EdgeInsets.only(top: 20, left: 20, right: 20),
                  constraints: new BoxConstraints(
                    minHeight: 30.0,
                    maxHeight: 50.0,
                  ),
                  child: loadCheckInList(),
                ),

                ///Checkin button
                GestureDetector(
                  onTap: () {
                    if (flyyOffers.checkedIn == null || !flyyOffers.checkedIn) {
                      setState(() {
                        FlyyUtility.checkInTitle = "Checking in...";
                      });
                      sendCheckInData(flyyOffers);
                    }
                  },
                  child: Container(
                    height: 45,
                    width: 100,
                    margin: const EdgeInsets.only(top: 20, bottom: 15),
                    child: Card(
                      color: Flyy.TOOLBAR_COLOR,
                      child: Center(
                        child: Text(
                          FlyyUtility.checkInTitle != null &&
                                  FlyyUtility.checkInTitle.isNotEmpty
                              ? FlyyUtility.checkInTitle
                              : flyyOffers.checkedIn == null ||
                                      !flyyOffers.checkedIn
                                  ? "Check In"
                                  : "Checked In",
                          style:
                              new TextStyle(fontSize: 14, color: Colors.white),
                        ),
                      ),
                    ),
                  ),
                ),
              ],
            ),
          ),
        ),
      );
    } else if (flyyOffers.offerTypeCode.contains('daily_offer')) {
      return GestureDetector(
        onTap: () {
          if (Platform.isAndroid) {
            final AndroidIntent intent = AndroidIntent(
                action: 'action_view',
                data: Uri.encodeFull(flyyOffers.deeplink));
            intent.launch();
          }
        },
        child: Container(
          child: Card(
            child: Column(
              children: <Widget>[
                Container(
                  margin: const EdgeInsets.only(
                      top: 20, right: 10, left: 10, bottom: 20),
                  child: Row(
                    children: <Widget>[
                      ///Icon
                      Padding(
                        padding: const EdgeInsets.only(right: 5),
                        child: flyyOffers.imageUrl != null &&
                                flyyOffers.imageUrl.isNotEmpty
                            ? flyyOffers.imageUrl.contains('svg')
                                ? SvgPicture.network(
                                    flyyOffers.imageUrl,
                                    height: 30,
                                    width: 30,
                                  )
                                : Image.network(
                                    flyyOffers.imageUrl,
                                    height: 30,
                                    width: 30,
                                  )
                            : Icon(
                                Icons.photo,
                                size: 30,
                              ),
                      ),

                      ///Two texts title and description
                      Expanded(
                        child: Container(
                          margin: const EdgeInsets.only(left: 10, right: 20),
                          alignment: Alignment.centerLeft,
                          child: Column(
                            crossAxisAlignment: CrossAxisAlignment.start,
                            children: <Widget>[
                              Text(
                                FlyyUtility.cleanHtmlStrings(flyyOffers.title),
                                style: new TextStyle(
                                    fontWeight: FontWeight.bold,
                                    fontSize: 12,
                                    color: FlyyUtility.parseColor("#373535")),
                              ),
                              Padding(
                                padding: const EdgeInsets.only(top: 2),
                                child: Text(
                                  FlyyUtility.cleanHtmlStrings(
                                      flyyOffers.description),
                                  style: new TextStyle(
                                      fontSize: 10,
                                      color: FlyyUtility.parseColor("#585957")),
                                ),
                              )
                            ],
                          ),
                        ),
                      ),

                      ///small box to display tokens and cash
                      Column(
                        children: <Widget>[
                          Container(
                            child: Padding(
                              padding: const EdgeInsets.only(bottom: 5),
                              child: Row(
                                mainAxisAlignment:
                                    MainAxisAlignment.spaceEvenly,
                                children: <Widget>[
                                  flyyOffers.currencyIconUrl != null
                                      ? Image.network(
                                          flyyOffers.currencyIconUrl,
                                          height: 10,
                                          width: 10,
                                        )
                                      : SizedBox.shrink(),

                                  ///Text
                                  Padding(
                                    padding: const EdgeInsets.only(left: 5.0),
                                    child: Text(
                                      flyyOffers.currencyLabel,
                                      style: new TextStyle(
                                          fontSize: 8,
                                          color: FlyyUtility.parseColor(
                                              "#373535")),
                                    ),
                                  )
                                ],
                              ),
                            ),
                          ),
                          Container(
                            height: 25,
                            width: 70,
                            decoration: BoxDecoration(
                              border: Border.all(
                                color: Flyy.TOOLBAR_COLOR,
                              ),
                              borderRadius: BorderRadius.circular(5.0),
                            ),
                            child: Padding(
                              padding: const EdgeInsets.all(5),
                              child: Center(
                                child: Text(
                                  setDailyOffersPrizeText(context, flyyOffers),
                                  textAlign: TextAlign.center,
                                  style: new TextStyle(
                                      fontSize: 8,
                                      color: FlyyUtility.parseColor("#373535")),
                                ),
                              ),
                            ),
                          ),
                        ],
                      )
                    ],
                  ),
                ),
              ],
            ),
          ),
        ),
      );
    } else if (flyyOffers.offerTypeCode.contains('game')) {
      return GestureDetector(
        onTap: () {
          Navigator.push(
            context,
            MaterialPageRoute(builder: (context) => FlyyGameZonePage()),
          );
        },
        child: Container(
          child: Card(
            child: Column(
              children: <Widget>[
                ///Tokens layout icon and text
                Padding(
                  padding: const EdgeInsets.only(right: 10, top: 10),
                  child: Align(
                    alignment: Alignment.centerRight,
                    child: Container(
                      child: Column(
                        children: <Widget>[
                          flyyOffers.currencyIconUrl != null &&
                                  flyyOffers.currencyIconUrl.isNotEmpty
                              ? Image.network(
                                  flyyOffers.currencyIconUrl,
                                  height: 20,
                                  width: 20,
                                )
                              : SizedBox.shrink(),
                          Text(
                            flyyOffers.currencyLabel != null &&
                                    flyyOffers.currencyLabel.isNotEmpty
                                ? 'Earn ' + flyyOffers.currencyLabel
                                : '',
                            style: new TextStyle(
                                fontSize: 10,
                                color: FlyyUtility.parseColor("#373535")),
                          )
                        ],
                      ),
                    ),
                  ),
                ),

                ///Game offer title
                Padding(
                    padding: const EdgeInsets.only(bottom: 15, top: 8),
                    child: setCardIconAndTitle(
                        context,
                        flyyOffers.title != null && flyyOffers.title.isNotEmpty
                            ? FlyyUtility.cleanHtmlStrings(flyyOffers.title)
                            : 'Game Zone',
                        "https://ghc.theflyy.com/assets/fs_assets/game_zone.svg")),

                ///Invite and earn description
                Center(
                    child: Padding(
                        padding: const EdgeInsets.only(left: 20, right: 20),
                        child: Text(
                          flyyOffers.description != null
                              ? FlyyUtility.cleanHtmlStrings(
                                  flyyOffers.description)
                              : '',
                          style: new TextStyle(
                              fontSize: 12,
                              color: FlyyUtility.parseColor("#585957")),
                          textAlign: TextAlign.center,
                        ))),

                ///games button
                GestureDetector(
                  onTap: () {
                    Navigator.push(
                      context,
                      MaterialPageRoute(
                          builder: (context) => FlyyGameZonePage()),
                    );
                  },
                  child: Container(
                    height: 45,
                    width: 100,
                    margin: const EdgeInsets.only(top: 10, bottom: 15),
                    child: Card(
                      color: Flyy.TOOLBAR_COLOR,
                      child: Center(
                        child: Text(
                          flyyOffers.buttonText != null &&
                                  flyyOffers.buttonText.isNotEmpty
                              ? flyyOffers.buttonText
                              : 'Play',
                          style:
                              new TextStyle(fontSize: 14, color: Colors.white),
                        ),
                      ),
                    ),
                  ),
                ),
              ],
            ),
          ),
        ),
      );
    } else {
      ///invite and earn offer type
      return GestureDetector(
        onTap: () {
          Navigator.push(
            context,
            MaterialPageRoute(
                builder: (context) => FlyyInviteAndEarnPage(
                      flyyOffers: flyyAppInfo.info.flyyOfferTypes[index],
                    )),
          );
        },
        child: Container(
          child: Card(
            child: Column(
              children: <Widget>[
                ///Tokens layout icon and text
                Padding(
                  padding: const EdgeInsets.only(right: 10, top: 10),
                  child: Align(
                    alignment: Alignment.centerRight,
                    child: Container(
                      child: Column(
                        children: <Widget>[
                          flyyOffers.currencyIconUrl != null &&
                                  flyyOffers.currencyIconUrl.isNotEmpty
                              ? Image.network(
                                  flyyOffers.currencyIconUrl,
                                  height: 20,
                                  width: 20,
                                )
                              : SizedBox.shrink(),
                          Text(
                            flyyOffers.currencyLabel != null &&
                                    flyyOffers.currencyLabel.isNotEmpty
                                ? 'Earn ' + flyyOffers.currencyLabel
                                : '',
                            style: new TextStyle(
                                fontSize: 10,
                                color: FlyyUtility.parseColor("#373535")),
                          )
                        ],
                      ),
                    ),
                  ),
                ),

                ///Invite and earn title
                Padding(
                    padding: const EdgeInsets.only(bottom: 15, top: 8),
                    child: setCardIconAndTitle(
                        context,
                        flyyOffers.title != null && flyyOffers.title.isNotEmpty
                            ? FlyyUtility.cleanHtmlStrings(flyyOffers.title)
                            : 'Invite & Earn',
                        "https://ghc.theflyy.com/assets/fs_assets/ic_invite.svg")),

                ///Invite and earn description
                Center(
                    child: Padding(
                        padding: const EdgeInsets.only(left: 20, right: 20),
                        /*child: Html(
                    data: flyyOffers.description,
                    style: {
                      "body": Style(
                        fontSize: FontSize(12.0),
                      ),
                    },
                  ),*/
                        child: Text(
                          flyyOffers.description != null
                              ? FlyyUtility.cleanHtmlStrings(
                                  flyyOffers.description)
                              : '',
                          style: new TextStyle(
                              fontSize: 12,
                              color: FlyyUtility.parseColor("#585957")),
                          textAlign: TextAlign.center,
                        ))),

                ///Rewards text
                Center(
                  child: flyyOffers.offerTypeCode.contains('gamification')
                      ? SizedBox.shrink()
                      : Container(
                          margin: const EdgeInsets.only(top: 20),
                          child: Row(
                            mainAxisAlignment: MainAxisAlignment.center,
                            children: <Widget>[
                              flyyOffers.currencyIconUrl != null &&
                                      flyyOffers.currencyIconUrl.isNotEmpty
                                  ? Padding(
                                      padding: const EdgeInsets.only(right: 5),
                                      child: Image.network(
                                        flyyOffers.currencyIconUrl,
                                        height: 20,
                                        width: 20,
                                      ),
                                    )
                                  : SizedBox.shrink(),
                              Text(
                                setRewardsForInviteAndEarnOfferType(
                                    context, flyyOffers),
                                style: new TextStyle(
                                    fontSize: 10,
                                    color: FlyyUtility.parseColor("#585957")),
                              )
                            ],
                          ),
                        ),
                ),

                ///Invite and earn button
                GestureDetector(
                  onTap: () {
                    Navigator.push(
                      context,
                      MaterialPageRoute(
                          builder: (context) => FlyyInviteAndEarnPage(
                                flyyOffers: flyyOffers,
                              )),
                    );
                  },
                  child: Container(
                    height: 45,
                    width: 100,
                    margin: const EdgeInsets.only(top: 10, bottom: 15),
                    child: Card(
                      color: Flyy.TOOLBAR_COLOR,
                      child: Center(
                        child: Text(
                          flyyOffers.buttonText != null &&
                                  flyyOffers.buttonText.isNotEmpty
                              ? flyyOffers.buttonText
                              : 'See More',
                          style:
                              new TextStyle(fontSize: 14, color: Colors.white),
                        ),
                      ),
                    ),
                  ),
                ),

                ///User level
                Center(
                  child: flyyOffers.offerTypeCode.contains('gamification')
                      ? Container(
                          margin: const EdgeInsets.only(bottom: 20),
                          child: Row(
                            mainAxisAlignment: MainAxisAlignment.center,
                            children: <Widget>[
                              currentLevelIconUrl != null &&
                                      currentLevelIconUrl.isNotEmpty
                                  ? Image.network(
                                      currentLevelIconUrl,
                                      height: 15,
                                      width: 15,
                                    )
                                  : SvgPicture.network(
                                      "https://ghc.theflyy.com/assets/fs_assets/ic_level.svg",
                                      height: 15,
                                      width: 15,
                                    ),
                              Padding(
                                padding: const EdgeInsets.only(left: 5),
                                child: Text(
                                  currentLevel != null
                                      ? 'Your Current Level : ' + currentLevel
                                      : flyyOffers.currentLevel != null &&
                                              flyyOffers.currentLevel.isNotEmpty
                                          ? 'Your Current Level: ' +
                                              flyyOffers.currentLevel
                                          : 'Your Current Level: NA',
                                  style: new TextStyle(
                                      fontSize: 10,
                                      color: FlyyUtility.parseColor("#585957")),
                                ),
                              ),
                            ],
                          ),
                        )
                      : SizedBox.shrink(),
                ),
              ],
            ),
          ),
        ),
      );
    }
  }

  ///Setup listview to display daily offers/challenges
  Widget setDailyOffersListView(BuildContext context, int index) {
    DailyOffers dailyOffers = flyyAppInfo.info.dailyOffers[index];
    return Container(
      height: 50,
      padding: const EdgeInsets.only(left: 15, right: 15),
      child: Row(
        children: <Widget>[
          ///Icon
          Padding(
            padding: const EdgeInsets.only(right: 10),
            child: dailyOffers.iconUrl != null && dailyOffers.iconUrl.isNotEmpty
                ? SvgPicture.network(
                    dailyOffers.iconUrl,
                    height: 30,
                    width: 30,
                  )
                : Icon(
                    Icons.photo,
                    size: 30,
                  ),
          ),

          ///Two texts title and description
          Expanded(
            child: Padding(
              padding: const EdgeInsets.only(left: 5, top: 10),
              child: Container(
                alignment: Alignment.centerLeft,
                child: Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children: <Widget>[
                    Text(
                      FlyyUtility.cleanHtmlStrings(dailyOffers.title),
                      style: new TextStyle(
                          fontSize: 14,
                          color: FlyyUtility.parseColor("#373535")),
                    ),
                    Text(
                      FlyyUtility.cleanHtmlStrings(dailyOffers.description),
                      style: new TextStyle(
                          fontSize: 12,
                          color: FlyyUtility.parseColor("#585957")),
                    )
                  ],
                ),
              ),
            ),
          ),

          ///small box to display tokens and cash
          dailyOffers.reward != null && dailyOffers.rewardType != null
              ? Container(
                  height: 30,
                  width: 100,
                  margin: const EdgeInsets.only(left: 10),
                  decoration: BoxDecoration(
                    border: Border.all(
                      color: Flyy.TOOLBAR_COLOR,
                    ),
                    borderRadius: BorderRadius.circular(5.0),
                  ),
                  child: Padding(
                    padding: const EdgeInsets.all(5),
                    child: Center(
                      child: Text(
                        dailyOffers.reward != null &&
                                dailyOffers.rewardType != null
                            ? flyyUtility.checkAmount(dailyOffers.reward) +
                                ' ' +
                                dailyOffers.rewardType
                            : '',
                        textAlign: TextAlign.center,
                        style: new TextStyle(
                            fontSize: 10,
                            color: FlyyUtility.parseColor("#373535")),
                      ),
                    ),
                  ),
                )
              : SizedBox.shrink()
        ],
      ),
    );
  }

  ///Common widget to display card title
  Widget setCardIconAndTitle(BuildContext context, String title, String icon) {
    return Column(
      mainAxisAlignment: MainAxisAlignment.center,
      children: <Widget>[
        icon != null && icon.isNotEmpty
            ? SvgPicture.network(
                icon,
                height: 20,
                width: 20,
                color: FlyyUtility.parseColor(flyyAppInfo.info.bgTheme),
              )
            : SizedBox.shrink(),
        Container(
          padding: const EdgeInsets.only(left: 10, top: 10),
          child: Text(
            title != null && title.isNotEmpty ? title : '',
            style: new TextStyle(
                fontSize: 14, color: FlyyUtility.parseColor("#373535")),
            overflow: TextOverflow.ellipsis,
          ),
        )
      ],
    );
  }

  String setRewardsForInviteAndEarnOfferType(
      BuildContext context, FlyyOffers flyyOffers) {
    int minWin = flyyOffers.minReward;
    int maxWin = flyyOffers.maxReward;
    if (minWin > 0) {
      if (flyyOffers.currencyLabel == ('cash') ||
          flyyOffers.currencyLabel == ('Cash')) {
        return flyyUtility.setRewardText(minWin, flyyOffers.currencyLabel) +
            " to " +
            flyyUtility.setRewardText(maxWin, flyyOffers.currencyLabel);
      } else {
        return flyyUtility.checkAmount(minWin) +
            " to " +
            flyyUtility.setRewardText(maxWin, flyyOffers.currencyLabel);
      }
    } else {
      return 'Upto ' +
          flyyUtility.setRewardText(maxWin, flyyOffers.currencyLabel);
    }
  }

  ///daily challenges prize text
  String setDailyOffersPrizeText(BuildContext context, FlyyOffers flyyOffers) {
    if (flyyOffers.minReward > 0) {
      return flyyUtility.checkAmount(flyyOffers.minReward) +
          " to " +
          flyyUtility.checkAmount(flyyOffers.maxReward);
    } else {
      return "Upto " + flyyUtility.checkAmount(flyyOffers.maxReward);
    }
  }

  ///check in horizontal listview
  Widget setCheckinListView(BuildContext context, int index) {
    print("POOJA " + FlyyUtility.checkinBeanList.length.toString());
    FlyyStepBean flyyStepBean = FlyyUtility.checkinBeanList[index];
    return Container(
      child: GestureDetector(
        onTap: () {
          showPopUp();
        },
        child: Row(
          children: <Widget>[
            Column(
              children: <Widget>[
                setImageAsPerCheckinState(flyyStepBean),
                Padding(
                  padding: const EdgeInsets.only(top: 5),
                  child: Text(
                    flyyStepBean.name,
                    style: TextStyle(
                        fontSize: 12, fontWeight: flyyStepBean.fontWeight),
                  ),
                )
              ],
            ),
            index < FlyyUtility.checkinBeanList.length - 1
                ? addLineOrDash()
                : SizedBox.shrink(),
          ],
        ),
      ),
    );
  }

  ///daily checkin icon
  Widget setImageAsPerCheckinState(FlyyStepBean flyyStepBean) {
    Color iconColor = null;
    if (flyyStepBean.iconDrawable.contains("ic_checkin_filled") ||
        flyyStepBean.iconDrawable.contains("ic_checkin_filler")) {
      iconColor = Flyy.TOOLBAR_COLOR;
    }
    if (flyyStepBean.iconDrawable.contains("http") ||
        flyyStepBean.iconDrawable.contains("https")) {
      if (flyyStepBean.iconDrawable.contains(".png") ||
          flyyStepBean.iconDrawable.contains(".jpg")) {
        return Image.network(
          flyyStepBean.iconDrawable,
          height: 20,
          width: 20,
          color: iconColor,
        );
      } else if (flyyStepBean.iconDrawable.contains(".svg")) {
        return SvgPicture.network(
          flyyStepBean.iconDrawable,
          height: 20,
          width: 20,
          color: iconColor,
        );
      }
    }
    /*else if (flyyStepBean.iconDrawable.contains("svg")) {
      return SvgPicture.asset(
        flyyStepBean.iconDrawable,
        height: 20,
        width: 20,
      );
    } else {
      return Image.asset(
        flyyStepBean.iconDrawable,
        height: 20,
        width: 20,
        color: Flyy.TOOLBAR_COLOR,
      );
    }*/
  }

  ///Api to send checkin data
  Future<FlyyCheckInDataGetResponse> sendCheckInData(
      FlyyOffers flyyOffers) async {
    APIClient apiClient = new APIClient();
    final response = await apiClient.getClient("checkin");
    if (response.statusCode == 200) {
      final responseJson = json.decode(response.body);
      setState(() {
        if (response.statusCode == 200) {
          final responseJson = json.decode(response.body);
          FlyyCheckInDataGetResponse flyyCheckInResponse =
              FlyyCheckInDataGetResponse.fromJson(responseJson);
          flyyOffers.streak = flyyCheckInResponse.streak;
          flyyOffers.checkedIn = true;
          flyyOffers.buttonText = "Checked In";
          for (int i = 0; i < flyyAppInfo.info.flyyOfferTypes.length; i++) {
            if (flyyAppInfo.info.flyyOfferTypes[i].offerTypeCode ==
                flyyOffers.offerTypeCode) {
              flyyAppInfo.info.flyyOfferTypes[i] = flyyOffers;
              break;
            }
          }
          flyyUtility.getBeanList(flyyOffers);
          FlyyUtility.checkInTitle = "Checked In";
        }
      });

      return FlyyCheckInDataGetResponse.fromJson(responseJson);
    } else {
      setState(() {
        FlyyUtility.checkInTitle = "Check In";
      });
      throw Exception('Failed to load data ' + response.toString());
    }
  }

  ///Dotted or a dash in between daily checkin
  Widget addLineOrDash() {
    return Padding(
      padding: const EdgeInsets.only(bottom: 35),
      child: Dash(
          direction: Axis.horizontal,
          length: 30,
          dashLength: 5,
          dashColor: FlyyUtility.parseColor("585957")),
    );
  }

  ///First layout with username, level, cash, rewards, token.
  Widget setUserNameLayout(BuildContext context) {
    String currentLevel, currentLevelIconUrl;
    for (int i = 0; i < flyyAppInfo.info.flyyOfferTypes.length; i++) {
      if (flyyAppInfo.info.flyyOfferTypes[i].currentLevel != null &&
          flyyAppInfo.info.flyyOfferTypes[i].currentLevel.isNotEmpty) {
        currentLevel = flyyAppInfo.info.flyyOfferTypes[i].currentLevel;
        currentLevelIconUrl = flyyAppInfo.info.flyyOfferTypes[i].currentIconUrl;
        break;
      }
    }
    return Container(
      color: Flyy.TOOLBAR_COLOR,
      padding: const EdgeInsets.only(left: 20),
      child: Column(
        children: <Widget>[
          ///user name layout
          Align(
            alignment: Alignment.centerLeft,
            child: Text(
              flyyAppInfo.info.userName != null &&
                      flyyAppInfo.info.userName.isNotEmpty
                  ? 'Hello ' + FlyyUtility.titleCase(flyyAppInfo.info.userName)
                  : 'Hello User',
              style: TextStyle(color: Colors.white, fontSize: 20),
              textAlign: TextAlign.left,
            ),
          ),

          ///To show user level
          flyyAppInfo.info.currentLevel != null || currentLevel != null
              ? Container(
                  margin: const EdgeInsets.only(top: 5),
                  child: Align(
                    alignment: Alignment.centerLeft,
                    child: Row(
                      children: <Widget>[
                        currentLevelIconUrl != null &&
                                currentLevelIconUrl.isNotEmpty
                            ? Container(
                                height: 25,
                                width: 25,
                                decoration: new BoxDecoration(
                                  color: Colors.white,
                                  shape: BoxShape.circle,
                                ),
                                child: Center(
                                  child: Image.network(
                                    currentLevelIconUrl,
                                    height: 15,
                                    width: 15,
                                  ),
                                ),
                              )
                            : Container(
                                height: 25,
                                width: 25,
                                decoration: new BoxDecoration(
                                  color: Colors.white,
                                  shape: BoxShape.circle,
                                ),
                                child: Center(
                                  child: SvgPicture.network(
                                    "https://ghc.theflyy.com/assets/fs_assets/ic_level.svg",
                                    height: 15,
                                    width: 15,
                                  ),
                                ),
                              ),
                        Padding(
                          padding: const EdgeInsets.only(left: 5),
                          child: Text(
                            flyyAppInfo.info.currentLevel != null &&
                                    flyyAppInfo.info.currentLevel.isNotEmpty
                                ? flyyAppInfo.info.currentLevel
                                : currentLevel != null &&
                                        currentLevel.isNotEmpty
                                    ? currentLevel
                                    : 'NA',
                            style: TextStyle(color: Colors.white, fontSize: 14),
                            textAlign: TextAlign.left,
                          ),
                        ),
                      ],
                    ),
                  ),
                )
              : SizedBox.shrink(),

          ///Row layout to show cash/tokens listview and rewards
          Container(
            margin: const EdgeInsets.only(top: 40),
            child: Row(
              mainAxisAlignment: MainAxisAlignment.spaceBetween,
              children: <Widget>[
                loadBalancesLayout(context),

                ///Rewards
                GestureDetector(
                  onTap: () {
                    Navigator.push(
                      context,
                      MaterialPageRoute(
                          builder: (context) => FlyyRewardsPage(
                                flyyAppInfo: flyyAppInfo,
                              )),
                    ).then((value) {
                      setState(() {
                        loadBalancesLayout(context);
                      });
                    });
                  },
                  child: Container(
                    margin:
                        const EdgeInsets.only(left: 20, right: 20, bottom: 25),
                    decoration: BoxDecoration(
                      border: Border.all(
                        color: Colors.white,
                      ),
                      borderRadius: BorderRadius.circular(20),
                    ),
                    child: Row(
                      children: <Widget>[
                        Padding(
                          padding: const EdgeInsets.only(left: 8),
                          child: SvgPicture.network(
                            "https://ghc.theflyy.com/assets/fs_assets/ic_reward.svg",
                            height: 15,
                            width: 15,
                          ),
                        ),
                        Padding(
                          padding: const EdgeInsets.all(8),
                          child: Text(
                            'Rewards',
                            style: new TextStyle(
                                fontSize: 14, color: Colors.white),
                          ),
                        )
                      ],
                    ),
                  ),
                )
              ],
            ),
          )
        ],
      ),
    );
  }

  void showPopUp() async {
    /*  await showMenu(
      context: context,
      position: RelativeRect.fromLTRB(100, 400, 100, 400),
      items: [
        PopupMenuItem(
          child: Text("View"),
        ),
      ],
      elevation: 8.0,
    );*/
  }

  Widget loadBalancesLayout(BuildContext context) {
    if (FlyyUtility.flyyWalletCompleteDataList != null &&
        FlyyUtility.flyyWalletCompleteDataList.length > 0) {
      return FlyyOffersAndRewardsBalanceList();
    } else {
      return FutureBuilder<FlyyWalletModel>(
          future: flyyUtility.fetchWalletData(),
          builder: (context, response) {
            if (response.hasData && response.data != null) {
              FlyyWalletModel flyyWalletModel = response.data;

              if (flyyWalletModel != null) {
                return FlyyOffersAndRewardsBalanceList();
              }
            } else {
              return SizedBox.shrink();
            }
            return SizedBox.shrink();
          });
    }
  }

  Widget loadCheckInList() {
    return ListView.builder(
      itemBuilder: setCheckinListView,
      itemCount: FlyyUtility.checkinBeanList.length,
      scrollDirection: Axis.horizontal,
      shrinkWrap: true,
    );
  }
}
