import 'dart:convert';

import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:flutter/services.dart';
import 'package:fluttertoast/fluttertoast.dart';
import 'package:flyyfluttersdk/Flyy.dart';
import 'package:flyyfluttersdk/helpers/APIClient.dart';
import 'package:flyyfluttersdk/helpers/FlyyCommonColors.dart';
import 'package:flyyfluttersdk/helpers/FlyyUtility.dart';
import 'package:flyyfluttersdk/helpers/ReusableAppbarWidget.dart';
import 'package:flyyfluttersdk/model/FlyyAccountDetails.dart';
import 'package:flyyfluttersdk/model/FlyyWalletCompleteData.dart';
import 'package:flyyfluttersdk/model/RedeemData.dart';

class FlyyRedeemPage extends StatelessWidget {
  FlyyWalletCompleteData flyyWalletCompleteData;

  FlyyRedeemPage({Key key, @required this.flyyWalletCompleteData})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: ReusableAppbarWidget.getAppBar(context, 'Redeem'),
      body: FlyyRedeemStateFulWidget(
          flyyWalletCompleteData: flyyWalletCompleteData),
    );
  }
}

class FlyyRedeemStateFulWidget extends StatefulWidget {
  FlyyWalletCompleteData flyyWalletCompleteData;

  FlyyRedeemStateFulWidget({Key key, @required this.flyyWalletCompleteData})
      : super(key: key);

  @override
  State<StatefulWidget> createState() {
    return new FlyyRedeemState();
  }
}

class FlyyRedeemState extends State<FlyyRedeemStateFulWidget> {
  FlyyUtility flyyUtility;
  int selectedRadio;
  bool isValidated;
  final accountNumber = TextEditingController(),
      ifsc = TextEditingController(),
      accountHoldersName = TextEditingController(),
      upiId = TextEditingController();
  static String upiBlockCharacterSet =
      "_-`!'#\$%&'()*+,‐./:;<=>?{|}~¡¢£¤¥¦§¨©ª«¬®ˉ°±²³´µ¶¸¹º»¼½¾¿ÀÁÂÃÄÅÆÇÈÉÊËÌÍÎÏÐÑÒÓÔÕÖ×ØÙÚÛÜÝÞßàáâãäåæçèéêëìíîïðñòóôõö÷øùúûüýþÿ \\\'\"";
  static String ifscBlockCharacterSet = "@" + upiBlockCharacterSet;
  final _formKey = GlobalKey<FormState>();

  @override
  void initState() {
    super.initState();
    flyyUtility = new FlyyUtility();
    FlyyWalletCompleteData flyyWalletCompleteData =
        widget.flyyWalletCompleteData;
    if (flyyWalletCompleteData.flyyAccountDetails != null) {
      if (flyyWalletCompleteData.flyyAccountDetails.acType == 'bank') {
        selectedRadio = 1;
      } else {
        selectedRadio = 2;
      }

      FlyyAccountDetails flyyAccountDetails =
          flyyWalletCompleteData.flyyAccountDetails;

      accountNumber.text =
          flyyAccountDetails.number != null ? flyyAccountDetails.number : '';
      accountNumber.selection = TextSelection.fromPosition(
        TextPosition(offset: accountNumber.text.length),
      );

      ifsc.text =
          flyyAccountDetails.number != null ? flyyAccountDetails.number : '';
      ifsc.selection = TextSelection.fromPosition(
        TextPosition(offset: ifsc.text.length),
      );

      accountHoldersName.text =
          flyyAccountDetails.number != null ? flyyAccountDetails.number : '';
      accountHoldersName.selection = TextSelection.fromPosition(
        TextPosition(offset: accountHoldersName.text.length),
      );

      upiId.text =
          flyyAccountDetails.number != null ? flyyAccountDetails.number : '';
      upiId.selection = TextSelection.fromPosition(
        TextPosition(offset: upiId.text.length),
      );
    } else {
      selectedRadio = 1;
    }
  }

  ///set selected radio value either 1 for bank or 2 for upi
  setSelectedRadio(int val) {
    setState(() {
      selectedRadio = val;
    });
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: Container(
        color: FlyyCommonColors.BG_GREY,
        height: double.maxFinite,
        child: SingleChildScrollView(
          child: Container(
            margin: const EdgeInsets.all(5),
            child: Card(
              child: Padding(
                padding: const EdgeInsets.only(top: 10),
                child: Column(
                  crossAxisAlignment: CrossAxisAlignment.center,
                  children: <Widget>[
                    Row(
                      mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                      children: <Widget>[
                        ///Bank radio button option
                        Row(
                          children: <Widget>[
                            Radio(
                              value: 1,
                              groupValue: selectedRadio,
                              onChanged: (val) {
                                setSelectedRadio(val);
                              },
                              activeColor: Flyy.TOOLBAR_COLOR,
                            ),
                            Text(
                              'BANK',
                              style: TextStyle(
                                  fontSize: 14,
                                  color: FlyyUtility.parseColor('#544e4e')),
                            ),
                          ],
                        ),

                        ///Upi radio button option
                        Row(
                          children: <Widget>[
                            Radio(
                              value: 2,
                              groupValue: selectedRadio,
                              onChanged: (val) {
                                setSelectedRadio(val);
                              },
                              activeColor: Flyy.TOOLBAR_COLOR,
                            ),
                            Text(
                              'UPI',
                              style: TextStyle(
                                  fontSize: 14,
                                  color: FlyyUtility.parseColor('#544e4e')),
                            ),
                          ],
                        ),
                      ],
                    ),

                    ///divider
                    Padding(
                      padding: const EdgeInsets.only(
                          left: 20, right: 20, top: 10, bottom: 10),
                      child: Divider(
                        height: 1,
                        color: FlyyUtility.parseColor('#D3D3D3'),
                      ),
                    ),

                    ///Bank and upi layouts
                    setBankAndUpiDetailsUI(widget.flyyWalletCompleteData)
                  ],
                ),
              ),
            ),
          ),
        ),
      ),
    );
  }

  Widget setBankAndUpiDetailsUI(FlyyWalletCompleteData flyyWalletCompleteData) {
    return Container(
      margin: const EdgeInsets.only(left: 30, right: 30),
      child: Form(
        key: _formKey,
        child: Column(
          children: <Widget>[
            selectedRadio == 1
                ? Column(
                    children: <Widget>[
                      ///Account number
                      TextFormField(
                        controller: accountNumber,
                        keyboardType: TextInputType.number,
                        textInputAction: TextInputAction.next,
                        decoration: InputDecoration(
                            hintText: 'A/C Number',
                            labelStyle: TextStyle(
                                fontSize: 14,
                                color: FlyyUtility.parseColor('#323232')),
                            hintStyle: TextStyle(
                                fontSize: 12,
                                color: FlyyUtility.parseColor('#9A9A9A'))),
                        style: TextStyle(fontSize: 14),
                        validator: (String arg) {
                          if (arg != null && arg.isNotEmpty) {
                            return null;
                          } else {
                            return 'Required';
                          }
                        },
                      ),

                      ///IFSC code
                      TextFormField(
//                        inputFormatters: [
//                          new WhitelistingTextInputFormatter(
//                              ifscBlockCharacterSet)
//                        ],
                        controller: ifsc,
                        keyboardType: TextInputType.text,
                        textInputAction: TextInputAction.next,
                        textCapitalization: TextCapitalization.characters,
                        decoration: InputDecoration(
                            hintText: 'IFSC',
                            labelStyle: TextStyle(
                                fontSize: 14,
                                color: FlyyUtility.parseColor('#323232')),
                            hintStyle: TextStyle(
                                fontSize: 12,
                                color: FlyyUtility.parseColor('#9A9A9A'))),
                        style: TextStyle(fontSize: 14),
                        validator: (String arg) {
                          if (arg != null && arg.isNotEmpty) {
                            return null;
                          } else {
                            return 'Required';
                          }
                        },
                      ),

                      ///Account holders name
                      TextFormField(
                        controller: accountHoldersName,
                        keyboardType: TextInputType.text,
                        textCapitalization: TextCapitalization.characters,
                        decoration: InputDecoration(
                            hintText: 'Account Holder’s Name',
                            labelStyle: TextStyle(
                                fontSize: 14,
                                color: FlyyUtility.parseColor('#323232')),
                            hintStyle: TextStyle(
                                fontSize: 12,
                                color: FlyyUtility.parseColor('#9A9A9A'))),
                        style: TextStyle(fontSize: 14),
                        validator: (String arg) {
                          if (arg != null && arg.isNotEmpty) {
                            return null;
                          } else {
                            return 'Required';
                          }
                        },
                      ),
                    ],
                  )
                :

                ///UPI
                TextFormField(
                    controller: upiId,
                    keyboardType: TextInputType.emailAddress,
//                    inputFormatters: [
//                      new WhitelistingTextInputFormatter(upiBlockCharacterSet)
//                    ],
                    decoration: InputDecoration(
                        hintText: 'Enter UPI ID',
                        labelStyle: TextStyle(
                            fontSize: 14,
                            color: FlyyUtility.parseColor('#323232')),
                        hintStyle: TextStyle(
                            fontSize: 12,
                            color: FlyyUtility.parseColor('#9A9A9A'))),
                    style: TextStyle(fontSize: 14),
                    validator: (String arg) {
                      if (arg != null && arg.isNotEmpty) {
                        return null;
                      } else {
                        return 'Required';
                      }
                    },
                  ),

            ///Will be transferred message
            Padding(
              padding: const EdgeInsets.only(top: 15),
              child: Text(
                selectedRadio == 1
                    ? flyyUtility.setRewardText(
                            flyyWalletCompleteData.balance, "Cash") +
                        ' will be transferred into above mention bank account'
                    : flyyUtility.setRewardText(
                            flyyWalletCompleteData.balance, "Cash") +
                        ' will be transferred into above mention UPI Id',
                style: TextStyle(
                    fontSize: 12, color: FlyyUtility.parseColor('#544e4e')),
              ),
            ),

            ///Withdraw button
            GestureDetector(
              onTap: () {
                if (selectedRadio == 1) {
                  if (accountNumber.text != null &&
                      accountNumber.text.isNotEmpty &&
                      accountHoldersName.text != null &&
                      accountHoldersName.text.isNotEmpty &&
                      ifsc.text != null &&
                      ifsc.text.isNotEmpty) {
                    FlyyUtility.checkIfInterentPresent().then((isConnected) {
                      if (isConnected != null && isConnected) {
                        // Internet Present Case
                        try {
                          sendDataToServer();
                        } catch (e) {
                          print(e);
                        }
                      } else {
                        // No-Internet Case
                        FlyyUtility.showToast(
                            context,
                            ("Please check your internet connection and try again."),
                            ToastGravity.BOTTOM);
                      }
                    });
//                    Scaffold.of(context).showSnackBar(
//                        SnackBar(content: Text('Processing Data')));

                  }
                } else {
                  if (upiId.text != null && upiId.text.isNotEmpty) {
                    FlyyUtility.checkIfInterentPresent().then((isConnected) {
                      if (isConnected != null && isConnected) {
                        // Internet Present Case
                        try {
                          sendDataToServer();
                        } catch (e) {
                          print(e);
                        }
                      } else {
                        // No-Internet Case
                        FlyyUtility.showToast(
                            context,
                            ("Please check your internet connection and try again."),
                            ToastGravity.BOTTOM);
                      }
                    });
                  }
                }
              },
              child: Container(
                height: 45,
                width: 100,
                margin: const EdgeInsets.only(top: 10, bottom: 15),
                child: Card(
                  color: Flyy.TOOLBAR_COLOR,
                  child: Center(
                    child: Text(
                      'WITHDRAW',
                      style: new TextStyle(fontSize: 14, color: Colors.white),
                    ),
                  ),
                ),
              ),
            ),
          ],
        ),
      ),
    );
  }

  Future<RedeemData> sendDataToServer() async {
    showAlertDialog(context);
    Map<String, String> bodyParams = new Map();
    bodyParams['ac_type'] = selectedRadio == 1 ? 'bank' : 'upi';
    bodyParams['number'] = accountNumber.text;
    bodyParams['ifsc'] = ifsc.text;
    bodyParams['name'] = accountHoldersName.text;
    bodyParams['upi_id'] = upiId.text;

    APIClient apiClient = new APIClient();

    final response =
        await apiClient.postClient("transfer_requests", bodyParams, null, null, false);
    if (response.statusCode == 200) {
      final responseJson = json.decode(response.body);
      RedeemData redeemData = RedeemData.fromJson(responseJson);

      if (redeemData.success) {
        flyyUtility.fetchWalletData();
      }

      FlyyUtility.showToast(context, redeemData.message, ToastGravity.CENTER);
      Navigator.of(context, rootNavigator: true).pop();
      Navigator.pop(context);
      return RedeemData.fromJson(responseJson);
    } else {
      Navigator.of(context, rootNavigator: true).pop();
      FlyyUtility.showToast(
          context, "Something went wrong", ToastGravity.CENTER);
      throw Exception('Failed to load data ' + response.toString());
    }
  }

  Widget showAlertDialog(BuildContext context) {
    showDialog(
      barrierDismissible: false,
      useRootNavigator: true,
      context: context,
      builder: (BuildContext context) {
        return AlertDialog(
          content: new Row(
            children: [
              CircularProgressIndicator(),
              Container(
                  margin: EdgeInsets.only(left: 5),
                  child: Text("Processing, please wait...")),
            ],
          ),
        );
      },
    );
  }
}
