import 'dart:convert';

import 'package:app_settings/app_settings.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'package:flutter_svg/flutter_svg.dart';
import 'package:fluttertoast/fluttertoast.dart';
import 'package:flyyfluttersdk/Flyy.dart';
import 'package:flyyfluttersdk/helpers/APIClient.dart';
import 'package:flyyfluttersdk/helpers/FlyyCommonColors.dart';
import 'package:flyyfluttersdk/helpers/FlyyOffersAndRewardsBalanceList.dart';
import 'package:flyyfluttersdk/helpers/FlyyUtility.dart';
import 'package:flyyfluttersdk/helpers/ReusableAppbarWidget.dart';
import 'package:flyyfluttersdk/model/FlyyAppInfo.dart';
import 'package:flyyfluttersdk/model/FlyyReward.dart';
import 'package:flyyfluttersdk/model/FlyyRewardData.dart';
import 'package:flyyfluttersdk/model/FlyyScratchReward.dart';
import 'package:flyyfluttersdk/views/FlyyWalletPage.dart';
import 'package:scratcher/scratcher.dart';

///Stateless widget
class FlyyRewardsPage extends StatelessWidget {
  FlyyAppInfo flyyAppInfo;

  FlyyRewardsPage({Key key, @required this.flyyAppInfo}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: ReusableAppbarWidget.getAppBar(context, 'Rewards'),
      body: FlyyRewardsStateFulWidget(flyyAppInfo: flyyAppInfo),
    );
  }
}

///Stateful widget
class FlyyRewardsStateFulWidget extends StatefulWidget {
  FlyyAppInfo flyyAppInfo;

  FlyyRewardsStateFulWidget({Key key, @required this.flyyAppInfo})
      : super(key: key);

  @override
  State<StatefulWidget> createState() {
    return new FlyyRewardsState();
  }
}

///State class
class FlyyRewardsState extends State<FlyyRewardsStateFulWidget> {
  FlyyReward flyyReward;
  FlyyRewardData flyyRewardData;
  List<FlyyRewardData> rewardsList;
  FlyyAppInfo flyyAppInfo;
  int page = 0;
  bool isLoading = false;
  FlyyUtility flyyUtility;

  function(data) {
    setState(() {
      updateRewardsList(data);
    });
  }

  void removeItem(int index) {
    setState(() {
      rewardsList = List.from(rewardsList)..removeAt(index);
    });
  }

  ///Call fetch rewards and load UI when response is success.
  @override
  Widget build(BuildContext context) {
    flyyAppInfo = widget.flyyAppInfo;
    flyyUtility = new FlyyUtility();
    return loadUserAndWalletLayout(context);
  }

  ///First layout with username, level, cash, rewards, token.
  loadUserAndWalletLayout(BuildContext context) {
    String currentLevel, currentLevelIconUrl;
    for (int i = 0; i < flyyAppInfo.info.flyyOfferTypes.length; i++) {
      if (flyyAppInfo.info.flyyOfferTypes[i].currentLevel != null &&
          flyyAppInfo.info.flyyOfferTypes[i].currentLevel.isNotEmpty) {
        currentLevel = flyyAppInfo.info.flyyOfferTypes[i].currentLevel;
        currentLevelIconUrl = flyyAppInfo.info.flyyOfferTypes[i].currentIconUrl;
        break;
      }
    }
    return Container(
      height: double.maxFinite,
      color: FlyyCommonColors.BG_GREY,
      child: SingleChildScrollView(
        child: Column(
          children: <Widget>[
            setUserNameLayout(context),
            makeNetworkCall(context),
          ],
        ),
      ),
    );
  }

  ///First layout with username, level, cash, rewards, token.
  Widget setUserNameLayout(BuildContext context) {
    String currentLevel, currentLevelIconUrl;
    for (int i = 0; i < flyyAppInfo.info.flyyOfferTypes.length; i++) {
      if (flyyAppInfo.info.flyyOfferTypes[i].currentLevel != null &&
          flyyAppInfo.info.flyyOfferTypes[i].currentLevel.isNotEmpty) {
        currentLevel = flyyAppInfo.info.flyyOfferTypes[i].currentLevel;
        currentLevelIconUrl = flyyAppInfo.info.flyyOfferTypes[i].currentIconUrl;
        break;
      }
    }
    return Container(
      color: Flyy.TOOLBAR_COLOR,
      padding: const EdgeInsets.only(left: 20),
      child: Column(
        children: <Widget>[
          ///user name layout
          Align(
            alignment: Alignment.centerLeft,
            child: Text(
              flyyAppInfo.info.userName != null &&
                      flyyAppInfo.info.userName.isNotEmpty
                  ? 'Hello ' + FlyyUtility.titleCase(flyyAppInfo.info.userName)
                  : 'Hello User',
              style: TextStyle(color: Colors.white, fontSize: 20),
              textAlign: TextAlign.left,
            ),
          ),

          ///To show user level
          flyyAppInfo.info.currentLevel != null || currentLevel != null
              ? Container(
                  margin: const EdgeInsets.only(top: 5),
                  child: Align(
                    alignment: Alignment.centerLeft,
                    child: Row(
                      children: <Widget>[
                        currentLevelIconUrl != null &&
                                currentLevelIconUrl.isNotEmpty
                            ? Container(
                                height: 25,
                                width: 25,
                                decoration: new BoxDecoration(
                                  color: Colors.white,
                                  shape: BoxShape.circle,
                                ),
                                child: Center(
                                  child: Image.network(
                                    currentLevelIconUrl,
                                    height: 15,
                                    width: 15,
                                  ),
                                ),
                              )
                            : Container(
                                height: 25,
                                width: 25,
                                decoration: new BoxDecoration(
                                  color: Colors.white,
                                  shape: BoxShape.circle,
                                ),
                                child: Center(
                                  child: SvgPicture.network(
                                    "https://ghc.theflyy.com/assets/fs_assets/ic_level.svg",
                                    height: 15,
                                    width: 15,
                                  ),
                                ),
                              ),
                        Padding(
                          padding: const EdgeInsets.only(left: 5),
                          child: Text(
                            flyyAppInfo.info.currentLevel != null &&
                                    flyyAppInfo.info.currentLevel.isNotEmpty
                                ? flyyAppInfo.info.currentLevel
                                : currentLevel != null &&
                                        currentLevel.isNotEmpty
                                    ? currentLevel
                                    : 'NA',
                            style: TextStyle(color: Colors.white, fontSize: 14),
                            textAlign: TextAlign.left,
                          ),
                        ),
                      ],
                    ),
                  ),
                )
              : SizedBox.shrink(),

          ///Row layout to show cash/tokens listview and rewards
          Container(
            margin: const EdgeInsets.only(top: 40),
            child: Row(
              mainAxisAlignment: MainAxisAlignment.spaceBetween,
              children: <Widget>[
                FlyyOffersAndRewardsBalanceList(),
                //Rewards
                GestureDetector(
                  onTap: () {
                    Navigator.push(
                      context,
                      MaterialPageRoute(builder: (context) => FlyyWalletPage()),
                    ).then((value) {
                      setState(() {
                        setUserNameLayout(context);
                      });
                    });
                  },
                  child: Container(
                    margin:
                        const EdgeInsets.only(left: 20, right: 20, bottom: 25),
                    decoration: BoxDecoration(
                      border: Border.all(
                        color: Colors.white,
                      ),
                      borderRadius: BorderRadius.circular(20),
                    ),
                    child: Row(
                      children: <Widget>[
                        Padding(
                          padding: const EdgeInsets.only(left: 8),
                          child: SvgPicture.network(
                            "https://ghc.theflyy.com/assets/fs_assets/ic_wallet.svg",
                            height: 15,
                            width: 15,
                          ),
                        ),
                        Padding(
                          padding: const EdgeInsets.all(8),
                          child: Text(
                            'Wallet',
                            style: new TextStyle(
                                fontSize: 14, color: Colors.white),
                          ),
                        )
                      ],
                    ),
                  ),
                )
              ],
            ),
          )
        ],
      ),
    );
  }

  ///fetch data
  Widget makeNetworkCall(BuildContext context) {
    return FutureBuilder<bool>(
      future: FlyyUtility.checkIfInterentPresent(),
      builder: (BuildContext context, AsyncSnapshot response) {
        if (response != null && response.data != null && response.data) {
          // Internet Present Case
          return FutureBuilder<FlyyReward>(
            future: fetchRewards(),
            builder: (BuildContext context, AsyncSnapshot response) {
              if (response.connectionState == ConnectionState.waiting) {
                return Container(
                    height: 650,
                    color: Colors.white,
                    child: Center(
                        child: Container(child: CircularProgressIndicator())));
              } else {
                if (response.hasData && response.data != null) {
                  flyyReward = response.data;
                  if (rewardsList != null && rewardsList.length > 0) {
                    return setRewardsPageUI();
                  } else {
                    return setNoDataUI();
                  }
                } else if (response.hasError) {
                  return Center(child: Text("${response.error}"));
                }
              }
              return Container(
                  height: 500,
                  child: Center(child: CircularProgressIndicator()));
            },
          );
        } else {
          // No-Internet Case
          return noInternetLayout(context);
        }
      },
    );
  }

  Widget setRewardsPageUI() {
    return NotificationListener<ScrollNotification>(
        onNotification: (ScrollNotification scrollInfo) {
          if (page > 0 &&
              !isLoading &&
              rewardsList.length < flyyReward.totalCount &&
              scrollInfo.metrics.pixels == scrollInfo.metrics.maxScrollExtent) {
            fetchRewards();
            // start loading data
//            setState(() {
            isLoading = true;
//            });
          }
          return true;
        },
        child: SingleChildScrollView(
          child: loadGridView(context),
        ));
  }

  ///Container holding the bottom content
  Widget loadGridView(BuildContext context) {
    return Container(
      margin: const EdgeInsets.all(5),
      child: Card(
        child: Column(
          children: <Widget>[
            setGridViewList(context),
            Container(
              height: isLoading ? 50.0 : 0,
              color: Colors.transparent,
              child: Center(
                child: new CircularProgressIndicator(),
              ),
            ),
            seeOffersLayout(),
          ],
        ),
      ),
    );
  }

  Widget setGridViewUI(BuildContext context, int index) {
    FlyyRewardData flyyRewardData = rewardsList[index];
    if (flyyRewardData.isScratched) {
      ///Card scratched
      return Container(
        height: 100,
        width: 100,
        padding: const EdgeInsets.all(10),
        color: Colors.white,
        child: GestureDetector(
          onTap: () => openFullScreenDialog(context, flyyRewardData),
          child: Card(
            color: FlyyUtility.parseColor("#FAF7F7"),
            shape: RoundedRectangleBorder(
              borderRadius: BorderRadius.circular(10),
            ),
            child: Column(
              mainAxisAlignment: MainAxisAlignment.center,
              children: <Widget>[
                Image.network(
                  "https://ghc.theflyy.com/assets/fs_assets/ic_medal.png",
                  height: 30,
                  width: 30,
                ),
                Padding(
                  padding: const EdgeInsets.all(10),
                  child: Text(
                    flyyRewardData.offerValue == 0
                        ? 'Better luck next time'
                        : flyyUtility.setRewardText(flyyRewardData.offerValue,
                            flyyRewardData.rewardType),
                    style: TextStyle(
                        fontSize: 12, color: FlyyUtility.parseColor('#4a4a4a')),
                  ),
                )
              ],
            ),
          ),
        ),
      );
    } else if (flyyRewardData.isAvailableNow) {
      ///Can scratch the card now
      return GestureDetector(
        onTap: () => openFullScreenDialog(context, flyyRewardData),
        child: Container(
          height: 100,
          width: 100,
          margin: const EdgeInsets.all(10),
          decoration: new BoxDecoration(
            borderRadius: BorderRadius.circular(10),
            image: new DecorationImage(
              image: new NetworkImage(
                  "https://ghc.theflyy.com/assets/fs_assets/ic_rewards_scratch_bg.png"),
              colorFilter: ColorFilter.mode(
                  FlyyUtility.parseColor(widget.flyyAppInfo.info.scTheme),
                  BlendMode.overlay),
              fit: BoxFit.cover,
            ),
          ),
          child: Column(
            mainAxisAlignment: MainAxisAlignment.center,
            children: <Widget>[
              SvgPicture.network(
                "https://ghc.theflyy.com/assets/fs_assets/ic_rewards_scratch_icon.svg",
                height: 50,
                width: 50,
                color: FlyyUtility.parseColor(
                    widget.flyyAppInfo.info.scSecondaryTheme),
              ),
            ],
          ),
        ),
      );
    } else {
      ///Scratch card in future
      return Container(
        height: 100,
        width: 100,
        padding: const EdgeInsets.all(10),
        child: GestureDetector(
          onTap: () => openFullScreenDialog(context, flyyRewardData),
          child: Card(
            elevation: 2,
            shape: RoundedRectangleBorder(
              borderRadius: BorderRadius.circular(10),
            ),
            color: FlyyUtility.parseColor(flyyRewardData.bgColor),
            child: Column(
              children: <Widget>[
                Padding(
                  padding: const EdgeInsets.all(5),
                  child: Text(
                    FlyyUtility.getFormattedDateDDMMMYYY(
                        flyyRewardData.offerAvailableFrom),
                    style: TextStyle(fontSize: 12, color: Colors.white),
                  ),
                )
              ],
            ),
          ),
        ),
      );
    }
  }

  String setCardValue(FlyyRewardData flyyRewardData) {
    if (flyyRewardData.rewardType == null ||
        flyyRewardData.rewardType.isNotEmpty) {
      if (flyyRewardData.minPriceRange != null &&
          flyyRewardData.minPriceRange > 0) {
        if (FlyyUtility.equalsIgnoreCase(flyyRewardData.rewardType, 'Cash'))
          return flyyUtility.setRewardText(
                  flyyRewardData.minPriceRange, flyyRewardData.rewardType) +
              " to " +
              flyyUtility.setRewardText(
                  flyyRewardData.maxPriceRange, flyyRewardData.rewardType);
        else
          return flyyRewardData.minPriceRange.toString() +
              " to " +
              flyyUtility.setRewardText(
                  flyyRewardData.maxPriceRange, flyyRewardData.rewardType);
      } else {
        return "upto " +
            flyyUtility.setRewardText(
                flyyRewardData.maxPriceRange, flyyRewardData.rewardType);
      }
    }
  }

  Future<FlyyReward> fetchRewards() async {
    page++;
    APIClient apiClient = new APIClient();
    final response = await apiClient.getClient("user_offers?page=$page");
    if (response.statusCode == 200) {
      final responseJson = json.decode(response.body);
      flyyReward = FlyyReward.fromJson(responseJson);

      if (rewardsList == null) rewardsList = new List();
      if (flyyReward.rewardsList != null && flyyReward.rewardsList.length > 0)
        rewardsList.addAll(flyyReward.rewardsList);
      isLoading = false;

      return flyyReward;
    } else {
      throw Exception('Failed to load data for reward ' + response.toString());
    }
  }

  ///no internet layout
  Widget noInternetLayout(BuildContext context) {
    return Container(
      margin: const EdgeInsets.all(20),
      child: Column(
        mainAxisAlignment: MainAxisAlignment.center,
        children: <Widget>[
          Image.network(
            "https://ghc.theflyy.com/assets/fs_assets/ic_no_internet.png",
            height: 80,
            width: 80,
          ),
          Padding(
            padding: const EdgeInsets.only(top: 10, bottom: 10),
            child: Text(
              "Couldn't connect to the internet. Please check your network.",
              textAlign: TextAlign.center,
              style: TextStyle(
                  fontSize: 15, color: FlyyUtility.parseColor('#575555')),
            ),
          ),
          Row(
            mainAxisAlignment: MainAxisAlignment.spaceEvenly,
            children: <Widget>[
              GestureDetector(
                onTap: () {
                  makeNetworkCall(context);
                },
                child: Row(
                  children: <Widget>[
                    Icon(
                      Icons.refresh,
                      size: 30,
                      color: FlyyUtility.parseColor('#717171'),
                    ),
                    Padding(
                      padding: const EdgeInsets.only(left: 5),
                      child: Text(
                        'Tap To Retry',
                        style: TextStyle(
                            fontSize: 14,
                            color: FlyyUtility.parseColor('#717171')),
                      ),
                    )
                  ],
                ),
              ),
              GestureDetector(
                onTap: () {
                  AppSettings.openAppSettings();
                },
                child: Row(
                  children: <Widget>[
                    Image.network(
                      "https://ghc.theflyy.com/assets/fs_assets/ic_settings.png",
                      height: 30,
                      width: 30,
                    ),
                    Padding(
                      padding: const EdgeInsets.only(left: 5),
                      child: Text(
                        'Go To Settings',
                        style: TextStyle(
                            fontSize: 14,
                            color: FlyyUtility.parseColor('#717171')),
                      ),
                    )
                  ],
                ),
              ),
            ],
          )
        ],
      ),
    );
  }

  void openFullScreenDialog(
      BuildContext context, FlyyRewardData flyyRewardData) {
    Navigator.of(context)
        .push(OverlayDialog(
            flyyRewardData: flyyRewardData,
            func: function,
            flyyAppInfo: widget.flyyAppInfo))
        .then((rewardsData) {
//      setState(() {
//        if (rewardsData != null) {
//          updateRewardsList(rewardsData);
//        }
//      });
    });
  }

  Widget setGridViewList(BuildContext context) {
    return rewardsList != null && rewardsList.length > 0
        ? GridView.builder(
            physics: const NeverScrollableScrollPhysics(),
            itemBuilder: (context, position) {
              return setGridViewUI(context, position);
            },
            itemCount: rewardsList.length,
            gridDelegate:
                SliverGridDelegateWithFixedCrossAxisCount(crossAxisCount: 3),
            shrinkWrap: true,
          )
        : SizedBox.shrink();
  }

  ///see offers layout
  Widget seeOffersLayout() {
    return GestureDetector(
      onTap: () {
        Navigator.pop(context);
      },
      child: Center(
        child: Container(
          height: 45,
          width: 100,
          margin: const EdgeInsets.only(top: 25, bottom: 25),
          child: Card(
            color: Flyy.TOOLBAR_COLOR,
            child: Center(
              child: Text(
                'See Offers',
                style: new TextStyle(fontSize: 14, color: Colors.white),
              ),
            ),
          ),
        ),
      ),
    );
  }

  Widget setNoDataUI() {
    return Card(
      child: Container(
        height: 500,
        alignment: Alignment.center,
        margin: const EdgeInsets.all(20),
        child: Column(
          mainAxisAlignment: MainAxisAlignment.center,
          children: <Widget>[
            Image.network(
              "https://ghc.theflyy.com/assets/fs_assets/ic_broke.png",
              height: 100,
              width: 100,
            ),
            Text(
              'You haven\'t Earned any Rewards yet! Get started by clicking on the Offers.',
              textAlign: TextAlign.center,
              style: TextStyle(
                  fontSize: 16, color: FlyyUtility.parseColor('#323232')),
            ),
            Container(
                margin: const EdgeInsets.only(top: 20),
                child: seeOffersLayout()),
          ],
        ),
      ),
    );
  }

  void updateRewardsList(FlyyRewardData flyyRewardData) {
    if (flyyRewardData == null) return;
    if (rewardsList == null) rewardsList = new List();
    if (rewardsList != null) {
      for (int i = 0; i < rewardsList.length; i++) {
        if (rewardsList[i].ref == (flyyRewardData.ref)) {
          if (flyyRewardData.isScratched &&
              flyyRewardData.offerValue == 0 &&
              flyyRewardData.rewardType != "stamp") {
            removeItem(i);
          } else {
            flyyRewardData.isScratched = true;
            rewardsList[i] = flyyRewardData;
          }
          break;
        }
      }
    } else {
      rewardsList.add(flyyRewardData);
    }
  }
}

class OverlayDialog extends ModalRoute<FlyyRewardsPage> {
  FlyyRewardData flyyRewardData;
  FlyyAppInfo flyyAppInfo;
  final Function func;
  FlyyUtility flyyUtility = new FlyyUtility();

  OverlayDialog({Key key, this.flyyRewardData, this.func, this.flyyAppInfo});

  @override
  Duration get transitionDuration => Duration(milliseconds: 500);

  @override
  bool get opaque => false;

  @override
  bool get barrierDismissible => false;

  @override
  Color get barrierColor => Colors.black.withOpacity(0.7);

  @override
  String get barrierLabel => null;

  @override
  bool get maintainState => true;

  @override
  Widget buildPage(
    BuildContext context,
    Animation<double> animation,
    Animation<double> secondaryAnimation,
  ) {
    // This makes sure that text and other content follows the material style
    return WillPopScope(
      onWillPop: () async => false,
      child: SafeArea(
        child: buildOverlayContent(context),
      ),
    );
  }

  final scratchKey = GlobalKey<ScratcherState>();

  Widget buildOverlayContent(BuildContext context) {
    DateTime formattedTime = DateTime.parse(flyyRewardData.createdAt);
    return Container(
        padding: EdgeInsets.all(20),
        child: Center(
          child: Column(
            mainAxisSize: MainAxisSize.min,
            children: <Widget>[
              Text(
                flyyRewardData.offerDescription != null
                    ? flyyRewardData.offerDescription
                    : "",
                style: TextStyle(
                  color: Colors.white,
                  fontSize: 18,
                  decoration: TextDecoration.none,
                ),
                textAlign: TextAlign.center,
              ),
              Container(
                  margin: const EdgeInsets.only(
                      top: 40, left: 20, right: 20, bottom: 30),
                  child: loadCardViewAsPerConditions(context, flyyRewardData)),
              Text(
                "Ref Number: #" + flyyRewardData.ref,
                style: TextStyle(
                  color: Colors.white,
                  fontSize: 14,
                  decoration: TextDecoration.none,
                ),
                textAlign: TextAlign.center,
              ),
              Padding(
                padding: const EdgeInsets.only(top: 8.0),
                child: Text(
                  "Earned On: " +
                      FlyyUtility.getFormattedDateDDMMMYYY(
                          formattedTime.toLocal().toString()),
                  style: TextStyle(
                    color: Colors.white,
                    fontSize: 14,
                    decoration: TextDecoration.none,
                  ),
                  textAlign: TextAlign.center,
                ),
              ),
              GestureDetector(
                onTap: () {
                  Navigator.pop(context /*, flyyRewardData*/);
                },
                child: Container(
                    margin: EdgeInsets.only(top: 50),
                    child: Container(
                        padding: EdgeInsets.all(10),
                        width: 40,
                        height: 40,
                        decoration: new BoxDecoration(
                            shape: BoxShape.circle,
                            border: Border.all(
                                width: 1.5,
                                color: FlyyUtility.parseColor("#ffffff"))),
                        child: SvgPicture.network(
                          "https://ghc.theflyy.com/assets/fs_assets/ic_cross.svg",
                          width: 35,
                          height: 35,
                        ))),
              )
            ],
          ),
        ));
  }

  @override
  Widget buildTransitions(BuildContext context, Animation<double> animation,
      Animation<double> secondaryAnimation, Widget child) {
    // You can add your own animations for the overlay content
    return FadeTransition(
      opacity: animation,
      child: ScaleTransition(
        scale: animation,
        child: child,
      ),
    );
  }

  ///set card which is already scratched
  Widget setScratchedCardView(FlyyRewardData flyyRewardData) {
    return Container(
      height: 250,
      width: 250,
      color: Colors.transparent,
      child: Card(
        child: Column(
          mainAxisAlignment: MainAxisAlignment.center,
          children: <Widget>[
            Image.network(
              "https://ghc.theflyy.com/assets/fs_assets/ic_medal.png",
              height: 100,
              width: 100,
            ),
            Padding(
              padding: const EdgeInsets.only(top: 20),
              child: Text(
                flyyRewardData.offerValue == 0
                    ? 'Better luck next time'
                    : flyyUtility.setRewardText(
                        flyyRewardData.offerValue, flyyRewardData.rewardType),
                style: TextStyle(
                    fontSize: 20, color: FlyyUtility.parseColor('#4a4a4a')),
              ),
            )
          ],
        ),
      ),
    );
  }

  ///set card which will be scratched later
  Widget setScratchLaterCardView(FlyyRewardData flyyRewardData) {
    return Container(
      height: 250,
      width: 250,
      color: Colors.transparent,
      child: Card(
        child: Column(
          mainAxisAlignment: MainAxisAlignment.center,
          children: <Widget>[
            Padding(
              padding: const EdgeInsets.only(top: 20),
              child: Text(
                FlyyUtility.getFormattedDateDDMMMYYY(
                    flyyRewardData.offerAvailableFrom),
                style: TextStyle(
                    fontSize: 20, color: FlyyUtility.parseColor('#4a4a4a')),
              ),
            )
          ],
        ),
      ),
    );
  }

  ///set card to be scratched now
  Widget setScratchNowCard(
      BuildContext context, FlyyRewardData flyyRewardData) {
    return Container(
      margin: EdgeInsets.only(bottom: 30.0, top: 30),

      ///scratch area
      child: Scratcher(
        key: scratchKey,
        brushSize: 30,
        threshold: 50,
        accuracy: ScratchAccuracy.low,
        color: FlyyUtility.parseColor(flyyAppInfo.info.scTheme),
        onChange: (value) {
          print("Scratch progress: $value%");
        },
        onThreshold: () {
          scratchKey.currentState.reveal(duration: Duration(milliseconds: 500));
          updateUserRewardData(context, flyyRewardData);
          print("Threshold reached, you won!");
        },

        ///After scratch card is scratched
        child: setScratchedCardView(flyyRewardData),
      ),
    );
  }

  Widget loadCardViewAsPerConditions(
      BuildContext context, FlyyRewardData flyyRewardData) {
    if (flyyRewardData.isScratched) {
      return setScratchedCardView(flyyRewardData);
    } else if (flyyRewardData.isAvailableNow) {
      return setScratchNowCard(context, flyyRewardData);
    } else {
      return setScratchLaterCardView(flyyRewardData);
    }
  }

  Future<FlyyScratchReward> updateUserRewardData(
      BuildContext context, FlyyRewardData flyyRewardData) async {
    APIClient apiClient = new APIClient();
    final response = await apiClient.putClient(
        "scratch_user_offer/" + flyyRewardData.ref, null);
    if (response.statusCode == 200) {
      final responseJson = json.decode(response.body);
      FlyyScratchReward flyyScratchReward =
          FlyyScratchReward.fromJson(responseJson);

      flyyUtility.fetchWalletData();

//      func(flyyScratchReward.flyyRewardData);
      Navigator.pop(context, flyyScratchReward.flyyRewardData);

      return flyyScratchReward;
    } else {
      FlyyUtility.showToast(
          context, 'Something went wrong', ToastGravity.BOTTOM);
      throw Exception('Failed to load data ' + response.toString());
    }
  }
}
