import 'dart:convert';

import 'package:app_settings/app_settings.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'package:flyyfluttersdk/Flyy.dart';
import 'package:flyyfluttersdk/helpers/APIClient.dart';
import 'package:flyyfluttersdk/helpers/FlyyCommonColors.dart';
import 'package:flyyfluttersdk/helpers/FlyyUtility.dart';
import 'package:flyyfluttersdk/helpers/NoDataFoundWidget.dart';
import 'package:flyyfluttersdk/helpers/ReusableAppbarWidget.dart';
import 'package:flyyfluttersdk/model/FlyyLeaderboard.dart';
import 'package:flyyfluttersdk/model/FlyyTournamentData.dart';
import 'package:flyyfluttersdk/model/FlyyTournamentHistory.dart';
import 'package:flyyfluttersdk/views/FlyyGamesLeaderBoardDialog.dart';

///Stateless widget
class FlyyTournamentHistoryPage extends StatelessWidget {
  FlyyTournamentHistory flyyTournamentHistory;
  String prizeType;
  int page = 1;

  FlyyTournamentHistoryPage({Key key, this.prizeType}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    if (prizeType == null || prizeType.isEmpty) prizeType = "";
    return Scaffold(
      appBar: ReusableAppbarWidget.getAppBar(
          context,
          prizeType != null && prizeType.isNotEmpty
              ? prizeType + ' Tournament History'
              : 'Tournament History'),
      body: makeNetworkCall(context),
    );
  }

  ///fetch data
  Widget makeNetworkCall(BuildContext context) {
    NoDataFoundWidget noDataFoundWidget = new NoDataFoundWidget();
    return FutureBuilder<bool>(
      future: FlyyUtility.checkIfInterentPresent(),
      builder: (BuildContext context, AsyncSnapshot response) {
        if (response != null && response.data != null && response.data) {
          // Internet Present Case
          return FutureBuilder<FlyyTournamentHistory>(
            future: fetchTournamentHistory(page),
            builder: (BuildContext context, AsyncSnapshot response) {
              if (response.connectionState == ConnectionState.waiting) {
                return Container(
                    height: 500,
                    child: Center(child: CircularProgressIndicator()));
              } else {
                if (response.hasData && response.data != null) {
                  flyyTournamentHistory = response.data;

                  if (flyyTournamentHistory.success) {
                    return FlyyTournamentHistoryStateFulWidget(
                      flyyTournamentHistory: flyyTournamentHistory,
                      prizeType: prizeType,
                    );
                  } else {
                    return noDataFoundWidget.showNoDataLayout(
                        "No history found");
                  }
                } else if (response.hasError) {
                  return noDataFoundWidget.showNoDataLayout("No history found");
                  return Center(child: Text("${response.error}"));
                }
              }
              return Container(
                  height: 500,
                  child: Center(child: CircularProgressIndicator()));
            },
          );
        } else {
          // No-Internet Case
          return noInternetLayout(context);
        }
      },
    );
  }

  Future<FlyyTournamentHistory> fetchTournamentHistory(int page) async {
    APIClient apiClient = new APIClient();
    final response = await apiClient.getClient("v1/web_games_history?partner=" +
        Flyy.PARTNER_TOKEN +
        "&contact_number=" +
        Flyy.EXT_USER_ID +
        "&prize_type=" +
        prizeType +
        "&page=" +
        page.toString());
    if (response.statusCode == 200) {
      final responseJson = json.decode(response.body);

      return FlyyTournamentHistory.fromJson(responseJson);
    } else {
      throw Exception('Failed to load data ' + response.toString());
    }
  }

  ///no internet layout
  Widget noInternetLayout(BuildContext context) {
    return Container(
      margin: const EdgeInsets.all(20),
      child: Column(
        mainAxisAlignment: MainAxisAlignment.center,
        children: <Widget>[
          Image.network(
            "https://ghc.theflyy.com/assets/fs_assets/ic_no_internet.png",
            height: 80,
            width: 80,
          ),
          Padding(
            padding: const EdgeInsets.only(top: 10, bottom: 10),
            child: Text(
              "Couldn't connect to the internet. Please check your network.",
              textAlign: TextAlign.center,
              style: TextStyle(
                  fontSize: 15, color: FlyyUtility.parseColor('#575555')),
            ),
          ),
          Row(
            mainAxisAlignment: MainAxisAlignment.spaceEvenly,
            children: <Widget>[
              GestureDetector(
                onTap: () {
                  makeNetworkCall(context);
                },
                child: Row(
                  children: <Widget>[
                    Icon(
                      Icons.refresh,
                      size: 30,
                      color: FlyyUtility.parseColor('#717171'),
                    ),
                    Padding(
                      padding: const EdgeInsets.only(left: 5),
                      child: Text(
                        'Tap To Retry',
                        style: TextStyle(
                            fontSize: 14,
                            color: FlyyUtility.parseColor('#717171')),
                      ),
                    )
                  ],
                ),
              ),
              GestureDetector(
                onTap: () {
                  AppSettings.openAppSettings();
                },
                child: Row(
                  children: <Widget>[
                    Image.network(
                      "https://ghc.theflyy.com/assets/fs_assets/ic_settings.png",
                      height: 30,
                      width: 30,
                    ),
                    Padding(
                      padding: const EdgeInsets.only(left: 5),
                      child: Text(
                        'Go To Settings',
                        style: TextStyle(
                            fontSize: 14,
                            color: FlyyUtility.parseColor('#717171')),
                      ),
                    )
                  ],
                ),
              ),
            ],
          )
        ],
      ),
    );
  }
}

///Stateful widget
class FlyyTournamentHistoryStateFulWidget extends StatefulWidget {
  FlyyTournamentHistory flyyTournamentHistory;
  String prizeType;

  FlyyTournamentHistoryStateFulWidget(
      {Key key, @required this.flyyTournamentHistory, this.prizeType})
      : super(key: key);

  @override
  State<StatefulWidget> createState() {
    return new FlyyTournamentHistoryState();
  }
}

///State class
class FlyyTournamentHistoryState
    extends State<FlyyTournamentHistoryStateFulWidget> {
  FlyyTournamentHistory flyyTournamentHistory;
  List<FlyyTournamentData> liveList = new List();
  List<FlyyTournamentData> completedList = new List();
  int page = 1;
  bool isLoading = false;
  FlyyUtility flyyUtility;
  NoDataFoundWidget noDataFoundWidget;

  @override
  Widget build(BuildContext context) {
    flyyUtility = new FlyyUtility();
    noDataFoundWidget = new NoDataFoundWidget();
    flyyTournamentHistory = widget.flyyTournamentHistory;
    liveList = widget.flyyTournamentHistory.live;
    completedList = widget.flyyTournamentHistory.completed;
    if (liveList.length <= 0 && completedList.length <= 0)
      return noDataFoundWidget.showNoDataLayout("No history found");
    else
      return setTournamentsHistoryUI(context);
  }

  Widget setTournamentsHistoryUI(BuildContext context) {
    return Container(
      padding: const EdgeInsets.all(5),
      height: double.maxFinite,
      color: FlyyCommonColors.BG_GREY,
      child: NotificationListener<ScrollNotification>(
        onNotification: (ScrollNotification scrollInfo) {
          if (!isLoading &&
              scrollInfo.metrics.pixels == scrollInfo.metrics.maxScrollExtent) {
            page++;
            fetchTournamentHistory();
            isLoading = true;
          }
          return true;
        },
        child: SingleChildScrollView(
          child: Column(
            children: <Widget>[
              ///live history
              flyyTournamentHistory.live != null &&
                      flyyTournamentHistory.live.length > 0
                  ? Card(
                      shape: RoundedRectangleBorder(
                          borderRadius: BorderRadius.circular(5)),
                      child: Column(
                        children: <Widget>[
                          Container(
                            alignment: Alignment.center,
                            width: double.maxFinite,
                            margin: const EdgeInsets.only(bottom: 10),
                            padding: const EdgeInsets.only(top: 10, bottom: 10),
                            color: FlyyUtility.parseColor("#EEEEEE"),
                            child: Text("Live"),
                          ),
                          ListView.builder(
                            physics: NeverScrollableScrollPhysics(),
                            itemBuilder: setLiveListViewUI,
                            itemCount: liveList.length,
                            shrinkWrap: true,
                          ),
                        ],
                      ),
                    )
                  : SizedBox.shrink(),

              ///completed history
              flyyTournamentHistory.completed != null &&
                      flyyTournamentHistory.completed.length > 0
                  ? Card(
                      shape: RoundedRectangleBorder(
                          borderRadius: BorderRadius.circular(5)),
                      child: Column(
                        children: <Widget>[
                          Container(
                            width: double.maxFinite,
                            alignment: Alignment.center,
                            margin: const EdgeInsets.only(bottom: 10),
                            padding: const EdgeInsets.only(top: 10, bottom: 10),
                            color: FlyyUtility.parseColor("#EEEEEE"),
                            child: Text("Completed"),
                          ),
                          ListView.builder(
                            physics: NeverScrollableScrollPhysics(),
                            itemBuilder: setCompletedListViewUI,
                            itemCount: completedList.length,
                            shrinkWrap: true,
                          ),
                        ],
                      ),
                    )
                  : SizedBox.shrink(),
            ],
          ),
        ),
      ),
    );
  }

  Widget setLiveListViewUI(BuildContext context, int index) {
    FlyyTournamentData flyyTournamentData = flyyTournamentHistory.live[index];
    return setCommonListViewUI(context, flyyTournamentData, true);
  }

  Widget setCompletedListViewUI(BuildContext context, int index) {
    FlyyTournamentData flyyTournamentData =
        flyyTournamentHistory.completed[index];
    return setCommonListViewUI(context, flyyTournamentData, false);
  }

  Widget setCommonListViewUI(BuildContext context,
      FlyyTournamentData flyyTournamentData, bool isLive) {
    return Container(
      margin: const EdgeInsets.only(bottom: 15),
      child: Row(
        children: <Widget>[
          Container(
            margin: const EdgeInsets.only(left: 10, right: 10),
            decoration: BoxDecoration(
              borderRadius: BorderRadius.circular(5),
            ),
            child: Image.network(
              flyyTournamentData.gameThumbUrl,
              height: 80,
              width: 80,
            ),
          ),
          Row(
            children: <Widget>[
              Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                children: <Widget>[
                  Row(
                    mainAxisAlignment: MainAxisAlignment.spaceAround,
                    children: <Widget>[
                      Column(
                        crossAxisAlignment: CrossAxisAlignment.start,
                        children: <Widget>[
                          ///title
                          Text(
                            flyyTournamentData.title,
                            style: TextStyle(
                                fontSize: 14,
                                color: FlyyUtility.parseColor("#323232"),
                                fontWeight: FontWeight.bold),
                          ),

                          ///date and time starts at ends at
                          Row(
                            children: <Widget>[
                              Padding(
                                padding: const EdgeInsets.only(top: 5),
                                child: Row(
                                  children: <Widget>[
                                    Row(
                                      children: <Widget>[
                                        Text(
                                            FlyyUtility.getTimeFormat(
                                                flyyTournamentData.startsAt),
                                            style: TextStyle(
                                                fontSize: 12,
                                                color: FlyyUtility.parseColor(
                                                    "#323232"))),
                                        Padding(
                                          padding:
                                              const EdgeInsets.only(left: 5),
                                          child: Text(
                                              FlyyUtility.getDateFormat(
                                                  flyyTournamentData.startsAt),
                                              style: TextStyle(
                                                  fontSize: 12,
                                                  color: FlyyUtility.parseColor(
                                                      "#323232"))),
                                        ),
                                      ],
                                    ),
                                    Padding(
                                      padding: const EdgeInsets.only(
                                          right: 5, left: 5),
                                      child: Text(
                                        "-",
                                        style: TextStyle(
                                            color: FlyyUtility.parseColor(
                                                "#323232")),
                                      ),
                                    ),
                                    Row(
                                      children: <Widget>[
                                        Text(
                                            FlyyUtility.getTimeFormat(
                                                flyyTournamentData.endsAt),
                                            style: TextStyle(
                                                fontSize: 12,
                                                color: FlyyUtility.parseColor(
                                                    "#323232"))),
                                        Padding(
                                          padding:
                                              const EdgeInsets.only(left: 5),
                                          child: Text(
                                              FlyyUtility.getDateFormat(
                                                  flyyTournamentData.endsAt),
                                              style: TextStyle(
                                                  fontSize: 12,
                                                  color: FlyyUtility.parseColor(
                                                      "#323232"))),
                                        ),
                                      ],
                                    ),
                                  ],
                                ),
                              ),
                            ],
                          ),

                          ///divider
                          Container(
                            width: 200,
                            child: Divider(
                              thickness: 1,
                              color: FlyyUtility.parseColor("#FFCCCCCC"),
                            ),
                          ),
                        ],
                      ),

                      ///live or completed won and tokens
                      Container(
                          margin: const EdgeInsets.only(bottom: 20),
                          child: setLiveOrCompletedWon(
                              context, flyyTournamentData, isLive)),
                    ],
                  ),

                  ///total prize, entry fee and rank
                  Row(
                    children: <Widget>[
                      Padding(
                        padding: const EdgeInsets.only(right: 10, bottom: 10),
                        child: Column(
                          children: <Widget>[
                            Text(
                              "Total Prize",
                              style: TextStyle(
                                  fontSize: 12,
                                  color: FlyyUtility.parseColor("#323232"),
                                  fontWeight: FontWeight.bold),
                            ),
                            Padding(
                                padding: const EdgeInsets.only(top: 5),
                                child: Text(
                                  flyyUtility.setRewardText(
                                      flyyTournamentData.totalPrize,
                                      flyyTournamentData.prizeType),
                                  style: TextStyle(
                                      fontSize: 12,
                                      color: FlyyUtility.parseColor("#323232")),
                                )),
                          ],
                        ),
                      ),
                      Padding(
                        padding: const EdgeInsets.only(right: 10, bottom: 10),
                        child: Column(
                          children: <Widget>[
                            Text(
                              "Entry Fee",
                              style: TextStyle(
                                  fontSize: 12,
                                  color: FlyyUtility.parseColor("#323232"),
                                  fontWeight: FontWeight.bold),
                            ),
                            Padding(
                                padding: const EdgeInsets.only(top: 5),
                                child: Text(
                                  flyyTournamentData.entryFee == 0
                                      ? "FREE"
                                      : flyyUtility.setRewardText(
                                          flyyTournamentData.entryFee,
                                          flyyTournamentData.entryFeeType),
                                  style: TextStyle(
                                      fontSize: 12,
                                      color: FlyyUtility.parseColor("#323232")),
                                )),
                          ],
                        ),
                      ),
                      Padding(
                        padding: const EdgeInsets.only(right: 10, bottom: 10),
                        child: Column(
                          children: <Widget>[
                            Text(
                              "Your Rank",
                              style: TextStyle(
                                  fontSize: 12,
                                  color: FlyyUtility.parseColor("#323232"),
                                  fontWeight: FontWeight.bold),
                            ),
                            GestureDetector(
                              onTap: () {
                                List<FlyyLeaderboard>
                                leaderboardList = new List();
                                if (flyyTournamentData
                                    .leaderboardData !=
                                    null &&
                                    flyyTournamentData
                                        .leaderboardData
                                        .length >
                                        0)
                                  leaderboardList.addAll(
                                      flyyTournamentData
                                          .leaderboardData);
                                FlyyLeaderboard flyyLeaderBoard =
                                new FlyyLeaderboard(
                                    name: flyyTournamentData
                                        .yourName,
                                    mobile: flyyTournamentData
                                        .yourContactNumber,
                                    score: flyyTournamentData
                                        .yourScore !=
                                        null
                                        ? flyyTournamentData
                                        .yourScore
                                        .toDouble()
                                        : 0,
                                    rank: flyyTournamentData
                                        .yourRank,
                                    prize: flyyTournamentData
                                        .yourPrize,
                                    prizeType:
                                    flyyTournamentData
                                        .yourPrizeType);
                                leaderboardList.insert(
                                    0, flyyLeaderBoard);

                                showDialog(
                                    context: context,
                                    builder: (BuildContext context) =>
                                        FlyyGamesLeaderBoardDialog(
                                          flyyLeaderboardList: leaderboardList,
                                          flyyTournamentData:
                                              flyyTournamentData,
                                          shareText:
                                              flyyTournamentHistory.shareText,
                                        ));
                              },
                              child: Padding(
                                  padding: const EdgeInsets.only(top: 5),
                                  child: Text(
                                    "#" +
                                        flyyTournamentData.yourRank.toString(),
                                    style: TextStyle(
                                        fontSize: 12,
                                        decoration: TextDecoration.underline,
                                        color: FlyyUtility.parseColor(
                                            "#ff669900")),
                                  )),
                            ),
                          ],
                        ),
                      ),
                    ],
                  )
                ],
              ),
            ],
          ),
        ],
      ),
    );
  }

  Widget setLiveOrCompletedWon(BuildContext context,
      FlyyTournamentData flyyTournamentData, bool isLive) {
    if (isLive) {
      return Container(
        height: 30,
        width: 60,
        margin: const EdgeInsets.only(top: 10),
        child: Card(
          color: Flyy.TOOLBAR_COLOR,
          child: Center(
            child: Text(
              "LIVE",
              style: new TextStyle(fontSize: 12, color: Colors.white),
            ),
          ),
        ),
      );
    } else {
      return Container(
        child: Stack(alignment: Alignment.center, children: <Widget>[
          Image.network(
            "https://ghc.theflyy.com/assets/fs_assets/ticket_white.png",
            height: 30,
            width: 50,
          ),
          Column(
            children: <Widget>[
              Text(
                flyyTournamentData.yourPrize.toString(),
                textAlign: TextAlign.center,
                style: TextStyle(
                    color: FlyyUtility.parseColor("#323232"), fontSize: 12),
              ),
            ],
          )
        ]),
      );
    }
  }

  Future<FlyyTournamentHistory> fetchTournamentHistory() async {
    APIClient apiClient = new APIClient();
    final response = await apiClient.getClient("v1/web_games_history?partner=" +
        Flyy.PARTNER_TOKEN +
        "&contact_number=" +
        Flyy.EXT_USER_ID +
        "&prize_type=" +
        widget.prizeType +
        "&page=" +
        page.toString());
    if (response.statusCode == 200) {
      isLoading = false;

      final responseJson = json.decode(response.body);
      FlyyTournamentHistory flyyTournamentHistory =
          FlyyTournamentHistory.fromJson(responseJson);

      if (flyyTournamentHistory.success) {
        setState(() {
          if (flyyTournamentHistory.live != null &&
              flyyTournamentHistory.live.length > 0) {
            liveList.addAll(flyyTournamentHistory.live);
          }
          if (flyyTournamentHistory.completed != null &&
              flyyTournamentHistory.completed.length > 0) {
            completedList.addAll(flyyTournamentHistory.completed);
          }
        });
      }

      return flyyTournamentHistory;
    } else {
      throw Exception('Failed to load data ' + response.toString());
    }
  }
}
