import 'package:app_settings/app_settings.dart';
import 'package:flutter/material.dart';
import 'package:flutter_svg/flutter_svg.dart';
import 'package:flyyfluttersdk/Flyy.dart';
import 'package:flyyfluttersdk/helpers/FlyyCommonColors.dart';
import 'package:flyyfluttersdk/helpers/FlyyUtility.dart';
import 'package:flyyfluttersdk/helpers/NoDataFoundWidget.dart';
import 'package:flyyfluttersdk/helpers/ReusableAppbarWidget.dart';
import 'package:flyyfluttersdk/model/FlyyWalletCompleteData.dart';
import 'package:flyyfluttersdk/model/FlyyWalletModel.dart';
import 'package:flyyfluttersdk/views/FlyyGiftCardsPage.dart';
import 'package:flyyfluttersdk/views/FlyyRedeemPage.dart';
import 'package:flyyfluttersdk/views/FlyyTransactionsPage.dart';

class FlyyWalletPage extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: ReusableAppbarWidget.getAppBar(context, 'Wallet'),
      body: FlyyWalletStateFulWidget(),
    );
  }
}

class FlyyWalletStateFulWidget extends StatefulWidget {
  @override
  State<StatefulWidget> createState() {
    return new FlyyWalletState();
  }
}

class FlyyWalletState extends State<FlyyWalletStateFulWidget> {
  FlyyWalletModel flyyWalletModel;
  FlyyUtility flyyUtility;

  @override
  Widget build(BuildContext context) {
    flyyUtility = new FlyyUtility();
    return makeNetworkCall(context);
  }

  ///fetch data
  Widget makeNetworkCall(BuildContext context) {
    return FutureBuilder<bool>(
      future: FlyyUtility.checkIfInterentPresent(),
      builder: (BuildContext context, AsyncSnapshot response) {
        if (response != null && response.data != null && response.data) {
          // Internet Present Case
          return FutureBuilder<FlyyWalletModel>(
            future: flyyUtility.fetchWalletData(),
            builder: (BuildContext context, AsyncSnapshot response) {
              if (response.connectionState == ConnectionState.waiting) {
                return Center(child: CircularProgressIndicator());
              } else {
                if (response.hasData && response.data != null) {
                  flyyWalletModel = response.data;
                  return Scaffold(
                      body: Container(
                    height: double.maxFinite,
                    color: FlyyCommonColors.BG_GREY,
                    child: setWalletPageUI(context, flyyWalletModel),
                  ));
                } else if (response.hasError) {
                  return Center(child: Text("${response.error}"));
                }
              }
              return Center(child: CircularProgressIndicator());
            },
          );
        } else {
          // No-Internet Case
          return noInternetLayout(context);
        }
      },
    );
  }

  ///no internet layout
  Widget noInternetLayout(BuildContext context) {
    return Container(
      margin: const EdgeInsets.all(20),
      child: Column(
        mainAxisAlignment: MainAxisAlignment.center,
        children: <Widget>[
          Image.network(
            "https://ghc.theflyy.com/assets/fs_assets/ic_no_internet.png",
            height: 80,
            width: 80,
          ),
          Padding(
            padding: const EdgeInsets.only(top: 10, bottom: 10),
            child: Text(
              "Couldn't connect to the internet. Please check your network.",
              textAlign: TextAlign.center,
              style: TextStyle(
                  fontSize: 15, color: FlyyUtility.parseColor('#575555')),
            ),
          ),
          Row(
            mainAxisAlignment: MainAxisAlignment.spaceEvenly,
            children: <Widget>[
              GestureDetector(
                onTap: () {
                  makeNetworkCall(context);
                },
                child: Row(
                  children: <Widget>[
                    Icon(
                      Icons.refresh,
                      size: 30,
                      color: FlyyUtility.parseColor('#717171'),
                    ),
                    Padding(
                      padding: const EdgeInsets.only(left: 5),
                      child: Text(
                        'Tap To Retry',
                        style: TextStyle(
                            fontSize: 14,
                            color: FlyyUtility.parseColor('#717171')),
                      ),
                    )
                  ],
                ),
              ),
              GestureDetector(
                onTap: () {
                  AppSettings.openAppSettings();
                },
                child: Row(
                  children: <Widget>[
                    Image.network(
                      "https://ghc.theflyy.com/assets/fs_assets/ic_settings.png",
                      height: 30,
                      width: 30,
                    ),
                    Padding(
                      padding: const EdgeInsets.only(left: 5),
                      child: Text(
                        'Go To Settings',
                        style: TextStyle(
                            fontSize: 14,
                            color: FlyyUtility.parseColor('#717171')),
                      ),
                    )
                  ],
                ),
              ),
            ],
          )
        ],
      ),
    );
  }

  Widget setWalletPageUI(
      BuildContext context, FlyyWalletModel flyyWalletModel) {
    NoDataFoundWidget noDataFoundWidget = new NoDataFoundWidget();
    return FlyyUtility.flyyWalletCompleteDataList.length > 0
        ? Container(
            margin: const EdgeInsets.all(5),
            child: ListView.builder(
              itemBuilder: setWalletDataListView,
              itemCount: FlyyUtility.flyyWalletCompleteDataList.length,
              shrinkWrap: true,
            ),
          )
        : noDataFoundWidget.showNoDataLayout(
            "Your Wallet is Empty! Click on the Offers and start Earning");
  }

  Widget setWalletDataListView(BuildContext context, int index) {
    FlyyWalletCompleteData flyyWalletCompleteData =
        FlyyUtility.flyyWalletCompleteDataList[index];
    return Card(
      color: FlyyUtility.parseColor('#f5f6fa'),
      child: Column(
        children: <Widget>[
          ///balanace and redeem layout
          Row(
            mainAxisAlignment: MainAxisAlignment.spaceBetween,
            children: <Widget>[
              ///reward icon and balance layout
              Padding(
                padding: const EdgeInsets.all(8.0),
                child: Row(
                  children: <Widget>[
                    ///reward icon
                    flyyWalletCompleteData.rewardIcon != null &&
                            flyyWalletCompleteData.rewardIcon.isNotEmpty
                        ? Image.network(
                            flyyWalletCompleteData.rewardIcon,
                            height: 35,
                            width: 35,
                          )
                        : SizedBox.shrink(),

                    ///balances layout
                    Padding(
                      padding: const EdgeInsets.only(left: 5),
                      child: Column(
                        crossAxisAlignment: CrossAxisAlignment.start,
                        children: <Widget>[
                          ///balance static text
                          Text(
                            'Balance',
                            style: TextStyle(
                                fontSize: 12,
                                color: FlyyUtility.parseColor('#736c6c')),
                          ),

                          ///balance text dynamic
                          Text(
                            flyyUtility.setRewardText(
                                flyyWalletCompleteData.balance,
                                flyyWalletCompleteData.rewardType),
                            style: TextStyle(
                                fontSize: 16,
                                color: FlyyUtility.parseColor('#736c6c')),
                          ),
                        ],
                      ),
                    ),
                  ],
                ),
              ),

              ///Redeem button
              FlyyUtility.equalsIgnoreCase(
                          flyyWalletCompleteData.rewardType, 'Cash') ||
                      flyyWalletModel.activeCurrency != null &&
                          flyyWalletModel.activeCurrency.label ==
                              flyyWalletCompleteData.rewardType
                  ? GestureDetector(
                      onTap: () {
                        if (FlyyUtility.equalsIgnoreCase(
                            flyyWalletCompleteData.rewardType, 'Cash')) {
                          if (flyyWalletCompleteData.balance >=
                              flyyWalletCompleteData.minRedeemAmount) {
                            Navigator.push(
                              context,
                              MaterialPageRoute(
                                  builder: (context) => FlyyRedeemPage(
                                        flyyWalletCompleteData:
                                            flyyWalletCompleteData,
                                      )),
                            );
                          }
                        } else {
                          Navigator.push(
                            context,
                            MaterialPageRoute(
                                builder: (context) => FlyyGiftCardsPage(
                                      flyyWalletCompleteData:
                                          flyyWalletCompleteData,
                                      flyyWalletModel: flyyWalletModel,
                                    )),
                          );
                        }
                      },
                      child: setRedeemIconAsPerBalance(
                          flyyWalletCompleteData, flyyWalletModel),
                    )
                  : SizedBox.shrink()
            ],
          ),

          ///display wallet message if available
          flyyWalletCompleteData.walletMessage != null &&
                  flyyWalletCompleteData.walletMessage.isNotEmpty
              ? Container(
                  alignment: Alignment.centerLeft,
                  margin: const EdgeInsets.only(left: 20, top: 5),
                  child: Text(
                    flyyWalletCompleteData.walletMessage,
                    style: TextStyle(
                        fontSize: 12, color: FlyyUtility.parseColor('#736c6c')),
                  ),
                )
              : SizedBox.shrink(),

          ///total earned and redeemed layout
          Container(
            margin: const EdgeInsets.only(top: 10, bottom: 10),
            color: Colors.white,
            child: Column(
              children: <Widget>[
                ///total earned layout
                Padding(
                  padding: const EdgeInsets.all(10),
                  child: Row(
                    mainAxisAlignment: MainAxisAlignment.spaceBetween,
                    children: <Widget>[
                      Text(
                        'Total Earned',
                        style: TextStyle(
                            fontSize: 14,
                            color: FlyyUtility.parseColor('#736c6c')),
                      ),
                      Text(
                        flyyUtility.setRewardText(
                            flyyWalletCompleteData.totalEarned,
                            flyyWalletCompleteData.rewardType),
                        style: TextStyle(
                            fontSize: 14,
                            color: FlyyUtility.parseColor('#736c6c')),
                      )
                    ],
                  ),
                ),

                ///Divider
                Container(
                  width: double.infinity,
                  height: 1,
                  margin: const EdgeInsets.only(
                      left: 20, right: 20, bottom: 5, top: 5),
                  child: Divider(
                    color: FlyyUtility.parseColor("#D3D3D3"),
                  ),
                ),

                ///Total redeemed layout
                Padding(
                  padding: const EdgeInsets.all(10),
                  child: Row(
                    mainAxisAlignment: MainAxisAlignment.spaceBetween,
                    children: <Widget>[
                      Text(
                        'Total Redeemed',
                        style: TextStyle(
                            fontSize: 14,
                            color: FlyyUtility.parseColor('#736c6c')),
                      ),
                      Text(
                        flyyUtility.setRewardText(
                            flyyWalletCompleteData.totalRedeemed,
                            flyyWalletCompleteData.rewardType),
                        style: TextStyle(
                            fontSize: 14,
                            color: FlyyUtility.parseColor('#736c6c')),
                      )
                    ],
                  ),
                )
              ],
            ),
          ),

          ///all transactions layout
          GestureDetector(
            onTap: () {
              Navigator.push(
                context,
                MaterialPageRoute(
                    builder: (context) => FlyyTransactionsPage(
                          flyyWalletCompleteData: flyyWalletCompleteData,
                          flyyWalletModel: flyyWalletModel,
                          showLastBoughtGiftCardPopup: false,
                        )),
              );
            },
            child: Container(
              alignment: Alignment.bottomRight,
              margin: const EdgeInsets.only(right: 10, bottom: 10),
              child: Text(
                'All Transaction',
                style: TextStyle(
                  fontSize: 12,
                  color: FlyyUtility.parseColor('#736c6c'),
                  decoration: TextDecoration.underline,
                ),
              ),
            ),
          )
        ],
      ),
    );
  }

  Widget setRedeemIconAsPerBalance(
      FlyyWalletCompleteData flyyWalletCompleteData,
      FlyyWalletModel flyyWalletModel) {
    if ((flyyWalletModel.activeCurrency != null &&
        flyyWalletModel.activeCurrency.label ==
            flyyWalletCompleteData.rewardType)) {
      return commonRedeemUi(Flyy.TOOLBAR_COLOR);
    }

    if (FlyyUtility.equalsIgnoreCase(
        flyyWalletCompleteData.rewardType, 'Cash')) {
      if (flyyWalletCompleteData.balance >=
          flyyWalletCompleteData.minRedeemAmount) {
        return commonRedeemUi(Flyy.TOOLBAR_COLOR);
      } else {
        return commonRedeemUi(FlyyUtility.parseColor('#aaa'));
      }
    }
  }

  Widget commonRedeemUi(Color containerColor) {
    return Container(
      margin: const EdgeInsets.only(right: 10),
      alignment: Alignment.topRight,
      child: Card(
        color: containerColor,
        shape: RoundedRectangleBorder(
          borderRadius: BorderRadius.circular(20),
        ),
        child: Padding(
          padding: const EdgeInsets.all(10),
          child: Row(
            children: <Widget>[
              SvgPicture.network(
                "https://ghc.theflyy.com/assets/fs_assets/ic_redeem.svg",
                height: 15,
                width: 15,
              ),
              Padding(
                padding: const EdgeInsets.only(left: 5),
                child: Text(
                  'Redeem',
                  style: TextStyle(fontSize: 14, color: Colors.white),
                ),
              )
            ],
          ),
        ),
      ),
    );
  }
}
